#include <bits/stdc++.h>

using namespace std;
using ll = long long;

struct pt {
    ll x, y;
    int id = -1;

    ll crs(const pt &oth) const {
        return x * oth.y - y * oth.x;
    }

    ll dot(const pt &oth) const {
        return x * oth.x + y * oth.y;
    }

    friend pt operator-(const pt &a, const pt &b) {
        return pt{a.x - b.x, a.y - b.y};
    }

    bool operator<(const pt &oth) const {
        return x < oth.x;
    }
};

struct Fenwick {
    vector<pair<int, int>> t;
    int n;

    void init(int m) {
        n = m;
        t.assign(n + 1, {0, -1});
    }

    void update(pair<int, int> v, int x) {
        for (++x; x <= n; x += x & -x) {
            t[x] = max(t[x], v);
        }
    }

    pair<int, int> query(int x) {
        pair<int, int> ans = {0, -1};
        for (++x; x > 0; x -= x & -x) {
            ans = max(ans, t[x]);
        }
        return ans;
    }
};

int main() {
    ios::sync_with_stdio(false);
    cin.tie(nullptr);

    int task, group;
    cin >> task >> group;

    int n;
    cin >> n;

    vector<pt> p(n);
    for (int i = 0; i < n; ++i) {
        cin >> p[i].x >> p[i].y;
        p[i].id = i;
    }

    if (task == 1) {
        if (group == 1) {
            vector<int> ord(n);
            iota(ord.begin(), ord.end(), 0);
            sort(ord.begin(), ord.end(), [&](int i, int j) {
                return p[i].x < p[j].x;
            });
            vector<int> ans;
            for (int l = 0, r = n - 1, f = 0; l <= r; f ^= 1) {
                if (f == 0) {
                    ans.push_back(ord[l++]);
                } else {
                    ans.push_back(ord[r--]);
                }
            }
            for (int x: ans) {
                cout << x + 1 << " ";
            }
            cout << '\n';
        } else if (group == 2) {
            vector<int> ord(n);
            iota(ord.begin(), ord.end(), 0);
            vector<int> ans;
            int curAns = 0;
            do {
                int cur = 0;
                for (int i = 0; i + 2 < n; ++i) {
                    cur += (p[ord[i]] - p[ord[i + 1]]).dot(p[ord[i + 2]] - p[ord[i + 1]]) > 0;
                }
                if (cur > curAns) {
                    curAns = cur;
                    ans = ord;
                }
            } while (next_permutation(ord.begin(), ord.end()));
            for (int x : ans) {
                cout << x + 1 << " ";
            }
            cout << '\n';
        }
        return 0;
    }

    vector<int> shy(n), shx(n);

    auto choose = [&](vector<pt> u) -> pair<vector<pt>, vector<pt>> {
        int m = size(u);
        if (!is_sorted(u.begin(), u.end())) {
            sort(u.begin(), u.end());
        }
        vector<int> chosen(m);
        if (0) {
            vector<int> yy;
            for (int i = 0; i < m; ++i) {
                yy.push_back(u[i].y);
            }
            sort(yy.begin(), yy.end());
            for (int i = 0; i < m; ++i) {
                int j = u[i].id;
                shy[j] = std::lower_bound(yy.begin(), yy.end(), u[i].y) - yy.begin();
            }
            Fenwick lis, lds;
            lis.init(m), lds.init(m);
            vector<pair<int, int>> dp_lis(m), dp_lds(dp_lis);
            for (int i = 0; i < m; ++i) {
                int id = u[i].id;
                int y = shy[id];
                assert(y < m);
                assert(y >= 0);
                auto is = lis.query(y);
                auto ds = lds.query(m - y - 1);
                dp_lis[i] = is;
                dp_lis[i].first += 1;
                dp_lds[i] = ds;
                dp_lds[i].first += 1;
                lis.update(pair{dp_lis[i].first, i}, y);
                lds.update(pair{dp_lds[i].first, i}, m - y - 1);
            }
            auto mx_lis = max_element(dp_lis.begin(), dp_lis.end()) - dp_lis.begin();
            auto mx_lds = max_element(dp_lds.begin(), dp_lds.end()) - dp_lds.begin();
            if (dp_lis[mx_lis] > dp_lds[mx_lds]) {
                int f = mx_lis;
                while (f != -1) {
                    chosen[f] = true;
                    f = dp_lis[f].second;
                }
            } else {
                int f = mx_lds;
                while (f != -1) {
                    chosen[f] = true;
                    f = dp_lds[f].second;
                }
            }
        } else {
            vector<int> lis, lds;
            for (int i = 0; i < m; ++i) {
                while (!lis.empty() && u[lis.back()].y > u[i].y) {
                    lis.pop_back();
                }
                lis.push_back(i);
                while (!lds.empty() && u[lds.back()].y < u[i].y) {
                    lds.pop_back();
                }
                lds.push_back(i);
            }
            if (lis.size() < lds.size()) {
                swap(lis, lds);
            }
            for (int x : lis) {
                chosen[x] = true;
            }
        }
        vector<pt> a, b;
        for (int i = 0; i < m; ++i) {
            if (chosen[i]) {
                b.push_back(u[i]);
            } else {
                a.push_back(u[i]);
            }
        }
        return {a, b};
    };

    int c;
    cin >> c;
    if (task == 2) {
        sort(p.begin(), p.end(), [&](auto x, auto y) {
            return x.x < y.x;
        });

        vector<vector<pt>> g;
        while (!p.empty() && clock() / (1.0 * CLOCKS_PER_SEC) < 1.5) {
            auto [a, b] = choose(p);
            int f = size(b);
//            assert(f * f >= size(p));
            g.push_back(b);
            p = a;
        }
        if (!p.empty()) {
            g.push_back(p);
        }
        assert(g.size() <= 1000);

        for (int i = 0; i < size(g); ++i) {
            for (auto P : g[i]) {
                cout << P.id + 1 << ' ';
            }
//            cout << '\n';
        }

        return 0;
    } else if (task == 3 || task == 4) {
        if (task == 4) {

        }
    }

    return 0;
}