#pragma GCC Optimize("O3")

#include <iostream>
#include <vector>
#include <cstdint>
#include <deque>
#include <unordered_set>
#include <cmath>
#include <random>
#include <algorithm>

using namespace std;

namespace Rand {
    random_device dev;
    mt19937 rng(dev());

    int randint(int a, int b) { return uniform_int_distribution(a, b)(rng); }
}

class PV {
public:
    int64_t x, y;

    inline int64_t operator*(const PV &other) const {
        return x * other.x + y * other.y;
    }

    inline int64_t operator^(const PV &other) const {
        return x * other.y - other.x * y;
    }

    inline int64_t sqlen() const { return x * x + y * y; }

    inline double len() const { return sqrt(sqlen()); }

    inline double cos(const PV &other) const {
        return static_cast<double>((*this) * other) / len() / other.len();
    }

    inline double sin(const PV &other) const {
        return static_cast<double>((*this) ^ other) / len() / other.len();
    }

    inline double angle(const PV &other) const {
        return atan2(sin(other), cos(other));
    }

    inline bool is_sharp(const PV &other) const {
        return (*this) * other > 0;
    }

    inline PV operator-(const PV &other) const {
        return {x - other.x, y - other.y};
    }
};

void Task1(const vector<PV> &points) {
    int n = points.size();
    vector<int> seq(n), pos_by_id(n);
    for (int i = 0; i < n; ++i) seq[i] = i, pos_by_id[i] = i;
    unordered_set<int> stupid_angles;
    for (int i = 1; i + 1 < n; ++i)
        if (!(points[seq[i - 1]] - points[seq[i]]).is_sharp(points[seq[i + 1]] - points[seq[i]]))
            stupid_angles.insert(seq[i]);
    for (int i = 0; i < 1e6 && !stupid_angles.empty(); ++i) {
        int a_id = Rand::randint(0, min<int>(stupid_angles.size() - 1, 10));
        int pos = pos_by_id[*next(stupid_angles.begin(), a_id)];
        swap(seq[pos], seq[pos + 1]);
        pos_by_id[seq[pos]] = pos;
        pos_by_id[seq[pos + 1]] = pos + 1;
        for (int j = max(pos - 1, 0); j <= pos + 2 && j < n; ++j)
            stupid_angles.erase(seq[j]);
        for (int j = max(pos - 1, 1); j <= pos + 2 && j + 1 < n; ++j) {
            if (!(points[seq[j - 1]] - points[seq[j]]).is_sharp(points[seq[j + 1]] - points[seq[j]]))
                stupid_angles.insert(seq[j]);
        }
    }
    for (auto el: seq) cout << el + 1 << " ";
}

void Task2(const vector<PV> &points, int c) {
    int n = points.size();
    vector<pair<PV, int>> sorted_p(n);
    for (int i = 0; i < n; ++i) sorted_p[i] = {points[i], i};
    sort(sorted_p.begin(), sorted_p.end(), [](const pair<PV, int> &a, const pair<PV, int> &b) {
        PV e(1, 0);
        return e.angle(a.first) < e.angle(b.first);
    });
    vector<bool> used(n, false);
    int used_cnt = 0;
    while (used_cnt < n) {
        int min_p_id = -1;
        for (int i = 0; i < n; ++i) {
            if (used[i]) continue;
            if (min_p_id == -1 || sorted_p[i].first.y < sorted_p[min_p_id].first.y ||
                (sorted_p[i].first.y == sorted_p[min_p_id].first.y &&
                 sorted_p[i].first.x < sorted_p[i].first.x))
                min_p_id = i;
        }
        vector<int> points;
        points.push_back(min_p_id);
        used_cnt = 1;
        while (used_cnt < n) {
            int next_id = (min_p_id + 1) % n;
            while (used[next_id]) next_id = (next_id + 1) % n;
            while (points.size() > 2 &&
                   (sorted_p[points[points.size() - 2]].first - sorted_p[points.back()].first).sin(
                           sorted_p[next_id].first - sorted_p[points.back()].first) < 0) {
                used[points.back()] = false;
                --used_cnt;
                points.pop_back();
            }
            ++used_cnt;
            used[next_id] = true;
            points.push_back(next_id);
        }
        for (auto p: points) cout << sorted_p[p].second + 1 << " ";
    }
}

int main() {
    ios_base::sync_with_stdio(false);
    cin.tie(nullptr);
    cout.tie(nullptr);
    int task, group, n;
    cin >> task >> group;
    cin >> n;
    vector<PV> points(n);
    for (auto &pv: points) cin >> pv.x >> pv.y;
    if (task == 1) {
        Task1(points);
        return 0;
    }
    int c;
    cin >> c;
    if (task == 2) {
        Task2(points, c);
    }
    return 0;
}
