#include <iostream>
#include <fstream>
#include <sstream>
#include <vector>
#include <algorithm>
#include <stack>
#include <queue>
#include <deque>
#include <set>
#include <unordered_set>
#include <map>
#include <unordered_map>
#include <cmath>
#include <numeric>
#include <chrono>
#include <random>
#include <bitset>
#include <immintrin.h>
#include <assert.h>

using namespace std;

using ll = long long;
using pii = pair<int, int>;
using pil = pair<int, long long>;
using pli = pair<long long, int>;
using pll = pair<long long, long long>;
template <typename T>
using uset = unordered_set<T>;
template <typename T1, typename T2>
using umap = unordered_map<T1, T2>;

#define all(x) x.begin(), x.end()
#define rall(x) x.rbegin(), x.rend()

struct pnt {
    ll x, y;

    pnt() {}

    pnt(ll x, ll y) : x(x), y(y) {}
};

pnt operator+(pnt a, pnt b) {
    return { a.x + b.x, a.y + b.y };
}

pnt operator-(pnt a, pnt b) {
    return { a.x - b.x, a.y - b.y };
}

ll operator*(pnt a, pnt b) {
    return a.x * b.x + a.y * b.y;
}

ll operator%(pnt a, pnt b) {
    return a.x * b.y - a.y * b.x;
}

struct fenwick {
    int n;
    vector<int> f;

    fenwick() {}

    fenwick(int _n) {
        n = _n + 5;
        f.resize(n);
    }

    void add(int i, int v) {
        i++;
        for (; i < n; i += i & -i)
            f[i] += v;
    }

    int find(int pos) {
        int k = 0;
        int cur = 0;
        for (int i = 15; i > -1; i--)
            if (k + (1 << i) < n && cur + f[k + (1 << i)] < pos) {
                k += 1 << i;
                cur += f[k];
            }
        return k + 1;
    }
};

mt19937 rnd(chrono::steady_clock::now().time_since_epoch().count());
uniform_int_distribution<int> gen(-1e9, 1e9);

int n;
vector<pnt> a;

void genTest() {
#ifdef DEBUG
    n = 7;
    a.resize(n);
    for (int i = 0; i < n; i++)
        a[i].x = gen(rnd), a[i].y = gen(rnd);
#else
    cin >> n;
    a.resize(n);
    for (int i = 0; i < n; i++)
        cin >> a[i].x >> a[i].y;
#endif
}

inline void solve() {
    int task, group;
#ifdef DEBUG
    task = 1, group = 3;
#else
    cin >> task >> group;
#endif
    genTest();
    if (group == 1) {
        int l = 0, r = n - 1;
        vector<int> ord(n);
        for (int i = 0; i < n; i += 2, l++)
            ord[i] = l;
        for (int i = 1; i < n; i += 2, r--)
            ord[i] = r;
        for (int i : ord)
            cout << i + 1 << ' ';
        cout << '\n';
        return;
    }
    if (group == 2) {
        vector<int> ord(n);
        iota(all(ord), 0);
        int ans = 0;
        vector<int> opt = ord;
        do {
            int cnt = 0;
            for (int i = 1; i < n - 1; i++) {
                pnt pr = a[ord[i - 1]];
                pnt cur = a[ord[i]];
                pnt nx = a[ord[i + 1]];
                cnt += (cur - nx) * (cur - pr) > 0;
            }
            if (cnt > ans) {
                ans = cnt;
                opt = ord;
            }
        } while (next_permutation(all(ord)));
        for (int i : opt)
            cout << i + 1 << ' ';
        cout << '\n';
        return;
    }
    if (task == 1) {
        int i1 = rnd() % n;
        int i2 = i1;
        while (i2 == i1)
            i2 = rnd() % n;
        vector<int> ord{ i1, i2 };
        fenwick used(n);
        for (int i = 0; i < n; i++)
            if (i != i1 && i != i2)
                used.add(i, 1);
        for (int point = 2; point < n; point++) {
            int iters = 20000;
            bool found = false;
            while (true) {
                int i = rnd() % (n - point);
                int id = used.find(i + 1) - 1;
                pnt pr, cur, nx;
                pr = a[ord.size() - 2];
                cur = a[ord.back()];
                nx = a[id];
                if ((nx - cur) * (pr - cur) > 0) {
                    ord.push_back(id);
                    used.add(id, -1);
                    found = true;
                    break;
                }
            }
            if (found)
                continue;
            int id = used.find(1) - 1;
            ord.push_back(id);
            used.add(id, -1);
        }
        for (int i : ord)
            cout << i + 1 << ' ';
        cout << '\n';
        return;
    }
    int c;
    cin >> c;
    if (task == 3) {
        int q;
        cin >> q;
        while (q--) {
            int k;
            cin >> k;
            vector<int> ord(n);
            int l = 0, r = n - 1;
            int dir = 1;
            for (int i = 0; i < k + 1; i++) {
                if (dir == 1)
                    ord[i] = l++;
                else
                    ord[i] = r--;
                dir *= -1;
            }
            dir *= -1;
            for (int i = k + 1; i < n; i++) {
                if (dir == -1)
                    ord[i] = r--;
                else
                    ord[i] = l++;
            }
            for (int i : ord)
                cout << i + 1 << ' ';
            cout << endl;
        }
        return;
    }

}

signed main() {
    solve();
    return 0;
}
