#include <iostream>
#include <fstream>
#include <sstream>
#include <vector>
#include <algorithm>
#include <stack>
#include <queue>
#include <deque>
#include <set>
#include <unordered_set>
#include <map>
#include <unordered_map>
#include <cmath>
#include <numeric>
#include <chrono>
#include <random>
#include <bitset>
#include <immintrin.h>
#include <assert.h>

using namespace std;

using ll = long long;
using pii = pair<int, int>;
using pil = pair<int, long long>;
using pli = pair<long long, int>;
using pll = pair<long long, long long>;
template <typename T>
using uset = unordered_set<T>;
template <typename T1, typename T2>
using umap = unordered_map<T1, T2>;

#define all(x) x.begin(), x.end()
#define rall(x) x.rbegin(), x.rend()

struct pnt {
    ll x, y;

    pnt() {}

    pnt(ll x, ll y) : x(x), y(y) {}
};

pnt operator+(pnt a, pnt b) {
    return { a.x + b.x, a.y + b.y };
}

pnt operator-(pnt a, pnt b) {
    return { a.x - b.x, a.y - b.y };
}

ll operator*(pnt a, pnt b) {
    return a.x * b.x + a.y * b.y;
}

ll operator%(pnt a, pnt b) {
    return a.x * b.y - a.y * b.x;
}

struct fenwick {
    int n;
    vector<int> f;

    fenwick() {}

    fenwick(int _n) {
        n = _n + 5;
        f.resize(n);
    }

    void add(int i, int v) {
        i++;
        for (; i < n; i += i & -i)
            f[i] += v;
    }

    int find(int pos) {
        int k = 0;
        int cur = 0;
        for (int i = 15; i > -1; i--)
            if (k + (1 << i) < n && cur + f[k + (1 << i)] < pos) {
                k += 1 << i;
                cur += f[k];
            }
        return k + 1;
    }
};

const int mod = 1e9 + 7;

int add(int a, int b) {
    if (a + b >= mod)
        return a + b - mod;
    return a + b;
}

int sub(int a, int b) {
    if (a < b)
        return a + mod - b;
    return a - b;
}

int mul(int a, int b) {
    return (int)((1ll * a * b) % mod);
}

int binpow(int a, int pw) {
    int b = 1;
    while (pw) {
        if (pw & 1)
            b = mul(b, a);
        a = mul(a, a);
        pw >>= 1;
    }
    return b;
}

int inv(int a) {
    return binpow(a, mod - 2);
}

const int N = 1e5;
const int base = 1e6 + 3;
int pw[N], ipw[N];
int h1n[N], hn1[N];

void precalc() {
    pw[0] = 1;
    for (int i = 1; i < N; i++)
        pw[i] = mul(pw[i - 1], base);
    ipw[0] = 1;
    int in = inv(base);
    for (int i = 1; i < N; i++)
        ipw[i] = mul(ipw[i - 1], in);
}

struct node {
    int h = 0, len = 0;

    node() {}

    node(int v) : h(v), len(1) {}
};

node operator+(node a, node b) {
    node c;
    c.len = a.len + b.len;
    c.h = add(a.h, mul(pw[a.len], b.h));
    return c;
}

struct segtree {
    int sz;
    vector<node> tree;

    segtree() {}

    void build(int x, int tl, int tr) {
        if (tr - tl == 1) {
            tree[x] = tl;
            return;
        }
        int m = (tl + tr) / 2;
        build(2 * x + 1, tl, m);
        build(2 * x + 2, m, tr);
        tree[x] = tree[2 * x + 1] + tree[2 * x + 2];
    }

    segtree(int n) {
        sz = 1;
        while (sz < n)
            sz *= 2;
        tree.resize(2 * sz);
        build(0, 0, sz);
    }

    node get(int l, int r, int x, int tl, int tr) {
        if (l <= tl && tr <= r)
            return tree[x];
        if (l >= tr || r <= tl)
            return node();
        int m = (tl + tr) / 2;
        node s1, s2;
        s1 = get(l, r, 2 * x + 1, tl, m);
        s2 = get(l, r, 2 * x + 2, m, tr);
        return s1 + s2;
    }

    node get(int l, int r) {
        return get(l, r, 0, 0, sz);
    }
};

int n;
vector<pnt> a;

void genTest() {
    cin >> n;
    a.resize(n);
    for (int i = 0; i < n; i++)
        cin >> a[i].x >> a[i].y;
}

int calcHash(vector<int> a) {
    int n = a.size();
    int h = 0;
    for (int i = n - 1; i > -1; i--)
        h = add(mul(h, base), a[i]);
    return h;
}

vector<int> construct(int k) {
    vector<int> ord(n);
    int l = 1, r = n;
    int dir = 1;
    for (int i = 0; i < k + 1; i++) {
        if (dir == 1)
            ord[i] = l++;
        else
            ord[i] = r--;
        dir *= -1;
    }
    dir *= -1;
    for (int i = k + 1; i < n; i++) {
        if (dir == -1)
            ord[i] = r--;
        else
            ord[i] = l++;
    }
    if (k & 1)
        reverse(all(ord));
    return ord;
}

inline void solve() {
    int task, group;
    cin >> task >> group;
    genTest();
    if (group == 1) {
        int l = 0, r = n - 1;
        vector<int> ord(n);
        for (int i = 0; i < n; i += 2, l++)
            ord[i] = l;
        for (int i = 1; i < n; i += 2, r--)
            ord[i] = r;
        for (int i : ord)
            cout << i + 1 << ' ';
        cout << '\n';
        return;
    }
    if (group == 2) {
        vector<int> ord(n);
        iota(all(ord), 0);
        int ans = 0;
        vector<int> opt = ord;
        do {
            int cnt = 0;
            for (int i = 1; i < n - 1; i++) {
                pnt pr = a[ord[i - 1]];
                pnt cur = a[ord[i]];
                pnt nx = a[ord[i + 1]];
                cnt += (cur - nx) * (cur - pr) > 0;
            }
            if (cnt > ans) {
                ans = cnt;
                opt = ord;
            }
        } while (next_permutation(all(ord)));
        for (int i : opt)
            cout << i + 1 << ' ';
        cout << '\n';
        return;
    }
    if (task == 1) {
        return;
    }
    int c;
    cin >> c;
    if (task == 3) {
        int q;
        cin >> q;
        while (q--) {
            int k;
            cin >> k;
            vector<int> ord = construct(k);
            for (int i : ord)
                cout << i << ' ';
            cout << endl;
        }
        return;
    }
    if (task == 4) {
        vector<int> h(n + 1), rev(n + 1);
        segtree st(n + 1);
        vector<int> ord(n);
        int l = 1, r = n;
        int dir = 1;
        for (int i = 0; i < n; i++) {
            if (dir == 1)
                ord[i] = l++;
            else
                ord[i] = r--;
            dir *= -1;
        }
        for (int i = 1; i <= n; i++)
            h[i] = add(h[i - 1], mul(pw[i], ord[i - 1]));
        for (int i = 1; i <= n; i++)
            rev[i] = add(mul(rev[i - 1], base), ord[i - 1]);
        for (int k = c; k <= n - c; k++) {
            if (k & 1)
                cout << add(st.get(k / 2 + 2, n - k / 2 + 1).h, mul(pw[n - k], rev[k]));
            else
                cout << add(h[k], mul(st.get(k / 2 + 1, n - k / 2 + 1).h, pw[k]));
            cout << ' ';
        }
        cout << endl;
        int q;
        cin >> q;
        while (q--) {
            int k;
            cin >> k;
            vector<int> ord = construct(k);
            for (int i : ord)
                cout << i << ' ';
#ifdef DEBUG
            cout << calcHash(ord);
#endif
            cout << endl;
        }
    }
}

signed main() {
    precalc();
    solve();
    return 0;
}
