#include "bits/stdc++.h"

using ll = long long;
using ld = long double;
using namespace std;

struct Point {
    int x = 0, y = 0;

    Point() = default;
    Point(int x, int y) : x(x), y(y) {}
    Point(const Point &A, const Point &B) {
        x = B.x - A.x;
        y = B.y - A.y;
    }

    ll operator*(const Point &other) const {
        return 1ll * x * other.x + 1ll * y * other.y;
    }

    ll operator%(const Point &other) const {
        return 1ll * x * other.y - 1ll * other.x * y;
    }

    friend istream& operator>>(istream &in, Point &other) {
        return in >> other.x >> other.y;
    }
};

namespace task1 {
    int x[20000];
    int y[20000];
    bool used[20000];
    int n;

    void read() {
        cin >> n;
        for (int i = 0; i < n; i++) {
            cin >> x[i] >> y[i];
        }
    }

    ll dist(int i, int j) {
        ll a = x[i] - x[j];
        ll b = y[i] - y[j];
        return a * a + b * b;
    }

    void solve() {
        vector<int> ans = {0};
        vector<int> free;
        for (int i = 1; i < n; i++) {
            free.push_back(i);
        }

        while (ans.size() != n) {
            int u = ans.back();
            auto it = min_element(free.begin(), free.end(), [&u](int i, int j) -> bool {
                return dist(u, i) > dist(u, j);
            });
            ans.push_back(*it);
            swap(*it, free.back());
            free.pop_back();
        }

        for (int i = 0; i < n; i++) {
            cout << ans[i] + 1 << " \n"[i + 1 == n];
        }
    }
}

int sign(ll x) {
    if (x == 0) {
        return 0;
    } else if (x < 0) {
        return -1;
    } else {
        return 1;
    }
}

mt19937 rnd(228);

ld rld() {
    return 1.l * rnd() / numeric_limits<decltype(rnd())>::max();
}

vector<int> maximal(const vector<Point> &P, int n) {
    vector<int> p(n);
    iota(p.begin(), p.end(), 0);
    shuffle(p.begin(), p.end(), rnd);

    int cnt = 0;

    auto add = [&](int i) -> void {
        if (i - 1 < 0 || i + 1 >= n) return;
        cnt += Point(P[p[i]], P[p[i - 1]]) * Point(P[p[i]], P[p[i + 1]]) > 0;
    };

    auto del = [&](int i) -> void {
        if (i - 1 < 0 || i + 1 >= n) return;
        cnt -= Point(P[p[i]], P[p[i - 1]]) * Point(P[p[i]], P[p[i + 1]]) > 0;
    };

    for (int i = 1; i + 1 < n; i++) {
        add(i);
    }

    ld t = 1;

    while (1.l * clock() / CLOCKS_PER_SEC < 1.5) {
        int i = rnd() % (n - 1);

        int prev = cnt;

        del(i - 1);
        del(i);
        del(i + 1);
        del(i + 2);
        swap(p[i], p[i + 1]);
        add(i - 1);
        add(i);
        add(i + 1);
        add(i + 2);

        if (cnt > prev && rld() > expl((prev - cnt) / t)) {
            del(i - 1);
            del(i);
            del(i + 1);
            del(i + 2);
            swap(p[i], p[i + 1]);
            add(i - 1);
            add(i);
            add(i + 1);
            add(i + 2);
        }

        t *= 0.9995;
    }

    return p;
}

signed main() {
    ios::sync_with_stdio(false);
    cin.tie(nullptr);

    int task, group;
    cin >> task >> group;

    if (task == 1) {
//        task1::read();
//        task1::solve();
        return 0;
    }

    int n;
    cin >> n;

    vector<Point> P(n);
    for (int i = 0; i < n; i++) {
        cin >> P[i];
    }

    int c;
    cin >> c;

    if (task == 2) {
        vector<int> p = maximal(P, n);

        for (int i = 0; i < n; i++) {
            cout << p[i] + 1 << ' ';
        }
        return 0;
    }

    if (task == 3) {
//        int q;
//        cin >> q;
//
//        vector<int> mx = maximal(P, n);
//
//        while (q--) {
//            int k;
//            cin >> k;
//
//            auto p = mx;
//
//            int cnt = 0;
//
//            auto add = [&](int i) -> void {
//                if (i - 1 < 0 || i + 1 >= n) return;
//                cnt += Point(P[p[i]], P[p[i - 1]]) * Point(P[p[i]], P[p[i + 1]]) > 0;
//            };
//
//            auto del = [&](int i) -> void {
//                if (i - 1 < 0 || i + 1 >= n) return;
//                cnt -= Point(P[p[i]], P[p[i - 1]]) * Point(P[p[i]], P[p[i + 1]]) > 0;
//            };
//
//            for (int i = 1; i + 1 < n; i++) {
//                add(i);
//            }
//
//            cerr << cnt << endl;
//
//            for (int i = 1; i + 1 < n && cnt < k; i++) {
//                if (Point(P[p[i]], P[p[i - 1]]) * Point(P[p[i]], P[p[i + 1]]) <= 0) {
//                    del(i - 1);
//                    del(i);
//                    del(i + 1);
//                    del(i + 2);
//                    swap(p[i], p[i + 1]);
//                    add(i - 1);
//                    add(i);
//                    add(i + 1);
//                    add(i + 2);
//                }
//            }
//
//            for (int i = 0; i < n; i++) {
//                cout << p[i] + 1 << " \n"[i + 1 == n];
//            }
//            cout.flush();
//        }
//        return 0;
    }

    return 0;
}
