#include <algorithm>
#include <iostream>
#include <numeric>
#include <random>
#include <vector>

struct Point {
    int first;
    int second;
    int i;
    Point() = default;
    bool operator<(const Point& other) const {
        return std::make_pair(first, second) < std::make_pair(other.first, other.second);
    }
};

struct Vector {
    int x;
    int y;
    Vector() = default;
    Vector(int _x, int _y) : x(_x), y(_y) {}
    Vector(const std::pair<int, int>& from, const std::pair<int, int>& to) {
        x = to.first - from.first;
        y = to.second - from.second;
    }
    Vector(const Point& from, const Point& to) {
        x = to.first - from.first;
        y = to.second - from.second;
    }
    long long len2() const {
        return x * 1ll * x + y * 1ll * y;
    }
};

long long dot_prod(const Vector& a, const Vector& b) {
    return a.x * 1ll * b.x + a.y * 1ll * b.y;
}

long long cross_prod(const Vector& a, const Vector& b) {
    return a.x * 1ll * b.y - a.y * 1ll * b.x;
}

int calc(int n, const std::vector<std::pair<int, int>>& a, const std::vector<int>& perm) {
    int cnt = 0;
    for (int i = 1; i < n - 1; i++) {
        Vector pv(a[perm[i]], a[perm[i - 1]]);
        Vector cv(a[perm[i]], a[perm[i + 1]]);
        cnt += (dot_prod(pv, cv) > 0);
    }
    return cnt;
}

int main() {
    int task, group;
    std::cin >> task >> group;
    if (group == 1) {
        int n;
        std::cin >> n;
        for (int i = 0; i < n; i++) {
            int x, y;
            std::cin >> x >> y;
        }
        int lastf = 0;
        int lasts = n - 1;
        for (int i = 0; i < n; i++) {
            if (!(i & 1)) {
                std::cout << lastf++ + 1 << " \n"[i == n - 1];
            } else {
                std::cout << lasts-- + 1 << " \n"[i == n - 1];
            }
        }
        return 0;
    } else if (group == 2) {
        int n;
        std::cin >> n;
        std::vector<std::pair<int, int>> a(n);
        for (int i = 0; i < n; i++) {
            std::cin >> a[i].first >> a[i].second;
        }
        std::vector<int> perm(n);
        std::iota(perm.begin(), perm.end(), 0);
        std::vector<int> bestperm(perm.begin(), perm.end());
        int bestcnt = 0;
        do {
            int cnt = calc(n, a, perm);
            if (cnt > bestcnt) {
                bestcnt = cnt;
                bestperm = perm;
            }
        } while (std::next_permutation(perm.begin(), perm.end()));
        for (int i = 0; i < n; i++) {
            std::cout << bestperm[i] + 1 << " \n"[i == n - 1];
        }
        return 0;
    }
    if (task == 1) {
        return 0;
        int n;
        std::cin >> n;
        std::vector<Point> a(n);
        for (int i = 0; i < n; i++) {
            std::cin >> a[i].first >> a[i].second;
            a[i].i = i;
        }
        std::vector<int> ord(n);
        int first = 0;
        for (int i = 0; i < n; i++) {
            if (a[i] < a[first]) {
                first = i;
            }
        }
        ord[0] = a[first].i;
        auto cpoint = a[first];
        a.erase(a.begin() + first);
        bool curl = false;
        for (int i = 0; i < n; i++) {
            std::sort(a.begin(), a.end(), [&cpoint](const Point& a, const Point& b) {
                Vector v1(cpoint, a);
                Vector v2(cpoint, b);
                if (v1.x == 0 && v1.y == 0) {
                    return false;
                }
                if (v2.x == 0 && v2.y == 0) {
                    return true;
                }
                int hp1, hp2;
                if (v1.y > 0 || (v1.y == 0 && v1.x > 0)) {
                    hp1 = 1;
                } else {
                    hp1 = -1;
                }
                if (v2.y > 0 || (v2.y == 0 && v2.x > 0)) {
                    hp2 = 1;
                } else {
                    hp2 = -1;
                }
                if (hp1 != hp2) {
                    return (hp1 == 1);
                } else {
                    auto ccr = cross_prod(v1, v2);
                    if (ccr != 0) {
                        return (ccr > 0);
                    } else {
                        return (v1.len2() < v2.len2());
                    }
                }
            });
        }
        return 0;
    }
    if (group == 7) {
        int n, c;
        std::cin >> n;
        std::vector<std::pair<int, int>> a(n);
        for (int i = 0; i < n; i++) {
            std::cin >> a[i].first >> a[i].second;
        }
        std::cin >> c;
        int q;
        std::cin >> q;
        for (int qr = 0; qr < q; qr++) {
            int k;
            std::cin >> k;
            int cres = 0;
            int cf = 0;
            int cl = n - 1;
            bool cont1;
            for (int i = 0; i < n; i++) {
                if (cres == k) {
                    if (cont1) {
                        std::cout << cf++ + 1 << " \n"[i == n - 1];
                    } else {
                        std::cout << cl-- + 1 << " \n"[i == n - 1];
                    }
                } else {
                    if (i & 1) {
                        std::cout << cl-- + 1 << " \n"[i == n - 1];
                        cont1 = false;
                    } else {
                        std::cout << cf++ + 1 << " \n"[i == n - 1];
                        cont1 = true;
                    }
                    if (i != 0) {
                        cres++;
                    }
                }
            }
            std::cout << std::flush;
        }
        return 0;
    }
}