#include <iostream>
#include <random>
#include <algorithm>
#include <vector>
#include <bitset>
using namespace std;
using ll = long long;

struct Point {
    ll x;
    ll y;

    Point() {}

    Point(ll x_, ll y_): x(x_), y(y_) {}

    Point operator+(const Point& pt) const {
        return {x + pt.x, y + pt.y};
    }

    Point operator-(const Point& pt) const {
        return {x - pt.x, y - pt.y};
    }

    ll operator^(const Point& pt) const {
        return x * pt.x + y * pt.y;
    }

    ll operator*(const Point& pt) const {
        return x * pt.y - y * pt.x;
    }

    friend ostream& operator<<(ostream& o, const Point& pt) {
        o << "(" << pt.x << ' ' << pt.y << ")";
        return o;
    }
};

struct distcmp {
    Point pt;

    distcmp(Point p): pt(p) {}

    bool operator()(Point p0, Point p1) {
        //cout << pt << ' ' << p0 << ' ' << p1 << ": " << ((pt - p0) ^ (pt - 
        return ((pt - p0) ^ (pt - p0)) < ((pt - p1) ^ (pt - p1));
    }
};

const int MAXN = 80001;
Point pt[MAXN];

vector<int> solve_1(int n) {
    vector<int> perm(n);
    perm[0] = max_element(pt, pt + n, distcmp(pt[0])) - pt;
    perm[0] = max_element(pt, pt + n, distcmp(pt[perm[0]])) - pt;
    bitset<MAXN> free;
    for (int i = 0; i < n; ++i)
        free[i] = 1;
    free[perm[0]] = 0;
    for (int i = 1; i < n; ++i) {
        int mxj = free._Find_first();
        distcmp ds(pt[perm[i - 1]]);
        for (int j = free._Find_first(); j < n; j = free._Find_next(j)) {
            if (ds(pt[mxj], pt[j])) mxj = j;
        }
        perm[i] = mxj;
        free[perm[i]] = 0;
    }
    return perm;
}

mt19937 rng(177013);
vector<int> solve_2_random(int n) {
    vector<int> perm(n);
    for (int i = 0; i < n; ++i)
        perm[i] = i;
    shuffle(perm.begin(), perm.end(), rng);
    vector<int> ans;
    ans.push_back(perm.back());
    perm.pop_back();
    ans.push_back(perm.back());
    perm.pop_back();
    int c = 800;
    while (perm.size()) {
        int pi = ans[ans.size() - 2];
        int i = ans.back();
        int ni = perm.back();
        if (((pt[i] - pt[pi]) ^ (pt[i] - pt[ni])) <= 0) {
            ans.push_back(ni);
            perm.pop_back();
            continue;
        }
        //cout << perm.size() << ' ' << c << endl;
        if (rng() % 100 == 0) {
            if (--c < 0)
                return {};
            ans.push_back(ni);
            perm.pop_back();
            continue;
        }
        swap(perm.back(), perm[rng() % perm.size()]);
    }
    return ans;
}

vector<int> solve_2(int n) {
    vector<int> ans;
    while (true) {
        ans = solve_2_random(n);
        if (ans.size())
            return ans;
    }
}

signed main() {
    int ts, g;
    cin >> ts >> g;
    int n;
    cin >> n;
    for (int i = 0; i < n; ++i) {
        cin >> pt[i].x >> pt[i].y;
    }
    if (ts == 1) {
        vector<int> ans = solve_1(n);
        for (int i = 0; i < n; ++i)
            cout << ans[i] + 1 << ' ';
        cout << endl;
        return 0;
    }
    if (ts == 2) {
        vector<int> ans = solve_2(n);
        for (int i = 0; i < n; ++i)
            cout << ans[i] + 1 << ' ';
        cout << endl;
        return 0;
    }
    
}
