#include <iostream>
#include <random>
#include <cassert>
#include <algorithm>
#include <vector>
#include <bitset>
using namespace std;
using ll = long long;

struct Point {
    ll x;
    ll y;

    Point() {}

    Point(ll x_, ll y_): x(x_), y(y_) {}

    Point operator+(const Point& pt) const {
        return {x + pt.x, y + pt.y};
    }

    Point operator-(const Point& pt) const {
        return {x - pt.x, y - pt.y};
    }

    ll operator^(const Point& pt) const {
        return x * pt.x + y * pt.y;
    }

    ll operator*(const Point& pt) const {
        return x * pt.y - y * pt.x;
    }

    friend ostream& operator<<(ostream& o, const Point& pt) {
        o << "(" << pt.x << ' ' << pt.y << ")";
        return o;
    }
};

struct distcmp {
    Point pt;

    distcmp(Point p): pt(p) {}

    bool operator()(Point p0, Point p1) {
        //cout << pt << ' ' << p0 << ' ' << p1 << ": " << ((pt - p0) ^ (pt - 
        return ((pt - p0) ^ (pt - p0)) < ((pt - p1) ^ (pt - p1));
    }
};

const int MAXN = 80001;
Point pt[MAXN];

vector<int> solve_1(int n) {
    vector<int> perm(n);
    perm[0] = max_element(pt, pt + n, distcmp(pt[0])) - pt;
    perm[0] = max_element(pt, pt + n, distcmp(pt[perm[0]])) - pt;
    bitset<MAXN> free;
    for (int i = 0; i < n; ++i)
        free[i] = 1;
    free[perm[0]] = 0;
    for (int i = 1; i < n; ++i) {
        int mxj = free._Find_first();
        distcmp ds(pt[perm[i - 1]]);
        for (int j = free._Find_first(); j < n; j = free._Find_next(j)) {
            if (ds(pt[mxj], pt[j])) mxj = j;
        }
        perm[i] = mxj;
        free[perm[i]] = 0;
    }
    return perm;
}

mt19937 rng(177013);
vector<int> solve_2_random(int n, int c) {
    vector<int> perm(n);
    for (int i = 0; i < n; ++i)
        perm[i] = i;
    shuffle(perm.begin(), perm.end(), rng);
    vector<int> ans;
    ans.push_back(perm.back());
    perm.pop_back();
    ans.push_back(perm.back());
    perm.pop_back();
    while (perm.size()) {
        int pi = ans[ans.size() - 2];
        int i = ans.back();
        int ni = perm.back();
        if (((pt[i] - pt[pi]) ^ (pt[i] - pt[ni])) <= 0) {
            ans.push_back(ni);
            perm.pop_back();
            continue;
        }
        //cout << perm.size() << ' ' << c << endl;
        if (rng() % 8000 == 0) {
            if (--c < 0)
                return {};
            ans.push_back(ni);
            perm.pop_back();
            continue;
        }
        swap(perm.back(), perm[rng() % perm.size()]);
    }
    return ans;
}

vector<int> solve_2(int n, int c) {
    vector<int> ans;
    while (true) {
        ans = solve_2_random(n, c);
        if (ans.size())
            return ans;
    }
}

bool is_acute(const vector<int>& perm, int i) {
    if (i <= 0 || i + 1 >= int(perm.size()))
        return false;
    return ((pt[perm[i]] - pt[perm[i - 1]]) ^ (pt[perm[i]] - pt[perm[i + 1]])) > 0;
}

int acute_cnt(const vector<int>& perm, int n) {
    int ans = 0;
    for (int i = 1; i + 1 < n; ++i)
        ans += is_acute(perm, i);
    return ans;
}

int delta_swap(vector<int>& perm, int i, int j) {
    if (i == j)
        return 0;
    if (i > j)
        swap(i, j);
    int ans = 0;
    if (j == i + 1) {
        ans -= is_acute(perm, i - 1); ans -= is_acute(perm, i); ans -= is_acute(perm, i + 1); ans -= is_acute(perm, i + 2);
        swap(perm[i], perm[i + 1]);
        ans += is_acute(perm, i - 1); ans += is_acute(perm, i); ans += is_acute(perm, i + 1); ans += is_acute(perm, i + 2);
        return ans;
    }
    ans -= is_acute(perm, i - 1); ans -= is_acute(perm, i); ans -= is_acute(perm, i + 1);
    if (j > i + 2) ans -= is_acute(perm, j - 1); ans -= is_acute(perm, j); ans -= is_acute(perm, j + 1);
    swap(perm[i], perm[j]);
    ans += is_acute(perm, i - 1); ans += is_acute(perm, i); ans += is_acute(perm, i + 1);
    if (j > i + 2) ans += is_acute(perm, j - 1); ans += is_acute(perm, j); ans += is_acute(perm, j + 1);
    return ans;
}

vector<int> find_perm(int n, int k, const vector<int>& mxperm, const vector<int>& mnperm) {
    vector<int> perm = mxperm;
    vector<int> rmnperm(n);
    for (int i = 0; i < n; ++i)
        rmnperm[mnperm[i]] = i;
    int cnt_acute = acute_cnt(mxperm, n);
    for (int i = 0; i < n && cnt_acute > k; ++i) {
        int j = rmnperm[perm[i]];
        cnt_acute += delta_swap(perm, i, j);
        if (cnt_acute == k)
            return perm;
        if (cnt_acute < k) {
            cnt_acute += delta_swap(perm, i, j);
            int l = min(i, j);
            int r = max(i, j);
            for (int p = l; p < r && cnt_acute > k; ++p) {
                cnt_acute += delta_swap(perm, p, p + 1);
            }
            for (int p = r - 2; p >= l && cnt_acute > k; --p) {
                cnt_acute += delta_swap(perm, p, p + 1);
            }
            break;
        }
        if (i != j) {
            --i;
            continue;
        }
    }
    if (cnt_acute == k)
        return perm;
    int tmr = 50000;
    while (cnt_acute != k && tmr-- > 0) {
        int i = rng() % n;
        int j = rng() % n;
        int ncnt_acute = cnt_acute + delta_swap(perm, i, j);
        if (ncnt_acute > k + 3 || ncnt_acute < k - 3) {
            swap(perm[i], perm[j]);
            continue;
        } else {
            cnt_acute = ncnt_acute;
        }
    }
    if (acute_cnt(perm, n) != k) {
        exit(0);
    }
    //assert(acute_cnt(perm, n) == k);
    return perm;
}

void solve_3(int n, int c) {
    vector<int> mxperm = solve_1(n);
    vector<int> mnperm = solve_2(n, c);
    //assert(acute_cnt(mnperm, n) <= c);
    //assert(acute_cnt(mxperm, n) >= n - c);
    int q;
    cin >> q;
    while (q--) {
        int k;
        cin >> k;
        vector<int> ans = find_perm(n, k, mxperm, mnperm);
        for (auto& x : ans)
            cout << x + 1 << ' ';
        cout << endl;
    }
}

void solve_group3(int n, int g) {
    if (g == 8)
        solve_3(n, 25);
    if (g == 9)
        solve_3(n, 80);
    if (g == 10)
        solve_3(n, 300);
    if (g == 11)
        solve_3(n, 600);
    if (g == 12 || g == 13)
        solve_3(n, 5000);
}

signed main() {
    int ts, g;
    cin >> ts >> g;
    int n;
    cin >> n;
    for (int i = 0; i < n; ++i) {
        cin >> pt[i].x >> pt[i].y;
    }
    if (ts == 1) {
        vector<int> ans = solve_1(n);
        for (int i = 0; i < n; ++i)
            cout << ans[i] + 1 << ' ';
        cout << endl;
        return 0;
    }
    if (ts == 2) {
        vector<int> ans = solve_2(n, 800);
        for (int i = 0; i < n; ++i)
            cout << ans[i] + 1 << ' ';
        cout << endl;
        return 0;
    }
    if (ts == 3) {
        solve_group3(n, g);
    }
    
}
