#ifdef LOCAL
#define _GLIBCXX_DEBUG
#else
#pragma GCC optimize("O3,unroll-loops,fast-math")
#endif

#include <iostream>
#include <bits/stdc++.h>
#include <ext/pb_ds/tree_policy.hpp>
#include <ext/pb_ds/assoc_container.hpp>

using namespace __gnu_pbds;
using namespace std;

using ll = long long;
using ld = long double;
using pll = pair<ll, ll>;

ll INF = 1e18;
ll MOD = 1e9 + 7;
ld EPS = 1e-7;

#define mp make_pair
#define mt make_tuple
#define ordered_set tree<ll, null_type, less<>, rb_tree_tag, tree_order_statistics_node_update>

random_device rd;
mt19937_64 gen(rd());

struct SegmentTree {
    struct Node {
        ll x, ind;
    };

    Node NEUTRAL = {INF, 0};

    Node merge(Node p1, Node p2) {
        if (p1.x <= p2.x) return p1;
        return p2;
    }

    vector<Node> p;
    ll n;

    SegmentTree(ll n): n(n) {
        p.assign(4 * n, NEUTRAL);
    }

    Node _get(ll i, ll l, ll r, ll q_l, ll q_r) {
        if (q_l <= l && r <= q_r) return p[i];
        if (q_r <= l || r <= q_l) return NEUTRAL;

        ll m = (l + r) / 2;
        return merge(_get(2 * i + 1, l, m, q_l, q_r), _get(2 * i + 2, m, r, q_l, q_r));
    }

    void _set(ll i, ll l, ll r, ll q_i, ll q_x) {
        if (l + 1 == r) {
            p[i] = {q_x, l};
            return;
        }

        ll m = (l + r) / 2;
        if (q_i < m)
            _set(2 * i + 1, l, m, q_i, q_x);
        else
            _set(2 * i + 2, m, r, q_i, q_x);

        p[i] = merge(p[2 * i + 1], p[2 * i + 2]);
    }

    Node get(ll l, ll r) {
        return _get(0, 0, n, l, r);
    }

    void set(ll i, ll x) {
        _set(0, 0, n, i, x);
    }
};

vector<ll> used;
vector<ll> mate;
vector<vector<ll>> g;

bool kuhn(ll v, ll it) {
    if (used[v] == it) return false;
    used[v] = it;

    for (ll to : g[v]) {
        if (mate[to] == -1) {
            mate[to] = v;
            return true;
        }
    }

    for (ll to : g[v]) {
        if (kuhn(mate[to], it)) {
            mate[to] = v;
            return true;
        }
    }

    return false;
}

struct Edge {
    ll to, w;
};

vector<vector<Edge>> gp;

vector<ll> col;
void dfs(ll v) {
    for (Edge e : gp[v]) {
        if (col[e.to] == -1) {
            col[e.to] = 1 - col[v];
            dfs(e.to);
        }
    }
}

struct Point {
    ll x, y;

    Point operator+(const Point &other) const {
        return {x + other.x, y + other.y};
    }

    Point operator-(const Point &other) const {
        return {x - other.x, y - other.y};
    }

    ll operator*(const Point &other) const {
        return x * other.x + y * other.y;
    }
};

int main() {
#ifdef LOCAL
    freopen("inp.txt", "r", stdin);
    freopen("out.txt", "w", stdout);

    clock_t timer = clock();
#else
    ios_base::sync_with_stdio(false);
    cin.tie(nullptr);
    cout.tie(nullptr);
    cout.precision(64);
    cout << fixed;
#endif

    ll type, group;
    ll n;

    cin >> type >> group >> n;

    vector<Point> p(n);
    for (ll i = 0; i < n; i++) cin >> p[i].x >> p[i].y;

    vector<ll> perm(n);
    std::iota(perm.begin(), perm.end(), 0);

    if (type == 1) {
        ll bestAns = 0;
        for (ll i = 1; i < n - 1; i++) {
            Point x = p[perm[i - 1]] - p[perm[i]];
            Point y = p[perm[i + 1]] - p[perm[i]];

            bestAns += x * y > 0;
        }

        for (ll _ = 0; _ < 1500; _++) {
            ll ans = 0;
            vector<ll> ch;
            for (ll i = 1; i < n - 1; i++) {
                Point x = p[perm[i - 1]] - p[perm[i]];
                Point y = p[perm[i + 1]] - p[perm[i]];

                if (x * y <= 0) ch.push_back(i);
            }

            if (ch.empty()) break;

            ll i = ch[gen() % ch.size()];

            swap(perm[i], perm[i + 1]);

            assert(ans <= bestAns);
            bestAns = ans;
        }

        for (ll w : perm) cout << w + 1 << " ";

    } else if (type == 2) {
        ll c;
        cin >> c;

        vector<ll> bestVec = perm;
        ll bestAns = INF;

        while (bestAns > c) {
            ll ans = 0;

            for (ll i = 1; i < n - 1; i++) {
                Point x = p[perm[i - 1]] - p[perm[i]];
                Point y = p[perm[i + 1]] - p[perm[i]];

                ans += x * y > 0;
            }

            if (ans < bestAns) {
                bestAns = ans;
                bestVec = perm;
            }

            std::shuffle(perm.begin(), perm.end(), gen);
        }

        for (ll w : bestVec) cout << w + 1 << " ";
    }

#ifdef LOCAL
    cout << "\nExecution time: " << (ld)(clock() - timer) / CLOCKS_PER_SEC;
#endif
}
