#include <iostream>
#include <random>
#include <iomanip>
#include <algorithm>
#include <vector>
#include <set>

using namespace std;

#define RANGE(i, n) for (int i = 0; i < n; i++)
#define LOOP(n) RANGE(_, n)
#define LOOP2(n) RANGE2(_, n)
#define elif else if

#ifdef LOCAL
#define ENDLINE endl
#define FLUSH flush
#define DEBUG(x) cerr << #x << ": " << x << endl;
#else
#define ENDLINE '\n'
#define FLUSH ""
#define DEBUG(x) ;
#endif

using ll = long long;
using uint = unsigned int;
using ull = unsigned long long;
using pii = pair<int, int>;
using pss = pair<short, short>;

template<typename T, typename V>
istream &operator>>(istream &in, pair<T, V> &v) {
    in >> v.first >> v.second;
    return in;
}

template<typename T>
istream &operator>>(istream &in, vector<T> &v) {
    for (auto &val: v) {
        in >> val;
    }
    return in;
}


template<typename T, typename V>
ostream &operator<<(ostream &out, pair<T, V> v) {
    out << v.first << " " << v.second;
    return out;
}


template<typename T>
ostream &operator<<(ostream &out, vector<T> &v) {
    for (auto &val: v) {
        out << val << " ";
    }
    return out;
}

struct Point {
    int x, y;

    Point(int x, int y) {
        this->x = x;
        this->y = y;
    }

    Point() : x(0), y(0) {}
};

istream &operator>>(istream &in, Point &p) {
    in >> p.x >> p.y;
    return in;
}

struct Vector {
    int x, y;

    Vector(int x, int y) {
        this->x = x;
        this->y = y;
    }

    Vector(Point &p1, Point &p2) : x(p2.x - p1.x), y(p2.y - p1.y) {}

    Vector() : x(0), y(0) {}

    ll operator*(Vector &other) const {
        return (ll) this->x * other.x + (ll) this->y * other.y;
    }
};

int n;
vector<Point> points;
default_random_engine rnd(1791791791);
int time0;

namespace g1 {
    void solve() {
        int i = 1, j = n;
        bool k = false;
        while (j >= i) {
            if (k) {
                cout << j << " ";
                j--;
            } else {
                cout << i << " ";
                i++;
            }
            k = !k;
        }
        cout << endl;
    }
}

namespace g2 {
    int count(vector<int> &ps) {
        Point p = points[ps[1]];
        Vector v1 = Vector(points[0], points[1]);
        int c = 0;
        for (int i = 2; i < n; i++) {
            Vector v2 = Vector(p, points[ps[i]]);
            if (v1 * v2 < 0) {
                c++;
            }
            v1 = v2;
            p = points[ps[i]];
        }
        return c;
    }

    void solve() {
        vector<int> p(n), p_best(n);
        RANGE(i, n) {
            p_best[i] = p[i] = i;
        }
        int best = count(p);
        while (next_permutation(p.begin(), p.end())) {
            if (best < count(p)) {
                best = count(p);
                copy(p.begin(), p.end(), p_best.begin());
            }
        }
        RANGE(i, n) {
            cout << p_best[i] + 1 << " ";
        }
        cout << endl;
    }
}

namespace g34 {
    void solve() {
        vector<int> p(n), p_best(n);
        RANGE(i, n) {
            p_best[i] = p[i] = i;
        }
        int best = g2::count(p);
        while ((clock() - time0) < CLOCKS_PER_SEC * 1.5) {
            shuffle(p.begin(), p.end(), rnd);
            if (best < g2::count(p)) {
                best = g2::count(p);
                copy(p.begin(), p.end(), p_best.begin());
            }
        }
        RANGE(i, n) {
            cout << p_best[i] + 1 << " ";
        }
        cout << endl;
    }
}

int main() {
    time0 = clock();
    int task, group;
    cin >> task >> group;
    cin >> n;
    points.resize(n);
    cin >> points;
    if (group == 1) {
        g1::solve();
    } elif (group == 2) {
        g2::solve();
    } elif (task == 1) {
        g34::solve();
    }
}
