#include <iostream>
#include <vector>
#include <algorithm>
#include <set>
#include <map>
using namespace std;

ostream& operator<< (ostream& stream, const vector<int>& a) {
    for (auto to : a) {
        stream << to << " ";
    }
    return stream;
}

struct edge {
    int ai, bi, wi;
    int num;
    edge(): ai{-1}, bi{-1}, wi{0}, num{-1} {};
    edge(int ai, int bi, int wi, int num): ai{ai}, bi{bi}, wi{wi}, num{num} {};
};

ostream& operator<< (ostream& stream, const edge& a) {
    stream << "{" << a.ai << ", " << a.bi << ", " << a.wi << ", " << a.num << "}";
    return stream;
}

ostream& operator<< (ostream& stream, const pair<int, int>& a) {
    stream << "{" << a.first << ", " << a.second << "}";
    return stream;
}

ostream& operator<< (ostream& stream, const vector<edge>& a) {
    for (auto to : a) {
        stream << to << " ";
    }
    return stream;
}

ostream& operator<< (ostream& stream, const set<pair<int, int>>& a) {
    for (auto to : a) {
        stream << to << " ";
    }
    return stream;
}

struct edge_to {
    int to, wi;
    int num;
    edge_to(): to{-1}, wi{0}, num{-1} {};
    edge_to(int to, int wi, int num): to{to}, wi{wi}, num{num} {};
};

vector<edge> edges;
vector<vector<edge_to>> g;
vector<vector<int>> c;
int n, t;


bool check_1(const vector<int>& ans) {
    vector<int> idx(n);
    for (int i = 0; i < n; i++) {
        idx[i] = (int) c[i].size() - 1;
    }
    // cout << "edges = " << edges << "\n";
    for (auto num : ans) {
        auto e = edges[num];
        // cout << num << ": " << e.ai + 1 << ", " << e.bi + 1<< "\n";
        if (!(c[e.ai][idx[e.ai]] + c[e.bi][idx[e.bi]] >= e.wi)) {
            // cout << "num = " << num << "\n";
            return false;
        }
        idx[e.ai]--;
        idx[e.bi]--;
    }
    return true;
}

void solve_1() {
    vector<int> ans(n - 1);
    for (int i = 0; i < n - 1; i++) {
        ans[i] = i;
    }
    while (1) {
        // cout << ans << "\n";
        if (check_1(ans)) {
            cout << "YES\n";
            for (auto to : ans) {
                cout << to + 1 << " ";
            }            
            exit(0);
        }
        if (!next_permutation(ans.begin(), ans.end())) {
            break;
        }
    }
    cout << "NO";
}





void solve_2() {
    map<pair<int, int>, pair<int, int>> mapa;
    for (auto e : edges) {
        int ai = e.ai, bi = e.bi;
        if (ai > bi) {
            swap(ai, bi);
        }
        if (mapa.find(make_pair(ai, bi)) == mapa.end()) {
            mapa[make_pair(ai, bi)] = make_pair(e.wi, e.num);
        }
    }
    // for (auto to : mapa) {
    //     cout << to.first << ": " << to.second << "\n";
    // }
    // cout << "\n";
    vector<int> ans;
    ans.reserve(n);
    vector<int> idx(n);
    for (int i = 0; i < n; i++) {
        idx[i] = (int) c[i].size() - 1;
    }
    set<pair<int, int>> q;
    for (auto e : edges) {
        // cout << e.wi << ": " << c[e.ai][idx[e.ai]] << ", " << c[e.bi][idx[e.bi]] << "\n";
        if (c[e.ai][idx[e.ai]] + c[e.bi][idx[e.bi]] >= e.wi) {
            q.insert({-e.wi, e.num});
        }
    }
    // cout << "q = " << q << "\n";
    set<pair<int, int>> used;
    while (q.size()) {
        auto node = *q.begin();
        q.erase(q.begin());
        node.first = -node.first;
        // cout << node << "\n";
        used.insert(node);
        ans.push_back(node.second);
        vector<int> vers = {edges[node.second].ai, edges[node.second].bi};
        for (int v : vers) {
            idx[v]--;
        }
        for (int v : vers) {
            for (auto to : g[v]) {
                int u = to.to;
                edge e = edges[to.num];
                pair<int, int> cur = {e.wi, e.num};
                if (c[e.ai][idx[e.ai]] + c[e.bi][idx[e.bi]] >= e.wi) {
                    if (used.find(cur) != used.end()) {
                        continue;
                    }
                    q.insert({-cur.first, cur.second});
                } else {
                    q.erase(cur);
                }
            }
        }
    }
    if ((int) used.size() == n - 1) {
        cout << "YES\n" << ans;
    } else {
        cout << "NO\n";
    }
}





int main() {
    cin >> n >> t;
    edges.resize(n - 1);
    g.resize(n);
    c.resize(n);
    for (int i = 0; i < n - 1; i++) {
        int ai, bi, wi;
        cin >> ai >> bi >> wi;
        ai--, bi--;
        edges[i] = edge(min(ai, bi), max(ai, bi), wi, i);
        g[ai].push_back(edge_to(bi, wi, i));
        g[bi].push_back(edge_to(ai, wi, i));
    }
    
    for (int i = 0; i < n; i++) {
        c[i].resize(g[i].size());
        for (int j = 0; j < (int) g[i].size(); j++) {
            cin >> c[i][j];
        }
    }
    // for (int i = 0; i < n; i++) {
    //     cout << "c[" << i << "]: " << c[i] << "\n";
    // }
    solve_2();
}