#pragma GCC Optimize("O3")

#include <iostream>
#include <vector>
#include <algorithm>
#include <random>
#include <map>
#include <list>

using namespace std;

struct VecWay {
    int way_id, b_id, w;
};

vector<vector<VecWay>> graph;
vector<int> degs;
vector<vector<int>> sticks;
vector<multimap<int, int>> st_map;
vector<vector<int>> v_order;
vector<int> for_parent_c_a;
bool err = false;
list<int> res;
vector<list<int>::iterator> list_it_by_way;

void MyDFS(int id, int parent = -1) {
    if (err) return;
    for (auto ch: graph[id])
        if (ch.b_id != parent)
            MyDFS(ch.b_id, id);
    if (err) return;
    for (int ch_num = 0; ch_num < graph[id].size(); ++ch_num) {
        auto ch_way = graph[id][ch_num];
        if (ch_way.b_id == parent) continue;
        auto it1 = st_map[id].lower_bound(ch_way.w - for_parent_c_a[ch_way.b_id]);
        if (it1 == st_map[id].end()) {
            err = true;
            return;
        }
        v_order[id][it1->second] = ch_way.way_id;
        st_map[id].erase(it1);
    }
    if (!st_map[id].empty()) {
        for_parent_c_a[id] = st_map[id].rbegin()->first;
        v_order[id][st_map[id].rbegin()->second] = -1;
        st_map[id].erase(prev(st_map[id].end()));
    }
}

void MyDFS2(int id, int par_way_id = -1, int parent = -1) {
    if (err) return;
    if (parent == -1) {
        for (auto ch_way_id: v_order[id]) {
            res.push_back(ch_way_id);
            list_it_by_way[ch_way_id] = prev(res.end());
        }
    } else {
        int i = 0;
        while (v_order[id][i] != -1)++i;
        auto par_way_it = list_it_by_way[par_way_id];
        auto t_it = par_way_it;
        for (int j = i - 1; j >= 0; --j) {
            t_it = res.insert(t_it, v_order[id][j]);
            list_it_by_way[v_order[id][j]] = t_it;
        }
        t_it = next(par_way_it);
        for (int j = i + 1; j < v_order[id].size(); ++j) {
            res.insert(t_it, v_order[id][j]);
            list_it_by_way[v_order[id][j]] = prev(t_it);
        }
    }
    for (auto ch: graph[id])
        if (ch.b_id != parent)
            MyDFS2(ch.b_id, ch.way_id, id);
}

int main() {
    ios_base::sync_with_stdio(false);
    cin.tie(nullptr);
    cout.tie(nullptr);
    int n, g;
    cin >> n >> g;
    graph.resize(n);
    degs.assign(n, 0);
    for (int i = 0, a, b, w; i < n - 1; ++i) {
        cin >> a >> b >> w;
        graph[a - 1].emplace_back(i, b - 1, w);
        graph[b - 1].emplace_back(i, a - 1, w);
        ++degs[a - 1], ++degs[b - 1];
    }
    sticks.resize(n);
    st_map.resize(n);
    v_order.resize(n);
    for_parent_c_a.resize(n);
    list_it_by_way.assign(n - 1, res.end());
    for (int i = 0; i < n; ++i) {
        sticks[i].resize(degs[i]);
        v_order[i].assign(degs[i], -2);
        for (int j = 0; j < sticks[i].size(); ++j) {
            cin >> sticks[i][j];
            st_map[i].emplace(sticks[i][j], j);
        }
    }
    MyDFS(0);
    MyDFS2(0);
    if (err) cout << "No";
    else {
        cout << "Yes\n";
        reverse(res.begin(), res.end());
        for (auto el: res) cout << el + 1 << " ";
    }
    return 0;
}
