#include <algorithm>
#include <iostream>
#include <cstdint>
#include <vector>
#include <queue>
#include <set>

using namespace std;

struct Ulika {
    int64_t a, b, w;
};

vector<int64_t> dist;

void dfs(int64_t v, vector<vector<pair<int64_t, int64_t>>>& graph, vector<bool>& was,
         vector<pair<int64_t, int64_t>>& parent) {
    was[v] = true;
    for (auto p : graph[v]) {
        if (was[p.first]) {
            parent[v] = {p.first, p.second};
            dist[v] = 1+dist[p.first];
            continue;
        }
        dfs(p.first, graph, was, parent);
    }
}

bool cmp(int64_t a, int64_t b) {
    return dist[a]>dist[b];
}

void printans(vector<Ulika>& uliki, vector<pair<int64_t, int64_t>>& ans, int64_t n) {
    vector<pair<pair<int64_t, int64_t>, int64_t>> lst(n-1);
    for (int64_t i = 0; i < n-1; ++i) {
        if (uliki[i].a > uliki[i].b) {
            swap(uliki[i].a, uliki[i].b);
        }
        lst[i] = {{uliki[i].a, uliki[i].b}, i+1};
    }
    sort(lst.begin(), lst.end());
    cout << "Yes\n";
    for (auto i : ans) {
        int64_t L = 0, R = n-1;
        while (R-L > 1) {
            int64_t M = (L+R)/2;
            if (lst[M].first <= i) {
                L = M;
            } else {
                R = M;
            }
        }
        cout << lst[L].second << ' ';
    }
    cout << '\n';
}

int main() {
    ios_base::sync_with_stdio(false);
    cin.tie(nullptr);
    cout.tie(nullptr);

    int64_t n, g;
    cin >> n >> g;
    dist.assign(n, 0);

    vector<Ulika> uliki(n-1);
    vector<vector<pair<int64_t, int64_t>>> graph(n);
    for (int64_t i = 0; i < n-1; ++i) {
        cin >> uliki[i].a >> uliki[i].b >> uliki[i].w;
        --uliki[i].a;
        --uliki[i].b;
        graph[uliki[i].a].emplace_back(uliki[i].b, uliki[i].w);
        graph[uliki[i].b].emplace_back(uliki[i].a, uliki[i].w);
    }

    vector<pair<int64_t, int64_t>> parent(n, {-1, -1});
    vector<bool> was(n, false);
    dfs(0, graph, was, parent);

    vector<int64_t> order(n, 0);
    for (int64_t i = 0; i < n; ++i) {
        order[i] = i;
    }
    sort(order.begin(), order.end(), cmp);
    order.pop_back();

    vector<vector<int64_t>> a(n);
    vector<vector<pair<int64_t, int64_t>>> b(n);
    vector<multiset<pair<int64_t, int64_t>>> mul(n);
    for (int64_t i = 0; i < n; ++i) {
        a[i].resize(graph[i].size());
        b[i].resize(graph[i].size());
        for (int64_t j = 0; j < graph[i].size(); ++j) {
            cin >> a[i][j];
            mul[i].insert({a[i][j], j});
        }
    }

    for (auto i : order) {
        pair<int64_t, int64_t> t = *mul[i].begin();
        int64_t maxw = parent[i].second-t.first;
        auto point = mul[parent[i].first].lower_bound({maxw, -1});
        if (point == mul[parent[i].first].end()) {
            cout << "No\n";
            return 0;
        }
        b[i][t.second] = {parent[i].first, (*point).second};
        b[parent[i].first][(*point).second] = {i, t.second};
        mul[i].clear();
        mul[parent[i].first].erase(point);
    }

    vector<pair<int64_t, int64_t>> ans;
    vector<int64_t> down(n, 0);
    queue<pair<int64_t, int64_t>> q;
    for (int64_t i = 0; i < n; ++i) {
        if ((b[i][0].second == 0) && (i < b[i][0].first)) {
            q.emplace(i, b[i][0].first);
            ++down[i];
            ++down[b[i][0].first];
        }
    }

    while (!q.empty()) {
        pair<int64_t, int64_t> qfront = q.front();
        if (qfront.first > qfront.second) {
            swap(qfront.first, qfront.second);
        }
        q.pop();

        ans.push_back(qfront);

        if ((down[qfront.first] < b[qfront.first].size()) &&
            (b[qfront.first][down[qfront.first]].second == down[b[qfront.first][down[qfront.first]].first])) {
            q.emplace(qfront.first, b[qfront.first][down[qfront.first]].first);
            ++down[b[qfront.first][down[qfront.first]].first];
            ++down[qfront.first];
        }
        if ((down[qfront.second] < b[qfront.second].size()) &&
            (b[qfront.second][down[qfront.second]].second == down[b[qfront.second][down[qfront.second]].first])) {
            q.emplace(qfront.second, b[qfront.second][down[qfront.second]].first);
            ++down[b[qfront.second][down[qfront.second]].first];
            ++down[qfront.second];
        }
    }

    //reverse(ans.begin(), ans.end());
    printans(uliki, ans, n);

    return 0;
}
