#include "bits/stdc++.h"

using namespace std;

using ll = long long;

struct Edge {
    int u, v, pw, id;

    friend bool operator<(Edge L, Edge R) {
        if (L.u != R.u) return L.u < R.u;
        if (L.v != R.v) return L.v < R.v;
        if (L.pw != R.pw) return L.pw < R.pw;
        return false;
    }
};

struct Node {
    int val, used, id, id2;

    Node() : val(-1e9), used(true), id(-1), id2(-1) {};

    Node(int val, int used, int id, int id2) : val(val), used(used), id(id), id2(id2) {};

    friend Node operator+(Node L, Node R) {
        if (L.used && R.used) {
            return Node();
        }
        if (L.used) return R;
        if (R.used) return L;
        if (L.val != R.val) return (L.val < R.val ? L : R);
        return (L.id < R.id ? L : R);
    }
};

template<typename Node>
class Segtree {
public:
    template<typename Input>
    Segtree(int n, vector<Input> a) : n(n) {
        t.resize(4 * n + 1);
        build(0, 0, n, a);
    }

    template<typename Input>
    void update(int i, Input x) {
        update(0, 0, n, i, x);
    }

    Node get(int l, int r) {
        return get(0, 0, n, l, r);
    }

private:
    template<typename Input>
    void build(int v, int l, int r, vector<Input> &a) {
        if (r - l == 1) {
            t[v] = Node(a[l]);
            return;
        }
        int m = (l + r) / 2;
        build(2 * v + 1, l, m, a);
        build(2 * v + 2, m, r, a);
        t[v] = t[2 * v + 1] + t[2 * v + 2];
    }

    template<typename Input>
    void update(int v, int l, int r, int i, Input x) {
        if (r - l == 1) {
            t[v] = Node(x);
            return;
        }
        int m = (l + r) / 2;
        if (m > i) update(2 * v + 1, l, m, i, x);
        else update(2 * v + 2, m, r, i, x);
        t[v] = t[2 * v + 1] + t[2 * v + 2];
    }

    Node get(int v, int l, int r, int ql, int qr) {
        if (ql <= l && r <= qr) return t[v];
        if (ql > l || r > qr) return Node();

        int m = (l + r) / 2;
        return get(2 * v + 1, l, m, ql, qr) + get(2 * v + 2, m, r, ql, qr);
    }

    int n;
    vector<Node> t;
};

signed main() {
#ifdef LOCAL
    freopen("input.txt", "r", stdin);
    freopen("output.txt", "w", stdout);
#else
    ios_base::sync_with_stdio(false);
    cin.tie(nullptr);
#endif

    ll n, group;
    cin >> n >> group;

    vector<Edge> edges;
    vector<int> count(n);
    for (int i = 0; i < n - 1; ++i) {
        int u, v, pw;
        cin >> u >> v >> pw;

        count[u - 1]++;
        count[v - 1]++;
        edges.push_back({u - 1, v - 1, pw, i});
    }

    vector<vector<int>> a(n);
    for (int i = 0; i < n; ++i) {
        for (int j = 0; j < count[i]; ++j) {
            int num;
            cin >> num;

            a[i].push_back(num);
        }
    }

    if (group == 5 || group == 6 || group == 8) {
        auto acopy = a;

        deque<int> q;
        for (int i = 0; i < n; ++i) if (a[i].size() == 1) q.push_back(i);

        vector<pair<int, int>> ans(n - 1);
        vector<int> used(n);
        while (!q.empty()) {
            int v = q.front();
            q.pop_front();
            used[v] = true;
            int edge = -1;
            for (auto [u, id]: g[v]) if (!used[u]) edge = id;

            if (edge == -1) continue;

            int u = (edges[edge].u ^ edges[edge].v) ^ v;
            int first = -1;
            int fv = 1e9;
            for (auto [val, id]: a[u]) {
                if (val < edges[edge].pw - a[v].back().first) continue;
                if (first == -1 || fv > val) {
                    first = id;
                    fv = val;
                }
            }

            if (first == -1) {
                cout << "No" << "\n";
                return 0;
            }

            ans[edge] = {first, a[v].front().second};
            if (u > v) swap(ans[edge].first, ans[edge].second);

            par[u][first] = edge;
            par[v][a[v].front().second] = edge;

            a[u].erase(std::find(a[u].begin(), a[u].end(), make_pair(fv, first)));
            a[v].erase(a[v].begin());

            if (a[u].size() == 1)q.push_back(u);
        }

        a = acopy;

        cout << "Yes" << "\n";
        vector<int> usedE(n - 1);
        for (int j = 0; j < n - 1; ++j) {
            for (int i = 0; i < n - 1; ++i) {
                if (usedE[i]) continue;
                int level1 = ans[i].first;
                int level2 = ans[i].second;
                if (a[edges[i].u].size() == level1 + 1 &&
                    a[edges[i].v].size() == level2 + 1) {
                    cout << i + 1 << " ";
                    a[edges[i].u].pop_back();
                    a[edges[i].v].pop_back();
                    usedE[i] = true;
                    break;
                }
            }
        }
        return 0;
    } else if (group <= 1) {
        std::sort(edges.begin(), edges.end());

        do {
            vector<vector<int>> acopy = a;
            bool flag = true;
            for (auto [u, v, pw, id]: edges) {
                if (a[u].back() + a[v].back() < pw) {
                    flag = false;
                }
                a[u].pop_back();
                a[v].pop_back();
            }
            a = acopy;

            if (!flag) continue;
            cout << "Yes" << "\n";
            for (auto [u, v, pw, id]: edges) {
                cout << id + 1 << " ";
            }
            return 0;
        } while (std::next_permutation(edges.begin(), edges.end()));

        cout << "No" << "\n";
        return 0;
    } else if (group == 2) {
        vector<int> order(n - 1);
        for (int i = 0; i < n - 2; ++i) {
            bool can1 = (a[i].back() + a[i + 1][0] >= edges[i].pw); // second
            bool can2 = a[i + 1].size() > 1 && (a[i].back() + a[i + 1][1] >= edges[i].pw); // first

            if (can1 && can2) {
                if (a[i + 1][0] < a[i + 1][1]) {
                    order[i] = 1;
                    a[i + 1].erase(a[i + 1].begin());
                } else {
                    order[i] = 0;
                    a[i + 1].pop_back();
                }
            } else if (can1) {
                order[i] = 1;
                a[i + 1].erase(a[i + 1].begin());
            } else if (can2) {
                order[i] = 0;
                a[i + 1].pop_back();
            } else {
                cout << "No" << "\n";
                return 0;
            }
        }

        cout << "Yes" << "\n";
        function<void(int)> answer = [&](int v) {
            if (order[v] == 0) cout << v + 1 << " ";
            if (v + 1 != n - 1) answer(v + 1);
            if (order[v] == 1) cout << v + 1 << " ";
        };
        answer(0);
        return 0;
    } else if (group == 3) {
        vector<pair<int, int>> vals;
        for (int i = 0; i < n - 1; ++i) {
            vals.emplace_back(edges[i].pw - a[edges[i].v].back(), i);
        }
        std::sort(vals.begin(), vals.end());

        vector<pair<int, int>> vs;
        for (int i = 0; i < n - 1; ++i) vs.emplace_back(a[0][i], n - i - 2);

        std::sort(vs.begin(), vs.end());

        vector<int> ans(n - 1);
        bool flag = true;
        for (int i = 0; i < n - 1; ++i) {
            if (vs[i].first < vals[i].first) flag = false;
            ans[vs[i].second] = vals[i].second;
        }

        if (!flag) {
            cout << "No" << '\n';
            return 0;
        }
        cout << "Yes" << "\n";
        for (int i = 0; i < n - 1; ++i) {
            cout << ans[i] + 1 << " ";
        }


        return 0;
    } else if (group == 4) {
        vector<Edge> edg0, edg1;
        vector<vector<int>> can0, can1; // val, used, id
        for (int i = 0; i < n - 1; ++i) {
            if (edges[i].u == 0 && edges[i].v != 1) edg0.push_back(edges[i]);
            if (edges[i].u == 1 && edges[i].v != 0) edg1.push_back(edges[i]);
        }

        for (int i = 0; i < a[0].size(); i++) can0.push_back({a[0][i], false, i});
        for (int i = 0; i < a[1].size(); i++) can1.push_back({a[1][i], false, i});

        std::sort(can0.begin(), can0.end());
        std::sort(can1.begin(), can1.end());

        vector<Node> nodes0, nodes1;
        for (int i = 0; i < can0.size(); ++i) nodes0.push_back(Node(can0[i][0], can0[i][1], can0[i][2], i));
        for (int i = 0; i < can1.size(); ++i) nodes1.push_back(Node(can1[i][0], can1[i][1], can1[i][2], i));
        Segtree<Node> tree0(nodes0.size(), nodes0);
        Segtree<Node> tree1(nodes1.size(), nodes1);

        vector<int> used0(can0.size()), used1(can1.size());
        vector<int> par0(can0.size()), par1(can1.size());
        vector<int> ans(n - 1, -1);
        for (auto edg: edg0) {
            int need = edg.pw - a[edg.v][0];
            int l = -1, r = can0.size();
            while (r - l > 1) {
                int mid = (l + r) / 2;
                if (can0[mid][0] >= need) r = mid;
                else l = mid;
            }

            int id = tree0.get(r, can0.size()).id;
            int id2 = tree0.get(r, can0.size()).id2;
            if (id == -1) {
                cout << "No" << "\n";
                return 0;
            }
            tree0.update(id2, Node());
            used0[id] = true;
            par0[id] = edg.id;
        }
        for (auto edg: edg1) {
            int need = edg.pw - a[edg.v][0];
            int l = -1, r = can1.size();
            while (r - l > 1) {
                int mid = (l + r) / 2;
                if (can1[mid][0] >= need) r = mid;
                else l = mid;
            }
            int id = tree1.get(r, can1.size()).id;
            int id2 = tree1.get(r, can1.size()).id2;
            if (id == -1) {
                cout << "No" << "\n";
                return 0;
            }
            tree1.update(id2, Node());
            used1[id] = true;
            par1[id] = edg.id;
        }

        int l = 0;
        int r = 0;
        while (used0[l]) l++;
        while (used1[r]) r++;

        if (a[0][l] + a[1][r] < edges[0].pw) {
            cout << "No" << "\n";
            return 0;
        }

        cout << "Yes" << "\n";
        for (int i = 0; i < l; ++i) {
            cout << par0[i] + 1 << " ";
        }
        for (int i = 0; i < r; ++i) {
            cout << par1[i] + 1 << " ";
        }
        cout << 1 << " ";
        for (int i = l + 1; i < par0.size(); ++i) {
            cout << par0[i] + 1 << " ";
        }
        for (int i = r + 1; i < par1.size(); ++i) {
            cout << par1[i] + 1 << " ";
        }
    }
}