#include <iostream>
#include <vector>
#include <set>
#include <map>
#include <stack>
#include <cassert>
#include <algorithm>
#include <climits>
#define fastio

/*
идем в топологической сортировке
все дети уже были рассмотренны до нас
рассматриваем нас, добавляем в порядок соответсвующие ребра
если не можем добавить ребро, говорим что это ребро в родителя
в родителе сохраняем, что после ПОЛНОГО рассмотрения родителя нам нужно вернуться в данную вершину
*/


using namespace std;

#ifdef fastio
#define endl '\n'
#endif

// #define int long long
typedef pair<int, int> pii;
typedef long long ll;

int n, G;
int root;
bool ok = true;
vector<vector<pii>> g;
vector<pii> parent;
vector<int> order, ans, w;
vector<vector<int>> after_computed;
vector<vector<int>> c;
vector<map<int, stack<int>>> needed;

void resize() {
    g.resize(n);
    parent.resize(n);
    ans.reserve(n - 1);
    w.resize(n - 1);
    after_computed.resize(n);
    c.resize(n);
    needed.resize(n);
}

void dfs(int v, int p, int pi) {
    parent[v] = {p, pi};
    for (const auto &[u, i] : g[v]) {
        if (u == p) continue;
        dfs(u, v, i);
    }
    order.push_back(v);
}

void compute(int v) {

    if (!ok) return;
    int to_parent;
    multiset<pii> spend_to;
    set<int> cset;
    for (const auto &el : c[v]) {
        cset.insert(el);
    }
    for (const auto &[k, v] : needed[v]) {
        for (int i = 0; i < v.size(); ++i) {
            auto it = cset.lower_bound(k);
            if (it == cset.end()) {
                ok = false;
                return;
            }
            auto el = *it;
            cset.erase(el);
            spend_to.insert({el, k});
        }
    }

    assert(cset.size() <= 1);
    if (!cset.empty()) {
    to_parent = *cset.begin();
    cset.clear();
    }
    // во время вычислений эта вершина считается листом
    bool added_edge = false;
    while (!c[v].empty()) {
        int a = c[v].back();
        c[v].pop_back();
        if (a == to_parent) {
            added_edge = true;
            const auto [p, pi] = parent[v];

            int b = w[pi] - a;
            needed[p][b].push(pi);
            after_computed[p].push_back(v);
            break;
        }
        const auto [key, target] = *spend_to.lower_bound({a, INT_MIN});
        spend_to.erase(spend_to.find({key, target}));
        int i = needed[v][target].top();
        needed[v][target].pop();
        ans.push_back(i);
    }
    if (c[v].empty() && !added_edge) {
        for (const auto &u : after_computed[v]) {
            compute(u);
        }
        after_computed[v].clear();
    }
}

signed main() {
    #ifdef fastio
    cin.tie(0);
    ios::sync_with_stdio(0);
    #endif
    #ifdef LOCAL
    freopen("C:/Users/olymp/Documents/code/in.txt", "r", stdin);
    #endif

    cin >> n >> G;

    resize();


    for (int i = 0; i < n - 1; ++i) {
        int a, b;
        cin >> a >> b >> w[i];
        --a;
        --b;
        g[a].push_back({b, i});
        g[b].push_back({a, i});
    }

    for (int i = 0; i < n; ++i) {
        for (int j = 0; j < g[i].size(); ++j) {
            int el;
            cin >> el;
            c[i].push_back(el);
        }
        reverse(c[i].begin(), c[i].end());
    }

    for (int i = 0; i < n; ++i) {
        if (g[i].size() > 1 || i == n - 1) {
            root = i;
            break;
        }
    }

    dfs(root, -1, -1);

    for (const auto v : order) {
        if (v == 4) {
            cout << "";
        }
        if (!ok) break;
        if (v == 0) {
            cout << "" << endl;
        }
        compute(v);
    }

    if (!ok || ans.size() != n - 1) {
        cout << "No" << endl;
    } else {
        reverse(ans.begin(), ans.end());
        cout << "Yes" << endl;
        for (const auto &i : ans) {
            cout << i + 1 << " ";
        }
        cout << endl;
    }


    return 0;
}