#include <iostream>
#include <vector>
#include <set>
#include <map>
#include <stack>
#include <algorithm>
#include <climits>
#include <unordered_map>
#define fastio
// #define LOCAL

/*
идем в топологической сортировке
все дети уже были рассмотренны до нас
рассматриваем нас, добавляем в порядок соответсвующие ребра
если не можем добавить ребро, говорим что это ребро в родителя
в родителе сохраняем, что после ПОЛНОГО рассмотрения родителя нам нужно вернуться в данную вершину
*/


using namespace std;

#ifdef fastio
#define endl '\n'
#endif

// #define int long long
typedef pair<int, int> pii;
typedef long long ll;

int n, G;
bool ok = true;
vector<vector<pii>> g;
vector<pii> parent;
vector<int> order, ans, w;
vector<vector<int>> after_computed;
vector<vector<int>> c;
vector<map<int, stack<int>>> needed;

void resize() {
    g.resize(n);
    parent.resize(n);
    order.reserve(n);
    ans.reserve(n - 1);
    w.resize(n);
    after_computed.resize(n);
    c.resize(n);
    needed.resize(n);
}

void dfs(int v, int p, int pi) {
    parent[v] = {p, pi};
    for (const auto &[u, i] : g[v]) {
        if (u == p) continue;
        dfs(u, v, i);
    }
    order.push_back(v);
}

struct nmset {
    map<int, int> cnt;
    void insert(int el) {
        ++cnt[el];
    }
    void erase(int el) {
        --cnt[el];
        if (cnt[el] == 0) cnt.erase(el);
    }
    int lower_bound(int k) {
        auto it = cnt.lower_bound(k);
        if (it == cnt.end()) return -1;
        return it->first;
    }
};

void compute(int v) {
    if (!ok) return;
    
    int to_parent = -1;
    unordered_map<int, stack<int>> spend_to;
    nmset cset;
    for (const auto &el : c[v]) {
        cset.insert(el);
    }
    for (const auto &[k, v] : needed[v]) {
        for (int i = 0; i < v.size(); ++i) {
            int el = cset.lower_bound(k);
            if (el == -1) {
                ok = false;
                return;
            }
            cset.erase(el);
            spend_to[el].push(k);
        }
    }

    if (!cset.cnt.empty()) {
        to_parent = cset.cnt.begin()->first;
    }
    bool added_edge = false;
    while (!c[v].empty()) {
        int a = c[v].back();
        c[v].pop_back();
        if (a == to_parent) {
            if (v == 0) {
                exit(-1);
            }
            added_edge = true;
            const auto [p, pi] = parent[v];

            int b = max(w[pi] - a, (int)0);
            needed[p][b].push(pi);
            after_computed[p].push_back(v);
            break;
        }   
        int target = spend_to[a].top();
        spend_to[a].pop();
        int i = needed[v][target].top();
        needed[v][target].pop();
        ans.push_back(i);
    }
    if (c[v].empty() && !added_edge) {
        for (const auto &u : after_computed[v]) {
            compute(u);
        }
        after_computed[v].clear();  
    }
}

void inf() {
    while (true) {
        cout << "";
    }
}

signed main() {
    #ifdef fastio
    cin.tie(0);
    ios::sync_with_stdio(0);
    #endif
        // #ifdef LOCAL
    // freopen("/mnt/commons/code/in.txt", "r", stdin);
    // #endif


    cin >> n >> G;

    if (n < 2) {
        inf();
    }

    resize();


    for (int i = 0; i < n - 1; ++i) {
        int a, b;
        cin >> a >> b >> w[i];
        --a;
        --b;
        g[a].push_back({b, i});
        g[b].push_back({a, i});
    }

    for (int i = 0; i < n; ++i) {
        c[i].resize(g[i].size());
        for (int j = 0; j < g[i].size(); ++j) {
            cin >> c[i][j];
        }
        if (!c[i].empty()) reverse(c[i].begin(), c[i].end());
    }

    dfs(0, -1, -1);

    for (const auto v : order) {
        if (!ok) break;
        compute(v);
    }

    if (!ok || ans.size() != n - 1) {
        cout << "No" << endl;
    } else {
        reverse(ans.begin(), ans.end());
        cout << "Yes" << endl;
        for (const auto &i : ans) {
            cout << i + 1 << " ";
        }
        cout << endl;
    }


    return 0;
} 