#include <algorithm>
#include <iostream>
#include <vector>
#include <queue>
#include <map>
#include <cassert>

using namespace std;

const int N = 1000000;

vector<vector<int>> g;
vector<int> used, mt;
int step = 1;

int try_kuhn(int v) {
    if (used[v] == step) {
        return 0;
    }
    used[v] = step;
    for (int to : g[v]) {
        if (mt[to] == -1 || try_kuhn(mt[to])) {
            mt[to] = v;
            return 1;
        }
    }
    return 0;
}

signed main() {
    ios_base::sync_with_stdio(false);
    cin.tie(nullptr);
    int n, G;
    cin >> n >> G;
    vector<tuple<int, int, int>> e(n - 1);
    vector<pair<int, int>> all;
    vector<int> deg(n);
    vector<vector<int>> gg(n), a(n);
    map<pair<int, int>, int> idd;
    for (int i = 0; i < n - 1; i++) {
        int u, v, w;
        cin >> u >> v >> w, u--, v--;
        idd[{u, v}] = idd[{v, u}] = i;
        e[i] = make_tuple(u, v, w);
        gg[u].push_back(v);
        gg[v].push_back(u);
        deg[u]++;
        deg[v]++;
    }
    for (int i = 0; i < n; i++) {
        for (int j = 0; j < deg[i]; j++) {
            int x;
            cin >> x;
            a[i].push_back(x);
        }
        reverse(a[i].begin(), a[i].end());
    }
    for (int i = 0; i < n; i++) {
        for (int j = 0; j < deg[i]; j++) {
            all.emplace_back(i, j);
        }
    }
    sort(all.begin(), all.end());
    all.resize(unique(all.begin(), all.end()) - all.begin());
    auto id = [&](int i, int j) {
        return (int)(lower_bound(all.begin(), all.end(), pair{i, j}) - all.begin());
    };
    queue<int> q;
    q.push(0);
    vector<int> col(n, -1);
    col[0] = 0;
    while (!q.empty()) {
        int v = q.front();
        q.pop();
        for (int to : gg[v]) {
            if (col[to] == -1) {
                col[to] = (col[v] + 1) % 2;
                q.push(to);
            }
        }
    }
    g.resize(N);
    mt.resize(N, -1);
    used.resize(N);
    for (auto [u, v, w] : e) {
        if (col[u] == 1) {
            swap(u, v);
        }
        for (int i1 = 0; i1 < deg[u]; i1++) {
            for (int i2 = 0; i2 < deg[v]; i2++) {
                if (w <= a[u][i1] + a[v][i2]) {
                    g[id(u, i1)].push_back(id(v, i2));
                }
            }
        }
    }
    map<int, int> vv, ii;
    vector<int> st, fin;
    for (int v = 0; v < n; v++) {
        for (int i = 0; i < deg[v]; i++) {
            int h = id(v, i);
            vv[h] = v;
            ii[h] = i;
            if (col[v] == 0) {
                st.push_back(id(v, i));
            }  else {
                fin.push_back(id(v, i));
            }
        }
    }
    int sz = 0;
    for (int v : st) {
        sz += try_kuhn(v);
        step++;
    }
    vector<int> lb(n - 1), rb(n - 1);
    if (sz == n - 1) {
        for (int y : fin) {
            assert(mt[y] != -1);
            int AA = vv[mt[y]];
            int BB = vv[y];
            int II = idd[{AA, BB}];
//            cerr << "$$$$$$$$$  " << AA + 1 << ' ' << BB + 1 << "  " << II << '\n';
            lb[II] = ii[mt[y]];
            rb[II] = ii[y];
//            cerr << "!!! " << II + 1 << ' ' << lb[II] << ' ' << rb[II] << '\n';
        }
        vector<int> ans, hhh(n - 1), ppp(n);
        for (int l = 0; l < n - 1; l++) {
            for (int i = 0; i < n - 1; i++) {
                if (!hhh[i] && ((ppp[get<0>(e[i])] == lb[i] && ppp[get<1>(e[i])] == rb[i]) ||
                                 ppp[get<1>(e[i])] == lb[i] && ppp[get<0>(e[i])] == rb[i])) {
                    ppp[get<0>(e[i])]++;
                    ppp[get<1>(e[i])]++;
                    used[i] = 1;
                    ans.push_back(i);
                    break;
                }
            }
        }
        cout << "Yes\n";
        for (int b : ans) {
            cout << b + 1 << ' ';
        }
        cout << '\n';
    } else {
        cout << "No\n";
    }
    return 0;
}
/*

5 0
1 2 3
1 3 1
3 4 12
3 5 6
0 4
2
6 1 3
8
3


*/