#include <iostream>
#include <vector>
#include <algorithm>
#include <set>

using namespace std;
using ll = long long;

struct Edge {
    ll v, u, w;
};

void dfs(ll from, const vector<vector<ll>> &GG, vector<bool> &used, vector<ll> &tp) {
    used[from] = true;
    for (const auto &to: GG[from]) {
        if (!used[to]) {
            dfs(to, GG, used, tp);
        }
    }

    tp.push_back(from);
}

void solve() {
    ll n, g;
    cin >> n >> g;
    vector<ll> deg(n);
    vector<Edge> edges(n - 1);
    for (auto &item: edges) {
        cin >> item.v >> item.u >> item.w;
        --item.v;
        --item.u;
        ++deg[item.v];
        ++deg[item.u];
    }

    vector<vector<ll>> cs(n);
    for (ll i = 0; i < n; ++i) {
        for (ll j = 0; j < deg[i]; ++j) {
            ll x;
            cin >> x;
            cs[i].push_back(x);
        }
    }

    if (g == 0 || g == 1) {
        vector<ll> prm(n - 1);
        for (ll i = 0; i < n - 1; ++i) {
            prm[i] = i;
        }

        do {
            vector<vector<ll>> tCs = cs;
            bool bad = false;
            for (auto &eI: prm) {
                Edge e = edges[eI];
                if (tCs[e.v].back() + tCs[e.u].back() < e.w) {
                    bad = true;
                    break;
                }
                tCs[e.v].pop_back();
                tCs[e.u].pop_back();
            }

            if (!bad) {
                cout << "Yes\n";
                for (const auto &item: prm) {
                    cout << item + 1 << ' ';
                }
                cout << '\n';
                return;
            }
        } while (next_permutation(prm.begin(), prm.end()));

        cout << "No\n";
        return;
    } else if (g == 3) {
        vector<pair<ll, ll>> need;
        vector<pair<ll, ll>> have;
        for (ll i = 0; i < n - 1; ++i) {
            need.emplace_back(edges[i].w - cs[edges[i].u][0], i);
            have.emplace_back(cs[0][i], n - i - 2);
        }

        sort(need.begin(), need.end());
        sort(have.begin(), have.end());
        vector<ll> res(n - 1);
        for (ll i = 0; i < n - 1; ++i) {
            if (have[i].first < need[i].first) {
                cout << "No\n";
                return;
            }

            res[have[i].second] = need[i].second;
        }

        cout << "Yes\n";
        for (const auto &item: res) {
            cout << item + 1 << ' ';
        }
        cout << '\n';
    } else if (g == 2) {
        ll cP = cs[0][0];
        vector<vector<ll>> GG(n - 1);
        for (ll i = 0; i < n - 2; ++i) {
            Edge e = edges[i];
            ll U = e.u;
            if (cP + max(cs[U][0], cs[U][1]) < e.w) {
                cout << "No\n";
                return;
            } else if (cP + min(cs[U][0], cs[U][1]) < e.w) {
                cP = min(cs[U][0], cs[U][1]);

                if (cs[U][0] > cs[U][1]) {
                    GG[i + 1].push_back(i);
                } else {
                    GG[i].push_back(i + 1);
                }
            } else {
                cP = max(cs[U][0], cs[U][1]);
                if (cs[U][0] > cs[U][1]) {
                    GG[i].push_back(i + 1);
                } else {
                    GG[i + 1].push_back(i);
                }
            }
        }

        if (cP + cs[n - 1][0] < edges.back().w) {
            cout << "No\n";
            return;
        }

        vector<ll> tp;
        vector<bool> used(n - 1);
        for (ll i = 0; i < n - 1; ++i) {
            if (!used[i]) {
                dfs(i, GG, used, tp);
            }
        }

        reverse(tp.begin(), tp.end());

        cout << "Yes\n";
        for (const auto &item: tp) {
            cout << item + 1 << ' ';
        }
        cout << '\n';
    }
}

int main() {
    ios_base::sync_with_stdio(false);
    cin.tie(nullptr);
    solve();
    return 0;
}
/*
5 0
1 2 3
1 3 1
3 4 12
3 5 6
0 4
2
6 1 3
8
3

4 3
1 2 7
1 3 6
1 4 5
3 2 2
5
4
3

7 2
1 2 4
2 3 4
3 4 4
4 5 4
5 6 4
6 7 4
2
1 2
2 3
1 2
3 2
1 2
179
 */