#include <iostream>
#include <vector>
#include <algorithm>
#include <set>

using namespace std;
using ll = long long;

struct Edge {
    ll v, u, w;
};

struct EdI {
    ll u, w, id;
};

void dfs(ll from, const vector<vector<ll>> &GG, vector<bool> &used, vector<ll> &tp) {
    used[from] = true;
    for (const auto &to: GG[from]) {
        if (!used[to]) {
            dfs(to, GG, used, tp);
        }
    }

    tp.push_back(from);
}

struct Some {
    ll need, kid, id;

    bool operator<(const Some &other) const {
        return need < other.need;
    }
};

ll dfs(ll from, const vector<vector<EdI>> &gr, vector<vector<ll>> &GG, const vector<vector<ll>> &cs, ll p = -1,
       ll pId = -1) {
    if (from != 0 && gr[from].size() == 1) {
        return cs[from][0];
    }

    vector<Some> kids;
    for (const auto &[to, w, id]: gr[from]) {
        if (to == p) {
            continue;
        }

        kids.push_back({w - dfs(to, gr, GG, cs, from, id), to, id});
    }

    vector<pair<ll, ll>> have;
    for (ll i = 0; i < ll(gr[from].size()); ++i) {
        have.emplace_back(cs[from][i], ll(gr[from].size()) - i - 1);
    }

    sort(kids.begin(), kids.end());
    sort(have.begin(), have.end());
    if (from == 0) {
        vector<ll> ord;
        for (ll i = 0; i < ll(kids.size()); ++i) {
            if (have[i].first < kids[i].need) {
                cout << "No\n";
                exit(0);
            }
        }

        for (ll i = 0; i < ll(kids.size()); ++i) {
            for (ll j = i + 1; j < ll(kids.size()); ++j) {
                if (have[i].second < have[j].second) {
                    GG[kids[i].id].push_back(kids[j].id);
                } else {
                    GG[kids[j].id].push_back(kids[i].id);
                }
            }
        }

        return -1;
    } else {
        bool take = false;
        ll tId = ll(have.size()) - 1;
        for (ll i = 0; i < ll(kids.size()); ++i) {
            if (have[i + take].first < kids[i].need) {
                if (take) {
                    cout << "No\n";
                    exit(0);
                } else {
                    take = true;
                    tId = i;
                    if (i) {
                        if (have[i - 1].second < have[i].second) {
                            GG[kids[i - 1].id].push_back(pId);
                        } else {
                            GG[pId].push_back(kids[i - 1].id);
                        }
                    }

                    --i;
                }
            }
        }

        for (ll i = 0; i < tId; ++i) {
            if (have[i].second < have[pId].second) {
                GG[kids[i].id].push_back(pId);
            } else {
                GG[pId].push_back(kids[i].id);
            }
        }

        for (ll i = tId; i < ll(kids.size()); ++i) {
            if (have[tId].second < have[i + 1].second) {
                GG[pId].push_back(kids[i].id);
            } else {
                GG[kids[i].id].push_back(pId);
            }
        }

        for (ll i = 0; i < ll(kids.size()); ++i) {
            for (ll j = i + 1; j < ll(kids.size()); ++j) {
                if (have[i].second < have[j].second) {
                    GG[kids[i].id].push_back(kids[j].id);
                } else {
                    GG[kids[j].id].push_back(kids[i].id);
                }
            }
        }

        return have[tId].first;
    }
}

void solve() {
    ll n, g;
    cin >> n >> g;
    vector<ll> deg(n);
    vector<Edge> edges(n - 1);
    for (auto &item: edges) {
        cin >> item.v >> item.u >> item.w;
        --item.v;
        --item.u;
        ++deg[item.v];
        ++deg[item.u];
    }

    vector<vector<ll>> cs(n);
    for (ll i = 0; i < n; ++i) {
        for (ll j = 0; j < deg[i]; ++j) {
            ll x;
            cin >> x;
            cs[i].push_back(x);
        }
    }

    vector<vector<ll>> GG(n - 1);
    vector<vector<EdI>> gr(n);
    for (ll i = 0; i < n - 1; ++i) {
        Edge e = edges[i];
        gr[e.v].push_back({e.u, e.w, i});
        gr[e.u].push_back({e.v, e.w, i});
    }

    dfs(0, gr, GG, cs);

    vector<ll> tp;
    vector<bool> used(n - 1);
    for (ll i = 0; i < n - 1; ++i) {
        if (!used[i]) {
            dfs(i, GG, used, tp);
        }
    }

//    reverse(tp.begin(), tp.end());

    cout << "Yes\n";
    for (const auto &item: tp) {
        cout << item + 1 << ' ';
    }
    cout << '\n';

    /*if (g == 0 || g == 1) {
        vector<ll> prm(n - 1);
        for (ll i = 0; i < n - 1; ++i) {
            prm[i] = i;
        }

        do {
            vector<vector<ll>> tCs = cs;
            bool bad = false;
            for (auto &eI: prm) {
                Edge e = edges[eI];
                if (tCs[e.v].back() + tCs[e.u].back() < e.w) {
                    bad = true;
                    break;
                }
                tCs[e.v].pop_back();
                tCs[e.u].pop_back();
            }

            if (!bad) {
                cout << "Yes\n";
                for (const auto &item: prm) {
                    cout << item + 1 << ' ';
                }
                cout << '\n';
                return;
            }
        } while (next_permutation(prm.begin(), prm.end()));

        cout << "No\n";
        return;
    } else if (g == 3) {
        vector<pair<ll, ll>> need;
        vector<pair<ll, ll>> have;
        for (ll i = 0; i < n - 1; ++i) {
            need.emplace_back(edges[i].w - cs[edges[i].u][0], i);
            have.emplace_back(cs[0][i], n - i - 2);
        }

        sort(need.begin(), need.end());
        sort(have.begin(), have.end());
        vector<ll> res(n - 1);
        for (ll i = 0; i < n - 1; ++i) {
            if (have[i].first < need[i].first) {
                cout << "No\n";
                return;
            }

            res[have[i].second] = need[i].second;
        }

        cout << "Yes\n";
        for (const auto &item: res) {
            cout << item + 1 << ' ';
        }
        cout << '\n';
    } else if (g == 2) {
        ll cP = cs[0][0];
        vector<vector<ll>> GG(n - 1);
        for (ll i = 0; i < n - 2; ++i) {
            Edge e = edges[i];
            ll U = e.u;
            if (cP + max(cs[U][0], cs[U][1]) < e.w) {
                cout << "No\n";
                return;
            } else if (cP + min(cs[U][0], cs[U][1]) < e.w) {
                cP = min(cs[U][0], cs[U][1]);

                if (cs[U][0] > cs[U][1]) {
                    GG[i + 1].push_back(i);
                } else {
                    GG[i].push_back(i + 1);
                }
            } else {
                cP = max(cs[U][0], cs[U][1]);
                if (cs[U][0] > cs[U][1]) {
                    GG[i].push_back(i + 1);
                } else {
                    GG[i + 1].push_back(i);
                }
            }
        }

        if (cP + cs[n - 1][0] < edges.back().w) {
            cout << "No\n";
            return;
        }

        vector<ll> tp;
        vector<bool> used(n - 1);
        for (ll i = 0; i < n - 1; ++i) {
            if (!used[i]) {
                dfs(i, GG, used, tp);
            }
        }

        reverse(tp.begin(), tp.end());

        cout << "Yes\n";
        for (const auto &item: tp) {
            cout << item + 1 << ' ';
        }
        cout << '\n';
    } else {
        vector<vector<ll>> GG(n - 1);
        vector<vector<EdI>> gr(n);
        for (ll i = 0; i < n - 1; ++i) {
            Edge e = edges[i];
            gr[e.v].push_back({e.u, e.w, i});
            gr[e.u].push_back({e.v, e.w, i});
        }

        dfs(0, gr, GG, cs);

        vector<ll> tp;
        vector<bool> used(n - 1);
        for (ll i = 0; i < n - 1; ++i) {
            if (!used[i]) {
                dfs(i, GG, used, tp);
            }
        }

        reverse(tp.begin(), tp.end());

        cout << "Yes\n";
        for (const auto &item: tp) {
            cout << item + 1 << ' ';
        }
        cout << '\n';
    }*/
}

int main() {
    ios_base::sync_with_stdio(false);
    cin.tie(nullptr);
    solve();
    return 0;
}
/*
5 0
1 2 3
1 3 1
3 4 12
3 5 6
0 4
2
6 1 3
8
3

4 3
1 2 7
1 3 6
1 4 5
3 2 2
5
4
3

7 2
1 2 4
2 3 4
3 4 4
4 5 4
5 6 4
6 7 4
2
1 2
2 3
1 2
3 2
1 2
179
 */