#pragma GCC optimize("O3,unroll-loops")
#include <bits/stdc++.h>

#define pb push_back
#define fi first
#define se second
#define debug(x) std::cerr << #x << ": " << x << '\n'

using namespace std;
typedef long long i64;

struct dsu {
    vector<int> p, sz;
    dsu (int n) {
        p.assign(n, 0);
        sz.assign(n, 1);
        iota(p.begin(), p.end(), 0);
    }
    int get(int x) {
        if (p[x] != x) {
            p[x] = get(p[x]);
        }
        return p[x];
    }
    bool unite(int u, int v) {
        u = get(u), v = get(v);
        if (u == v) return false;
        else if (sz[u] > sz[v]) swap(u, v);
        sz[v] += sz[u];
        p[u] = v;
        return true;
    }
    bool check(int u, int v) {
        u = get(u), v = get(v);
        return (u == v);
    }
};

struct edge {
    int u, v, w, id;
};

const int nax = 4e5 + 228;
vector<array<int, 6>> g[nax];
vector<array<int, 3>> T[nax];
vector<int> gr[nax];
int clr[nax], mt[nax];
array<int, 5> me[nax];

void dfs(int v, int par){
    if (par < 1) clr[v] = 1;
    for (auto u : gr[v]) {
        if (u != par) {
            clr[u] = 3 - clr[v];
            dfs(u, v);
        }
        assert(clr[u] != clr[v]);
    }
}

bool dbg = 0;
int vis[nax];

bool kuhn(int v, int iter) {
    if (vis[v] == iter) {
        return false;
    }
    vis[v] = iter;
    for (auto [u, _, __, U, V, id] : g[v]) {
        if (mt[u] == -1) {
            mt[u] = v;
            me[u] = {U, V, _, __, id};
            return true;
        }
    }
    for (auto [u, _, __, U, V, id] : g[v]) {
        if (mt[u] != -1 && kuhn(mt[u], iter)) {
            mt[u] = v;
            me[u] = {U, V, _, __, id};
            return true;
        }
    }
    return false;
}

int main() {
    ios::sync_with_stdio(false);
    cin.tie(nullptr);

    int n, _;
    cin >> n >> _;
    dsu dsu(n + 1);
    bool group5 = 1;

    vector<edge> edges(n - 1);
    vector<vector<int>> c(n + 1), vs(n + 1);
    vector<int> deg(n + 1, 0);
    vector<int> ans;
    vector<int> iter(n + 1, 0);
    for (int i = 0; i < n - 1; ++i) {
        cin >> edges[i].u >> edges[i].v >> edges[i].w;
        ++deg[edges[i].u], ++deg[edges[i].v];
        edges[i].id = i + 1;
        gr[edges[i].u].pb(edges[i].v);
        gr[edges[i].v].pb(edges[i].u);
        T[edges[i].u].pb({edges[i].v, edges[i].w, edges[i].id});
        T[edges[i].v].pb({edges[i].u, edges[i].w, edges[i].id});
    }

    sort(edges.begin(), edges.end(), [&] (auto e1, auto e2) {
        return e1.w > e2.w;
    });

    for (int i = 1; i <= n; ++i) {
        c[i].assign(deg[i], 0);
        vs[i].assign(deg[i], 0);
        for (int j = 0; j < deg[i]; ++j) {
            cin >> c[i][j];
            if (j && c[i][j - 1] > c[i][j]) group5 = 0;
        }
    }

    if (!dbg && n <= 10) {
        vector<int> perm(n - 1, 0);
        iota(perm.begin(), perm.end(), 0);
        do {
            for (int i = 1; i <= n; ++i) iter[i] = 0;
            bool ok = 1;
            for (int i : perm) {
                auto [u, v, w, id] = edges[i];
                if (c[v][iter[v]] + c[u][iter[u]] < w) {
                    ok = 0;
                }
                ++iter[v], ++iter[u];
            }
            if (ok) {
                cout << "Yes\n";
                reverse(perm.begin(), perm.end());
                for (int i : perm) {
                    cout << edges[i].id << ' ';
                }
                cout << '\n';
                return 0;
            }
        } while (next_permutation(perm.begin(), perm.end()));
        cout << "No\n";
        return 0;
    }

    if (!dbg && group5) {
        for (int i = 0; i < n - 1; ++i) {
            for (int j = 0; j < n - 1; ++j) {
                auto [u, v, w, id] = edges[j];
                if (!dsu.check(u, v) && c[v][iter[v]] + c[u][iter[u]] >= w) {
                    dsu.unite(u, v);
                    ++iter[v];
                    ++iter[u];
                    ans.pb(id);
                    break;
                }
            }
        }

        if (ans.size() != n - 1) {
            cout << "No\n";
            return 0;
        }

        cout << "Yes\n";
        reverse(ans.begin(), ans.end());
        for (int x : ans) {
            cout << x << ' ';
        }
        cout << '\n';
        return 0;
    }

    memset(mt, -1, sizeof mt);
    memset(me, -1, sizeof me);
    int VS = 1;
    for (int i = 1; i <= n; ++i) {
        for (int j = 0; j < deg[i]; ++j) {
            vs[i][j] = VS++;
        }
    }

    dfs(1, -1);

    for (int i = 0; i < n - 1; ++i) {
        auto [u, v, w, id] = edges[i];
        if (clr[u] != 1) swap(edges[i].u, edges[i].v);
        u = edges[i].u, v = edges[i].v;
        assert(clr[u] == 1);
        for (int j = 0; j < deg[u]; ++j) {
            for (int l = 0; l < deg[v]; ++l) {
                int U = vs[u][j], V = vs[v][l];
                if (c[u][j] + c[v][l] >= w) {
                    g[U].pb({V, j, l, u, v, id});
                }
            }
        }
    }

    // find max matching and check that its size is n - 1
    int matchingSize = 0;
    for (int i = 1; i < nax; ++i) {
        if (kuhn(i, i)) {
            ++matchingSize;
        }
    }

    if (matchingSize < n - 1) {
        cout << "No\n";
        return 0;
    }

    cout << "Yes\n";
    vector<array<int, 5>> used;
    for (int i = 0; i < nax; ++i) {
        if (mt[i] != -1) {
            used.pb(me[i]);
        }
    }
    while (ans.size() < n - 1) {
        for (auto [u, v, cu, cv, id] : used) {
            if (cu == iter[u] && cv == iter[v]) {
                ++iter[u], ++iter[v];
                ans.pb(id);
                break;
            }
        }
    }

    reverse(ans.begin(), ans.end());
    for (auto el : ans) {
        cout << el << ' ';
    }
    cout << '\n';
}
