#include <iostream>
#include <algorithm>
#include <vector>
#include <set>
using namespace std;

const int MAXN = 2e5 + 123;
vector<pair<int, int>> edg[MAXN];
int weight[MAXN];
vector<int> stick[MAXN];
vector<int> stick_edg[MAXN];
vector<bool> stick_free[MAXN];
int deg[MAXN];

bool decompose_leafs(set<int>& leafs) {
    if (leafs.size() <= 1)
        return true;
    int v = *leafs.begin();
    leafs.erase(v);
    auto [u, ei] = edg[v][0];
    edg[v].clear();
    edg[u].erase(find(edg[u].begin(), edg[u].end(), pair<int, int>(v, ei)));
    if (edg[u].size() == 1)
        leafs.insert(u);

    //cout << "(" << ei + 1 << ' ' << v + 1 << ' ' << u + 1 << "): ";
    int vsti = find(stick_free[v].begin(), stick_free[v].end(), true) - stick_free[v].begin();
    int usti = -1;
    for (int ui = 0; ui < deg[u]; ++ui) if (stick_free[u][ui] && stick[v][vsti] + stick[u][ui] >= weight[ei]) {
        if (usti == -1 || stick[u][ui] < stick[u][usti])
            usti = ui;
    }
    //cout << vsti + 1 << ' ' << usti + 1 << "| " << weight[ei] << ' ' << stick[v][vsti] << ' ' << stick[u][usti] << endl;
    if (usti == -1)
        return false;
    stick_free[v][vsti] = stick_free[u][usti] = false;
    stick_edg[v][vsti] = stick_edg[u][usti] = ei;


    bool res = decompose_leafs(leafs);
    if (!res)
        return res;


    return true;
}

vector<int> order_edges(int n) {
    vector<vector<int>> aftedg(n - 1);
    vector<int> rdeg(n - 1, 0);
    for (int v = 0; v < n; ++v) {
        for (int vsti = 0; vsti + 1 < deg[v]; ++vsti) {
            int eu = stick_edg[v][vsti];
            int ev = stick_edg[v][vsti + 1];
            ++rdeg[eu];
            aftedg[ev].push_back(eu);
        }
    }
    vector<int> ans;
    vector<int> st;
    for (int v = 0; v < n - 1; ++v) if (rdeg[v] == 0)
        st.push_back(v);
    while (st.size()) {
        int v = st.back();
        st.pop_back();
        ans.push_back(v);
        for (auto u : aftedg[v]) {
            if (--rdeg[u] == 0)
                st.push_back(u);
        }
    }
    //reverse(ans.begin(), ans.end());
    return ans;
}

vector<int> solve(int n) {
    set<int> leafs;
    for (int v = 0; v < n; ++v) if (deg[v] == 1) leafs.insert(v);
    if (decompose_leafs(leafs)) {
        return order_edges(n);
    } else {
        return {};
    }
}

signed main() {
    ios_base::sync_with_stdio(false);
    cin.tie(nullptr);
    cout.tie(nullptr);
    int n, g;
    cin >> n >> g;
    for (int ei = 0; ei < n - 1; ++ei) {
        int u, v, w;
        cin >> u >> v >> w;
        --u, --v;
        edg[u].push_back({v, ei});
        edg[v].push_back({u, ei});
        weight[ei] = w;
    }
    for (int v = 0; v < n; ++v) {
        deg[v] = edg[v].size();
        stick[v].resize(deg[v]);
        stick_free[v].assign(deg[v], true);
        stick_edg[v].assign(deg[v], -1);
        for (auto& x : stick[v])
            cin >> x;
    }
    //cout << "HOH" << endl;
    vector<int> ans = solve(n);
    if (ans.empty()) {
        cout << "No\n";
        return 0;
    }
    cout << "Yes\n";
    for (int i = 0; i < n - 1; ++i)
        cout << ans[i] + 1 << ' ';
}
