#include <iostream>
#include <algorithm>
#include <unordered_map>
#include <vector>
#include <map>
using namespace std;

const int MAXN = 2e5 + 123;
unordered_map<int, int> edg[MAXN];
int weight[MAXN];
vector<int> stick_edg[MAXN];
multimap<int, int> stick[MAXN];
int deg[MAXN];

bool decompose_leafs(vector<int>& leafs) {
    if (leafs.size() <= 1)
        return true;
    int v = leafs.back();
    leafs.pop_back();
    auto [u, ei] = *edg[v].begin();
    edg[v].clear();
    edg[u].erase(v);
    if (edg[u].size() == 1)
        leafs.push_back(u);

    auto [vsti_weight, vsti] = *stick[v].begin();
    auto usti_iter = stick[u].lower_bound(weight[ei] - vsti_weight);
    //cout << v << ' ' << u << ": ";
    if (usti_iter == stick[u].end()) {
        return false;
    }
    int usti = usti_iter->second;
    stick[v].clear();
    stick[u].erase(usti_iter);

    stick_edg[v][vsti] = stick_edg[u][usti] = ei;


    bool res = decompose_leafs(leafs);
    if (!res)
        return res;


    return true;
}

vector<int> order_edges(int n) {
    vector<vector<int>> aftedg(n - 1);
    vector<int> rdeg(n - 1, 0);
    for (int v = 0; v < n; ++v) {
        for (int vsti = 0; vsti + 1 < deg[v]; ++vsti) {
            int eu = stick_edg[v][vsti];
            int ev = stick_edg[v][vsti + 1];
            ++rdeg[eu];
            aftedg[ev].push_back(eu);
        }
    }
    vector<int> ans;
    vector<int> st;
    for (int v = 0; v < n - 1; ++v) if (rdeg[v] == 0)
        st.push_back(v);
    while (st.size()) {
        int v = st.back();
        st.pop_back();
        ans.push_back(v);
        for (auto u : aftedg[v]) {
            if (--rdeg[u] == 0)
                st.push_back(u);
        }
    }
    //reverse(ans.begin(), ans.end());
    return ans;
}

vector<int> solve(int n) {
    vector<int> leafs;
    for (int v = 0; v < n; ++v) if (deg[v] == 1) leafs.push_back(v);
    if (decompose_leafs(leafs)) {
        return order_edges(n);
    } else {
        return {};
    }
}

signed main() {
    ios_base::sync_with_stdio(false);
    cin.tie(nullptr);
    cout.tie(nullptr);
    int n, g;
    cin >> n >> g;
    for (int ei = 0; ei < n - 1; ++ei) {
        int u, v, w;
        cin >> u >> v >> w;
        --u, --v;
        edg[u].insert({v, ei});
        edg[v].insert({u, ei});
        weight[ei] = w;
    }
    for (int v = 0; v < n; ++v) {
        deg[v] = edg[v].size();
        vector<int> stick_els(deg[v]);
        for (auto& x : stick_els)
            cin >> x;
        for (int i = 0; i < deg[v]; ++i)
            stick[v].insert({stick_els[i], i});
        stick_edg[v].assign(deg[v], -1);
    }
    //cout << "HOH" << endl;
    vector<int> ans = solve(n);
    if (ans.empty()) {
        cout << "No\n";
        return 0;
    }
    cout << "Yes\n";
    for (int i = 0; i < n - 1; ++i)
        cout << ans[i] + 1 << ' ';
}
