#ifdef LOCAL
#define _GLIBCXX_DEBUG
#else
#pragma GCC optimize("O3,unroll-loops,fast-math")
#endif

#include <iostream>
#include <bits/stdc++.h>
#include <ext/pb_ds/tree_policy.hpp>
#include <ext/pb_ds/assoc_container.hpp>

using namespace __gnu_pbds;
using namespace std;

using ll = long long;
using ld = long double;
using pll = pair<ll, ll>;

ll INF = 1e18;
ll MOD = 1e9 + 7;
ld EPS = 1e-7;

#define mp make_pair
#define mt make_tuple
#define ordered_set tree<ll, null_type, less<>, rb_tree_tag, tree_order_statistics_node_update>

random_device rd;
mt19937_64 gen(rd());

struct SegmentTree {
    struct Node {
        ll x, ind;
    };

    Node NEUTRAL = {INF, 0};

    Node merge(Node p1, Node p2) {
        if (p1.x <= p2.x) return p1;
        return p2;
    }

    vector<Node> p;
    ll n;

    SegmentTree(ll n): n(n) {
        p.assign(4 * n, NEUTRAL);
    }

    Node _get(ll i, ll l, ll r, ll q_l, ll q_r) {
        if (q_l <= l && r <= q_r) return p[i];
        if (q_r <= l || r <= q_l) return NEUTRAL;

        ll m = (l + r) / 2;
        return merge(_get(2 * i + 1, l, m, q_l, q_r), _get(2 * i + 2, m, r, q_l, q_r));
    }

    void _set(ll i, ll l, ll r, ll q_i, ll q_x) {
        if (l + 1 == r) {
            p[i] = {q_x, l};
            return;
        }

        ll m = (l + r) / 2;
        if (q_i < m)
            _set(2 * i + 1, l, m, q_i, q_x);
        else
            _set(2 * i + 2, m, r, q_i, q_x);

        p[i] = merge(p[2 * i + 1], p[2 * i + 2]);
    }

    Node get(ll l, ll r) {
        return _get(0, 0, n, l, r);
    }

    void set(ll i, ll x) {
        _set(0, 0, n, i, x);
    }
};

vector<ll> used;
vector<ll> mate;
vector<vector<ll>> g;

bool kuhn(ll v, ll it) {
    if (used[v] == it) return false;

    for (ll to : g[v]) {
        if (mate[to] == -1) {
            mate[to] = v;
            return true;
        }
    }

    for (ll to : g[v]) {
        if (kuhn(mate[to], it)) {
            mate[to] = v;
            return true;
        }
    }

    return false;
}

struct Edge {
    ll to, w;
};

vector<vector<Edge>> gp;

vector<ll> col;
void dfs(ll v) {
    for (Edge e : gp[v]) {
        if (col[e.to] == -1) {
            col[e.to] = 1 - col[v];
            dfs(e.to);
        }
    }
}

int main() {
#ifdef LOCAL
    freopen("inp.txt", "r", stdin);
    freopen("out.txt", "w", stdout);

    clock_t timer = clock();
#else
    ios_base::sync_with_stdio(false);
    cin.tie(nullptr);
    cout.tie(nullptr);
    cout.precision(64);
    cout << fixed;
#endif

    ll n, group;
    cin >> n >> group;

    map<pll, ll> ord;

    gp.assign(n, {});
    for (ll i = 0; i < n - 1; i++) {
        ll u, v, w;
        cin >> u >> v >> w;
        u--, v--;

        gp[u].push_back({v, w});
        gp[v].push_back({u, w});

        ord[{u, v}] = ord[{v, u}] = i;
    }

    col.assign(n, -1);
    col[0] = 0;
    dfs(0);

    vector<vector<ll>> c(n);
    vector<vector<ll>> ind(n);

    vector<ll> revTo1, revTo2;

    for (ll i = 0; i < n; i++) {
        c[i].assign(gp[i].size(), 0);
        ind[i].assign(gp[i].size(), 0);
        for (ll j = 0; j < gp[i].size(); j++) {
            cin >> c[i][j];

            if (col[i] == 0) {
                ind[i][j] = g.size();
                g.push_back({});
                used.push_back(-1);
                revTo1.push_back(i);
            } else {
                ind[i][j] = mate.size();
                mate.push_back(-1);
                revTo2.push_back(i);
            }
        }

        std::reverse(ind[i].begin(), ind[i].end());
        std::reverse(c[i].begin(), c[i].end());
    }

    for (ll v = 0; v < n; v++) {
        if (col[v]) continue;

        for (Edge e : gp[v]) {
            ll to = e.to;

            for (ll i = 0; i < c[v].size(); i++) {
                for (ll j = 0; j < c[to].size(); j++) {
                    if (c[v][i] + c[to][j] >= e.w) {
                        g[ind[v][i]].push_back(ind[to][j]);
                    }
                }
            }
        }
    }

    ll ans = 0;
    for (ll i = 0; i < g.size(); i++) ans += kuhn(i, ans);

    if (ans != n - 1) {
        cout << "No";
        return 0;
    }

    vector<pll> edges;
    for (ll i = 0; i < mate.size(); i++) {
        if (mate[i] == -1) return 0;
        edges.emplace_back(mate[i], i);
    }

    vector<ll> out;

    ll cnt = 0;

    vector<bool> usedEdges(n - 1);

    while (*std::min_element(usedEdges.begin(), usedEdges.end()) == 0) {
        for (ll i = 0; i < n - 1; i++) {
            pll w = edges[i];
            if (usedEdges[i]) continue;

            ll v = revTo1[w.first], to = revTo2[w.second];

            if (out.size() > n - 1) return 0;
            if (v >= n || to >= n) return 0;
            if (v < 0 || to < 0) return 0;
            if (ind[v].empty()) return 0;
            if (ind[to].empty()) return 0;

            if (ind[v].back() == w.first && ind[to].back() == w.second) {
                out.push_back(ord[{v, to}]);
                ind[v].pop_back();
                ind[to].pop_back();

                usedEdges[i] = true;
                break;
            }
        }
    }

    std::reverse(out.begin(), out.end());

    cout << "Yes\n";
    for (ll w : out) cout << w + 1 << " ";

#ifdef LOCAL
    cout << "\nExecution time: " << (ld)(clock() - timer) / CLOCKS_PER_SEC;
#endif
}
