#define _CRT_SECURE_NO_WARNINGS

#pragma gcc optimise("Ofast")
#pragma gcc optimise("unswith-loops")
#pragma gcc optimise("unroll-loops")
#pragma gcc optimise("no-stack-protector")
#pragma gcc optimise("fast-math")
#pragma gcc optimise("rename-registers")

#include <iostream>
#include <vector>
#include <map>
#include <unordered_map>
#include <unordered_set>
#include <set>
#include <iomanip>
#include <algorithm>
#include <numeric>
#include <cassert>

using namespace std;

#define int long long
typedef long long ll;
typedef pair<int, int> pii;
typedef long double ld;

#define walk(v) v.begin(), v.end()
#define sz(v) (int)v.size()

struct edge {
	int a, b, cost;
};

const int N = 2e5 + 228;
vector<int> marks[N];
vector<edge> edges;
vector<pii> arr[N];

void solve_fac(int n) {
	vector<int> order(n - 1);
	iota(walk(order), 0);
	do {
		vector<int> pointers(n);
		for (int i = 0; i < n; i++)
			pointers[i] = sz(marks[i]) - 1;
		bool flag = true;
		for (int i : order) {
			auto [fir, sec, cost] = edges[i];
			if (marks[fir][pointers[fir]] + marks[sec][pointers[sec]] >= cost) {
				pointers[fir]--;
				pointers[sec]--;
			}
			else {
				flag = false;
				break;
			}
		}
		if (flag) {
			cout << "Yes\n";
			for (int i : order)
				cout << i + 1 << " ";
			return;
		}
	} while (next_permutation(walk(order)));
	cout << "No";
}

void solve_bamboo(int n) {
	vector<int> after, before;
	for (int i = 0; i < n - 2; i++) {
		int fir = marks[i][0] + marks[i + 1][0], sec = marks[i][0] + marks[i + 1][1];
		if (fir < sec || sec < edges[i].cost) {
			if (fir >= edges[i].cost) {
				after.push_back(i);
				swap(marks[i + 1][0], marks[i + 1][1]);
				continue;
			}
		}
		if (sec < edges[i].cost) {
			cout << "No";
			return;
		}
		before.push_back(i);
	}
	if (marks[n - 2][0] + marks[n - 1][0] < edges.back().cost) {
		cout << "No";
		return;
	}
	cout << "Yes\n";
	for (int u : before)
		cout << u + 1 << " ";
	cout << n - 1 << " ";
	reverse(walk(after));
	for (int u : after)
		cout << u + 1 << " ";
}

const int INF = 1e10;

pair<bool, int> match(vector<pii>& f, vector<pii>& s) { // укладываем первый во второй, те s[i] > f[i] для любого i
	int extra = -1;
	int it1 = 0, it2 = 0;
	while (it1 < sz(f)) {
		if (f[it1].first > s[it2].first) {
			if (extra != -1)
				return { false, -1 };
			extra = s[it2++].first;
			continue;
		}
		else {
			it1++;
			it2++;
		}
	}
	if (extra == -1)
		extra = s[it2].first;
	return { true, extra };
}

vector<int> match_real(vector<pii>& f, vector<pii>& s) {
	int it1 = 0, it2 = 0;
	vector<int> ans(sz(s), -1);
	while (it1 < sz(f)) {
		if (f[it1].first > s[it2].first) {
			ans[s[it2].second] = -1;
			it2++;
			continue;
		}
		else {
			ans[s[it2].second] = f[it1].second;
			it1++;
			it2++;
		}
	}
	return ans;
}

void solve_double_hedgehog(int n) {
	vector<pii> needs1, needs2;
	int spec_edge = -1, spec_edge_idx;
	for (int i = 0; i < n - 1; i++) {
		if (edges[i].a == 0) {
			if (edges[i].b != 1)
				needs1.emplace_back(edges[i].cost - marks[i + 1][0], i);
			else {
				spec_edge = edges[i].cost;
				spec_edge_idx = i;
			}
		}
		else
			needs2.emplace_back(edges[i].cost - marks[i + 1][0], i);
	}
	assert(spec_edge != -1);
	sort(walk(needs1));
	sort(walk(needs2));
	vector<pii> supplies1, supplies2;
	for (int i = 0; i < sz(marks[0]); i++)
		supplies1.emplace_back(marks[0][i], sz(marks[0]) - i - 1);
	for (int i = 0; i < sz(marks[1]); i++)
		supplies2.emplace_back(marks[1][i], sz(marks[1]) - i - 1);
	sort(walk(supplies1));
	sort(walk(supplies2));
	auto [flag1, extra1] = match(needs1, supplies1);
	if (!flag1) {
		cout << "No";
		return;
	}
	auto [flag2, extra2] = match(needs2, supplies2);
	if (!flag2) {
		cout << "No";
		return;
	}
	if (spec_edge > extra1 + extra2) {
		cout << "No";
		return;
	}
	vector<int> arr1 = match_real(needs1, supplies1),
		arr2 = match_real(needs2, supplies2);
	int it1 = 0;
	while (arr1[it1] != -1)
		cout << arr1[it1++] + 1 << " ";
	int it2 = 0;
	while (arr2[it2] != -1)
		cout << arr2[it2++] + 1 << " ";
	cout << spec_edge_idx + 1 << " ";
	it1++, it2++;
	while (it1 < sz(arr1))
		cout << arr1[it1++] + 1 << " ";
	while (it2 < sz(arr2))
		cout << arr2[it2++] + 1 << " ";
}

void solve_hedgehog(int n) {
	set<pii> needs;
	for (int i = 0; i < n - 1; i++)
		needs.insert({ edges[i].cost - marks[i + 1][0], i });
	vector<int> ans;
	for (int i = n - 2; i >= 0; i--) {
		auto it = needs.upper_bound(make_pair(marks[0][i], INF));
		if (it == needs.begin()) {
			cout << "No";
			return;
		}
		it--;
		ans.push_back(it->second);
		needs.erase(it);
	}
	cout << "Yes\n";
	for (int u : ans)
		cout << u + 1 << " ";
}

void solve_hedgehog2(int n) {
	vector<pii> needs;
	for (int i = 0; i < n - 1; i++)
		needs.push_back({ edges[i].cost - marks[i + 1][0], i });
	vector<pii> supplies;
	for (int i = 0; i < n - 1; i++)
		supplies.push_back({ marks[0][i], n - 1 - i });
	sort(walk(needs));
	sort(walk(supplies));
	vector<int> ans = match_real(needs, supplies);
	for (int i : ans) 
		if (i == -1) {
			cout << "No";
			return;
		}
	cout << "Yes\n";
	for (int u : ans)
		cout << u + 1 << " ";
}

void solve() {
	int n, g;
	cin >> n >> g;
	int f, s, c;
	for (int i = 0; i < n - 1; i++) {
		cin >> f >> s >> c;
		f--, s--;
		edges.emplace_back(f, s, c);
		arr[f].emplace_back(s, c);
		arr[s].emplace_back(f, c);
	}
	for (int i = 0; i < n; i++) {
		marks[i].resize(sz(arr[i]));
		for (int& u : marks[i])
			cin >> u;
	}
	if (g <= 1)
		solve_fac(n);
	else if (g == 2)
		solve_bamboo(n);
	else if (g == 3)
		solve_hedgehog2(n);
	else if (g == 4)
		solve_double_hedgehog(n);
	else
		cout << "No";
}

signed main() {
	cin.tie(nullptr)->sync_with_stdio(false);
	ios_base::sync_with_stdio(false);
	freopen("input.txt", "r", stdin);
	//freopen("output.txt", "w", stdout);
	solve();
}