#include <iostream>
#include <random>
#include <iomanip>
#include <algorithm>
#include <vector>
#include <set>

using namespace std;

#define RANGE(i, n) for (int i = 0; i < n; i++)
#define LOOP(n) RANGE(_, n)
#define LOOP2(n) RANGE2(_, n)
#define elif else if

#ifdef LOCAL
#define ENDLINE endl
#define FLUSH flush
#define DEBUG(x) cerr << #x << ": " << x << endl;
#else
#define ENDLINE '\n'
#define FLUSH ""
#define DEBUG(x) ;
#endif

using ll = long long;
using uint = unsigned int;
using ull = unsigned long long;
using pii = pair<int, int>;
using pss = pair<short, short>;

template<typename T>
istream &operator>>(istream &in, vector<T> &v) {
    for (auto &val: v) {
        in >> val;
    }
    return in;
}

template<typename T>
ostream &operator<<(ostream &out, vector<T> &v) {
    for (auto &val: v) {
        out << val << " ";
    }
    return out;
}

template<typename T, typename V>
ostream &operator<<(ostream &out, pair<T, V> v) {
    out << v.first << " " << v.second;
    return out;
}

mt19937_64 rnd(1791791791);

struct Node {
    int val;
    int data;
    ull y;
    Node *left = nullptr,
            *right = nullptr;
    int size;

    Node(int val, int data) {
        this->val = val;
        this->data = data;
        this->y = rnd();
        this->size = 1;
    }

    void update();
};

int get_size(Node *node) {
    if (node == nullptr) {
        return 0;
    }
    return node->size;
}

void Node::update() {
    this->size = get_size(this->left) + 1 + get_size(this->right);
}

Node *merge(Node *left, Node *right) {
    if (left == nullptr) {
        return right;
    }
    if (right == nullptr) {
        return left;
    }
    if (left->y >= right->y) {
        left->right = merge(left->right, right);
        left->update();
        return left;
    }
    right->left = merge(left, right->left);
    right->update();
    return right;
}

Node *merge(Node *left, Node *central, Node *right) {
    return merge(merge(left, central), right);
}

pair<Node *, Node *> split_size(Node *root, int left_size) {
    if (root == nullptr) {
        return {nullptr, nullptr};
    }
    if (get_size(root->left) > left_size) {
        auto [l, r] = split_size(root->left, left_size);
        root->left = r;
        root->update();
        return {l, root};
    } elif (get_size(root->left) == left_size) {
        auto l = root->left;
        root->left = nullptr;
        root->update();
        return {l, root};
    } elif (get_size(root->left) + 1 == left_size) {
        auto r = root->right;
        root->right = nullptr;
        root->update();
        return {root, r};
    } else {
        auto [l, r] = split_size(root->right, left_size - (get_size(root->left) + 1));
        root->right = l;
        root->update();
        return {root, r};
    }
}

pair<Node *, Node *> split_val_l(Node *root, int max_left) {
    if (root == nullptr) {
        return {nullptr, nullptr};
    }
    if (root->val > max_left) {
        auto [l, r] = split_val_l(root->left, max_left);
        root->left = r;
        root->update();
        return {l, root};
    } else {
        auto [l, r] = split_val_l(root->right, max_left);
        root->right = l;
        root->update();
        return {root, r};
    }
}

pair<Node *, Node *> split_val_r(Node *root, int min_right) {
    if (root == nullptr) {
        return {nullptr, nullptr};
    }
    if (root->val >= min_right) {
        auto [l, r] = split_val_r(root->left, min_right);
        root->left = r;
        root->update();
        return {l, root};
    } else {
        auto [l, r] = split_val_r(root->right, min_right);
        root->right = l;
        root->update();
        return {root, r};
    }
}

tuple<Node *, Node *, Node *> split_val3(Node *root, int central) {
    auto [lc, r] = split_val_l(root, central);
    auto [l, c] = split_val_r(lc, central);
    return {l, c, r};
}

Node *insert(Node *root, Node *node) {
    if (node == nullptr) {
        return root;
    }
    auto [l, r] = split_val_l(root, node->val);
    return merge(l, node, r);
}

Node *insert(Node *root, int val, int data) {
    Node *node = new Node(val, data);
    return insert(root, node);
}

void dfs(int v, vector<vector<int>> &g, vector<int> &num, vector<int> &o, int &NUMERATOR) {
    num[v] = NUMERATOR;
    NUMERATOR++;
    for (int u: g[v]) {
        o[u]--;
        if (o[u] == 0) {
            dfs(u, g, num, o, NUMERATOR);
        }
    }
}

int main() {
    cin.tie(nullptr);
    ios_base::sync_with_stdio(false);
    int n, g;
    cin >> n >> g;
    vector<vector<tuple<int, int, int>>> gr(n); // to, w, i
    RANGE(i, n - 1) {
        int a, b, w;
        cin >> a >> b >> w;
        a--;
        b--;
        gr[a].emplace_back(b, w, i);
        gr[b].emplace_back(a, w, i);
    }
    vector<Node *> cs(n, nullptr);
    RANGE(i, n) {
        RANGE(j, gr[i].size()) {
            int c;
            cin >> c;
            cs[i] = insert(cs[i], c, j);
        }
    }
    set<pair<int, int>> q; // now_deg, i
    vector<int> degs(n), degs_bfs(n);
    vector<vector<int>> s(n);
    RANGE(i, n) {
        degs_bfs[i] = degs[i] = gr[i].size();
        q.insert({degs[i], i});
        s[i].resize(degs[i]);
    }
    vector<bool> used(n, false);
    while (!q.empty()) {
        auto [_d, v] = *q.begin();
        q.erase(q.begin());
        used[v] = true;
        if (_d == 0) {
            continue;
        }
#ifdef LOCAL
        if (_d > 1) {
//            exit(1);
            return 1;
        }
#endif
        for (auto [u, w, i]: gr[v]) {
            if (!used[u]) {
                auto [l, cr] = split_val_r(cs[u], w - cs[v]->val);
                auto [c, r] = split_size(cr, 1);
                if (get_size(c) == 0) {
                    cout << "No" << ENDLINE;
                    return 0;
                }
                s[v][cs[v]->data] = s[u][c->data] = i;
                cs[u] = merge(l, r);
                cs[v] = nullptr;
                q.erase({degs_bfs[u], u});
                degs_bfs[u]--;
                q.insert({degs_bfs[u], u});
            }
        }
    }
    cout << "Yes" << ENDLINE;
    vector<vector<int>> gr2(n - 1);
    vector<int> num(n - 1, -1);
    vector<int> o(n - 1, 0);
    RANGE(i, n) {
        RANGE(j, degs[i] - 1) {
            gr2[s[i][j]].push_back(s[i][j + 1]);
            o[s[i][j + 1]]++;
        }
    }
    int NUMERATOR = 0;
    RANGE(i, n - 1) {
        if (num[i] == -1 && o[i] == 0) {
            dfs(i, gr2, num, o, NUMERATOR);
        }
    }
    vector<int> k(n - 1);
    RANGE(i, n - 1) {
        k[num[i]] = i + 1;
    }
    reverse(k.begin(), k.end());
    cout << k << ENDLINE;
}
