#include <iostream>
#include <vector>
#include <algorithm>
#include <random>

using namespace std;
namespace Test {
    random_device dev;
    uint32_t seed = dev();
    mt19937 rng(seed);

    int64_t randint64_t(int64_t a, int64_t b) {
        return uniform_int_distribution(a, b)(rng);
    }

    int64_t sol1(int64_t n, int64_t k, int64_t t, vector<int64_t> ts) {
        vector<int64_t> temp = ts;
        sort(temp.begin(), temp.end());
        for (auto &ty: ts) ty = distance(temp.begin(), lower_bound(temp.begin(), temp.end(), ty));
        vector<bool> difs_vec(n, false);
        for (auto ty: ts) difs_vec[ty] = true;
        int64_t difs_cnt = count(difs_vec.begin(), difs_vec.end(), true);
        if (difs_cnt <= t) {
            return 1;
        }
        vector<int64_t> add_pos(n, -1);
        vector<int64_t> cur_used(n, 0);
        int64_t timer = 1;
        int64_t cur_pos = 0, cur_box = 0;
        int64_t res = 0;
        while (cur_box < k) {
            if (add_pos[cur_pos] != -1) {
                cur_pos += add_pos[cur_pos];
                cur_box += cur_pos / n;
                cur_pos %= n;
            } else {
                int64_t st_pos = cur_pos;
                int64_t new_types = 0;
                for (; new_types <= t; ++cur_pos) {
                    if (cur_used[ts[cur_pos % n]] != timer) {
                        ++new_types;
                        cur_used[ts[cur_pos % n]] = timer;
                    }
                }
                --cur_pos;
                ++timer;
                add_pos[st_pos] = cur_pos - st_pos;
                cur_box += cur_pos / n;
                cur_pos %= n;
            }
            ++res;
        }
        return res;
    }

    int64_t sol2(int64_t n, int64_t k, int64_t t, vector<int64_t> ts) {
        vector<int64_t> temp = ts;
        sort(temp.begin(), temp.end());
        for (auto &ty: ts) ty = distance(temp.begin(), lower_bound(temp.begin(), temp.end(), ty));
        vector<bool> difs_vec(n, false);
        for (auto ty: ts) difs_vec[ty] = true;
        int64_t difs_cnt = count(difs_vec.begin(), difs_vec.end(), true);
        if (difs_cnt <= t) {
            return 1;
        }
        vector<int64_t> add_pos(n, -1);
        vector<int64_t> cur_used(n, 0);
        int64_t timer = 1;
        int64_t cur_pos = 0, cur_box = 0;
        int64_t res = 0;
        bool cycle_found = false;
        while (cur_box < k) {
            if (add_pos[cur_pos] != -1) {
                cycle_found = true;
                break;
            } else {
                int64_t st_pos = cur_pos;
                int64_t new_types = 0;
                for (; new_types <= t; ++cur_pos) {
                    if (cur_used[ts[cur_pos % n]] != timer) {
                        ++new_types;
                        cur_used[ts[cur_pos % n]] = timer;
                    }
                }
                --cur_pos;
                ++timer;
                add_pos[st_pos] = cur_pos - st_pos;
                cur_box += cur_pos / n;
                cur_pos %= n;
            }
            ++res;
        }
        if (cycle_found) {
            int64_t st_pos = cur_pos;
            int64_t plus_box_cnt = 0, plus_res_cnt = 0;
            do {
                cur_pos += add_pos[cur_pos];
                plus_box_cnt += cur_pos / n;
                cur_pos %= n;
                ++plus_res_cnt;
            } while (cur_pos != st_pos);
            int64_t cycles_cnt = (k - cur_box - 1) / plus_box_cnt;
            res += plus_res_cnt * cycles_cnt;
            cur_box += plus_box_cnt * cycles_cnt;
        }
        while (cur_box < k) {
            if (add_pos[cur_pos] != -1) {
                cur_pos += add_pos[cur_pos];
                cur_box += cur_pos / n;
                cur_pos %= n;
            } else {
                int64_t st_pos = cur_pos;
                int64_t new_types = 0;
                for (; new_types <= t; ++cur_pos) {
                    if (cur_used[ts[cur_pos % n]] != timer) {
                        ++new_types;
                        cur_used[ts[cur_pos % n]] = timer;
                    }
                }
                --cur_pos;
                ++timer;
                add_pos[st_pos] = cur_pos - st_pos;
                cur_box += cur_pos / n;
                cur_pos %= n;
            }
            ++res;
        }
        return res;
    }

    void gen(int64_t &n, int64_t &k, int64_t &t, vector<int64_t> &ts) {
        n = randint64_t(1, 10);
        k = randint64_t(1, 10);
        t = randint64_t(1, 10);
        ts.assign(n, 0);
        for (auto &ty: ts) ty = randint64_t(0, n - 1);
    }

    void Test() {
        int64_t n, k, t;
        vector<int64_t> ts;
//        int64_t n = 4, k = 2, t = 2;
//        vector<int64_t> ts{1, 0, 1, 2};
        while (true) {
            gen(n, k, t, ts);
            int64_t res1 = sol1(n, k, t, ts), res2 = sol2(n, k, t, ts);
            if (res1 != res2) {
                cout << n << " " << k << " " << t << "\n";
                for (auto ty: ts) cout << ty << " ";
                cout << "\n right: " << res1 << ", my: " << res2 << "\n";
                break;
            }
        }
    }
}

int main() {
    ios_base::sync_with_stdio(false);
    cin.tie(nullptr);
    cout.tie(nullptr);
//    Test::Test();
    int64_t n, k, t;
    cin >> n >> k >> t;
    vector<int64_t> ts(n);
    for (auto &ty: ts) cin >> ty;
    cout << Test::sol2(n, k, t, ts);
//    vector<int64_t> temp = ts;
//    sort(temp.begin(), temp.end());
//    for (auto &ty: ts) ty = distance(temp.begin(), lower_bound(temp.begin(), temp.end(), ty));
//    vector<bool> difs_vec(n, false);
//    for (auto ty: ts) difs_vec[ty] = true;
//    int64_t difs_cnt = count(difs_vec.begin(), difs_vec.end(), true);
//    if (difs_cnt <= t) {
//        cout << 1;
//        return 0;
//    }
    return 0;
}
