#ifdef LOCAL
#define _GLIBCXX_DEBUG
#else
#pragma GCC optimize("O3,unroll-loops,fast-math")
#endif

#include <iostream>
#include <bits/stdc++.h>
#include <ext/pb_ds/tree_policy.hpp>
#include <ext/pb_ds/assoc_container.hpp>

using namespace __gnu_pbds;
using namespace std;

using ll = long long;
using ld = long double;
using pll = pair<ll, ll>;

ll INF = 1e18;
ll MOD = 1e9 + 7;
ld EPS = 1e-7;

#define mp make_pair
#define mt make_tuple
#define ordered_set tree<ll, null_type, less<>, rb_tree_tag, tree_order_statistics_node_update>

random_device rd;
mt19937_64 gen(rd());

struct SegmentTree {

    vector<ll> p;
    ll n;

    SegmentTree(ll n): n(n) {
        p.assign(4 * n, 0);
    }

    ll _get(ll i, ll l, ll r, ll q_l, ll q_r) {
        if (q_l <= l && r <= q_r) return p[i];
        if (q_r <= l || r <= q_l) return 0;

        ll m = (l + r) / 2;
        return _get(2 * i + 1, l, m, q_l, q_r) + _get(2 * i + 2, m, r, q_l, q_r);
    }

    void _set(ll i, ll l, ll r, ll q_i, ll q_x) {
        if (l + 1 == r) {
            p[i] = q_x;
            return;
        }

        ll m = (l + r) / 2;
        if (q_i < m)
            _set(2 * i + 1, l, m, q_i, q_x);
        else
            _set(2 * i + 2, m, r, q_i, q_x);

        p[i] = p[2 * i + 1] + p[2 * i + 2];
    }

    ll get(ll l, ll r) {
        return _get(0, 0, n, l, r);
    }

    void set(ll i, ll x) {
        _set(0, 0, n, i, x);
    }
};

vector<ll> used;
vector<ll> mate;
vector<vector<ll>> g;

bool kuhn(ll v, ll it) {
    if (used[v] == it) return false;

    for (ll to : g[v]) {
        if (mate[to] == -1) {
            mate[to] = v;
            return true;
        }
    }

    for (ll to : g[v]) {
        if (kuhn(mate[to], it)) {
            mate[to] = v;
            return true;
        }
    }

    return false;
}

struct Edge {
    ll to, w;
};

vector<vector<Edge>> gp;

vector<ll> col;
void dfs(ll v) {
    for (Edge e : gp[v]) {
        if (col[e.to] == -1) {
            col[e.to] = 1 - col[v];
            dfs(e.to);
        }
    }
}

int main() {
#ifdef LOCAL
    freopen("inp.txt", "r", stdin);
    freopen("out.txt", "w", stdout);

    clock_t timer = clock();
#else
    ios_base::sync_with_stdio(false);
    cin.tie(nullptr);
    cout.tie(nullptr);
    cout.precision(64);
    cout << fixed;
#endif

    ll n, k, t;
    cin >> n >> k >> t;

    map<ll, vector<ll>> nxt;
    unordered_set<ll> u;

    vector<ll> a(n);
    for (ll i = 0; i < n; i++) {
        cin >> a[i];
        u.insert(a[i]);
        nxt[a[i]].push_back(i);
    }

    if (u.size() <= t) {
        cout << 1;
        return 0;
    }

    for (ll i = 0; i < n; i++) nxt[a[i]].push_back(i + n);

    SegmentTree st(2 * n);

    for (auto &w : nxt) std::reverse(w.second.begin(), w.second.end());

    for (auto w : nxt) st.set(w.second.back(), 1);

    ll mx = 45;

    vector<vector<pair<int, ll>>> up(mx + 1, vector<pair<int, ll>>(n));

    for (ll i = 0; i < n; i++) {
        if (i > 0) {
            st.set(nxt[a[i - 1]].back(), 0);
            nxt[a[i - 1]].pop_back();
            if (!nxt[a[i - 1]].empty()) st.set(nxt[a[i - 1]].back(), 1);
        }

        ll L = 0, R = n;
        while (R - L > 1) {
            ll m = L + (R - L) / 2;
            if (st.get(i, i + m) <= t)
                L = m;
            else
                R = m;
        }

        up[0][i] = {(i + L) % n, (i + L) % n < i};
    }

    for (ll i = 1; i <= mx; i++) {
        for (ll j = 0; j < n; j++) {
            ll v = up[i - 1][j].first;
            ll s = up[i - 1][j].second;

            up[i][j] = {up[i - 1][v].first, up[i - 1][v].second + s};
        }
    }

    auto gt = [&] (ll x) {
        ll v = 0, s = 0;
        for (ll i = mx; i >= 0; i--) {
            if (x >= 1ll << i) {
                x -= 1ll << i;
                s += up[i][v].second;
                v = up[i][v].first;
            }
        }

        return s;
    };

    ll L = 0, R = (1ll << mx);
    while (R - L > 1) {
        ll m = L + (R - L) / 2;
        if (gt(m) <= k - 1)
            L = m;
        else
            R = m;
    }

    cout << R;

#ifdef LOCAL
    cout << "\nExecution time: " << (ld)(clock() - timer) / CLOCKS_PER_SEC;
#endif
}
