#include "bits/stdc++.h"

using namespace std;

using ll = long long;

struct DSU {
    vector<int> p, size;

    DSU(int n) {
        p.resize(n), size.resize(n, 1);
        std::iota(p.begin(), p.end(), 0);
    }

    int get(int v) {
        if (p[v] == v) return v;
        p[v] = get(p[v]);
        return p[v] = get(p[v]);
    }

    bool connected(int u, int v) {
        return get(u) == get(v);
    }

    void join(int u, int v) {
        u = get(u);
        v = get(v);

        if (size[u] > size[v]) swap(u, v);
        p[u] = v;
        size[v] += size[u];
    }
};

void solve(int group) {
    if (group == 1) {
        int n, m1, m2;
        cin >> n >> m1 >> m2;

        vector<vector<int>> g1(n, vector<int>(n));
        vector<vector<int>> g2(n, vector<int>(n));

        for (int i = 0; i < m1; ++i) {
            int u, v;
            cin >> u >> v;

            g1[u - 1][v - 1] = 1;
            g1[v - 1][u - 1] = 1;
        }
        for (int i = 0; i < m2; ++i) {
            int u, v;
            cin >> u >> v;

            g2[u - 1][v - 1] = 1;
            g2[v - 1][u - 1] = 1;
        }

        function<bool(vector<vector<int>> &)> check = [](vector<vector<int>> &g) -> bool {
            int n = g.size();
            vector<int> used(n);

            function<void(int)> dfs = [&](int v) {
                used[v] = true;
                for (int i = 0; i < n; ++i) {
                    if (used[i] || !g[v][i]) continue;
                    dfs(i);
                }
            };
            dfs(0);

            return std::find(used.begin(), used.end(), 0) == used.end();
        };

        int M = n * (n - 1) / 2;
        vector<pair<int, int>> edges;
        for (int i = 0; i < n; ++i) {
            for (int j = i + 1; j < n; ++j) {
                edges.push_back({i, j});
            }
        }
        for (int mask = 0; mask < (1 << M); ++mask) {
            vector<pair<int, int>> remove;
            vector<pair<int, int>> add;
            for (int i = 0; i < M; ++i) {
                if ((mask >> i) & 1) {
                    int u = edges[i].first;
                    int v = edges[i].second;

                    if (g2[u][v]) remove.push_back({u, v});
                    else add.push_back({u, v});
                }
            }

            for (auto [u, v]: add) g2[u][v] ^= 1, g2[v][u] ^= 1;
            for (auto [u, v]: remove) g2[u][v] ^= 1, g2[v][u] ^= 1;
            bool flag = check(g2);
            for (auto [u, v]: add) g1[u][v] ^= 1, g1[v][u] ^= 1;
            for (auto [u, v]: remove) g1[u][v] ^= 1, g1[v][u] ^= 1;
            flag &= check(g1);

            for (auto [u, v]: add) g2[u][v] ^= 1, g2[v][u] ^= 1;
            for (auto [u, v]: remove) g2[u][v] ^= 1, g2[v][u] ^= 1;
            for (auto [u, v]: add) g1[u][v] ^= 1, g1[v][u] ^= 1;
            for (auto [u, v]: remove) g1[u][v] ^= 1, g1[v][u] ^= 1;

            if (add.size() + remove.size() > n) continue;

            if (flag) {
                cout << "Yes" << "\n";
                cout << add.size() + remove.size() << "\n";
                for (auto [u, v]: add) {
                    cout << u + 1 << " " << v + 1 << "\n";
                }
                for (auto [u, v]: remove) {
                    cout << u + 1 << " " << v + 1 << "\n";
                }
                return;
            }
        }

        cout << "No" << "\n";
    } else if (group == 2) {
        int n, m1, m2;
        cin >> n >> m1 >> m2;

        vector<vector<int>> g1(n);
        vector<vector<int>> g2(n);

        DSU ds(n);
        for (int i = 0; i < m1; ++i) {
            int u, v;
            cin >> u >> v;

            g1[u - 1].push_back(v - 1);
            g1[v - 1].push_back(u - 1);
            if (ds.get(u - 1) != ds.get(v - 1)) ds.join(u - 1, v - 1);
        }
        for (int i = 0; i < m2; ++i) {
            int u, v;
            cin >> u >> v;

            g2[u - 1].push_back(v - 1);
            g2[v - 1].push_back(u - 1);
        }

        if (ds.size[ds.get(0)] == n) {
            cout << "Yes" << "\n";
            cout << 0 << "\n";
            return;
        }

        vector<pair<int, int>> moves;
        vector<int> ost(n, n - 1);
        for (int i = 0; i < n; ++i) {
            for (int j = 0; j < n; ++j) {
                if (ost[i] == 1 || ost[j] == 1) continue;
                if (!ds.connected(i, j)) {
                    ost[i]--, ost[j]--;
                    moves.push_back({i + 1, j + 1});
                    ds.join(i, j);
                }
            }
        }
        if (ds.size[ds.get(0)] != n) {
            cout << "No" << "\n";
            return;
        }
        cout << "Yes" << "\n";
        cout << moves.size() << "\n";
        assert(moves.size() <= n);
        for (auto [u, v]: moves) cout << u << " " << v << "\n";
        return;
    } else if (group == 3) {
        int n, m1, m2;
        cin >> n >> m1 >> m2;

        vector<vector<int>> g1(n);
        vector<vector<int>> g2(n, vector<int>(n));

        DSU ds(n);
        for (int i = 0; i < m1; ++i) {
            int u, v;
            cin >> u >> v;

            g1[u - 1].push_back(v - 1);
            g1[v - 1].push_back(u - 1);
            if (ds.get(u - 1) != ds.get(v - 1)) ds.join(u - 1, v - 1);
        }
        for (int i = 0; i < m2; ++i) {
            int u, v;
            cin >> u >> v;

            g2[u - 1][v - 1] = 1;
            g2[v - 1][u - 1] = 1;
        }

        vector<int> used(n);
        vector<vector<int>> comps;
        function<void(int)> dfs = [&](int v) {
            used[v] = true;
            comps.back().push_back(v);

            for (auto u: g1[v]) {
                if (used[u]) continue;
                dfs(u);
            }
        };

        for (int i = 0; i < n; ++i) {
            if (!used[i]) {
                comps.emplace_back();
                dfs(i);
            }
        }

        assert(comps.size()==2);

        int count = 0;
        for (auto u: comps[0]) {
            for (auto v: comps[1]) {
                count += g2[u][v];
                if (!g2[u][v]) {
                    cout << "Yes" << "\n";
                    cout << 1 << "\n";
                    cout << u + 1 << " " << v + 1 << "\n";
                    return;
                }
            }
        }

        for (auto u: comps[0]) {
            for (auto v: comps[1]) {
                if (g2[u][v]) {
                    cout << "Yes" << '\n';
                    cout << 1 << "\n";
                    cout << u + 1 << " " << v + 1 << "\n";
                    return;
                }
            }
        }

        assert(false);
        cout << "No" << "\n";
    }
}

signed main() {
#ifdef LOCAL
    freopen("input.txt", "r", stdin);
    freopen("output.txt", "w", stdout);
#else
    ios_base::sync_with_stdio(false);
    cin.tie(nullptr);
#endif

    int t, g;
    cin >> t >> g;

    while (t--) solve(g);
}