#include <iostream>
#include <vector>
#include <algorithm>
#include <cmath>
#include <set>
#include <map>
#include <array>
#include <string>
#include <iomanip>
#include <queue>

using namespace std;
typedef long long ll;
typedef long double ld;
#define all(a) a.begin(), a.end()
#define fout(a) cout << fixed << setprecision(a)

const ll INF = 1e16;
const ll MOD = 1e9 + 7;
const ld EPS = 1e-9;

void solve();

int main() {
	ios_base::sync_with_stdio(0);
	cin.tie(0);
	ll t, group;
	cin >> t >> group;
	while (t--) {
		solve();
	}
	return 0;
}

struct FED {
	ll a, b;
	ll x() const {
		return (a == b);
	}
	bool operator<(const FED& c) const {
		if ((x() ^ c.x()) == 0) return a < c.a;
		if (c.x() && !x()) return 1;
		if (!c.x() && x()) return 0;
	}
};

struct Dsu {
	vector<ll> p, r;
	vector<FED> fed;
	Dsu(ll n) {
		p.resize(n + 1, 0);
		r.resize(n + 1, 1);
		fed.resize(n + 1, {0, 0});
		for (ll i = 1; i <= n; i++) {
			p[i] = i;
			fed[i] = {i, i};
		}
	}
	ll get(ll v) {
		if (v == p[v]) return v;
		p[v] = get(p[v]);
		return p[v];
	}
	bool unite(ll a, ll b) {
		a = get(a), b = get(b);
		if (a == b) return 0;
		if (r[a] < r[b]) {
			swap(a, b);
		}
		p[b] = a;
		r[a] += r[b];
		fed[a] = {a, b};
		return 1;
	}
};

const ll N = 3e5 + 20;
set<ll> g1[N], g2[N], g2d[N];
char used[N];

void add(ll u, ll v) {
	if (g1[u].find(v) != g1[u].end()) {
		g1[u].erase(v);
		g1[v].erase(u);
	} else {
		g1[u].insert(v);
		g1[v].insert(u);
	}
	if (g2[u].find(v) != g2[u].end()) {
		g2[u].erase(v);
		g2[v].erase(u);
	} else {
		g2[u].insert(v);
		g2[v].insert(u);
	}
}

void maked(ll v) {
	used[v] = 1;
	for (ll u : g2[v]) {
		if (!used[u]) {
			g2d[v].insert(u);
			g2d[u].insert(v);
			maked(u);
		}
	}
}

bool xoch(ll u, ll v) {
	if (g2d[u].find(v) == g2d[u].end()) {
		add(u, v);
		return 1;
	}
	return 0;
}

void solve() {
	ll n, m1, m2;
	cin >> n >> m1 >> m2;
	fill(g1, g1 + n + 1, set<ll>{});
	fill(g2, g2 + n + 1, set<ll>{});
	fill(g2d, g2d + n + 1, set<ll>{});
	fill(used, used + n + 1, 0);
	Dsu d = Dsu(n);
	for (ll i = 0; i < m1; i++) {
		ll u, v;
		cin >> u >> v;
		g1[u].insert(v);
		g1[v].insert(u);
		d.unite(u, v);
	}
	for (ll i = 0; i < m2; i++) {
		ll u, v;
		cin >> u >> v;
		g2[u].insert(v);
		g2[v].insert(u);
	}
	maked(1);
	set<FED> komp;
	for (ll i = 1; i <= n; i++) {
		komp.insert(d.fed[d.get(i)]);
	}
	vector<array<ll, 2>> ans;
	while (komp.size() >= 3) {
		auto i = komp.begin(), j = i, k = i;
		j++;
		k++;
		k++;
		array<ll, 2> a = {i->a, i->b}, b = {j->a, j->b}, c = {k->a, k->b};
		komp.erase(i);
		komp.erase(j);
		komp.erase(k);
		if (true) {
			bool fnd = 0;
			for (ll x : a) {
				for (ll y : b) {
					if (xoch(x, y) && d.unite(x, y)) {
						fnd = 1;
						ans.push_back({x, y});
						break;
					}
				}
				if (fnd) break;
			}
		}
		if (true) {
			bool fnd = 0;
			for (ll x : a) {
				for (ll y : c) {
					if (xoch(x, y) && d.unite(x, y)) {
						fnd = 1;
						ans.push_back({x, y});
						break;
					}
				}
				if (fnd) break;
			}
		}
		if (true) {
			bool fnd = 0;
			for (ll x : c) {
				for (ll y : b) {
					if (xoch(x, y) && d.unite(x, y)) {
						fnd = 1;
						ans.push_back({x, y});
						break;
					}
				}
				if (fnd) break;
			}
		}
		komp.insert(d.fed[d.get(a[0])]);
		komp.insert(d.fed[d.get(b[0])]);
		komp.insert(d.fed[d.get(c[0])]);
	}
	while (komp.size() >= 2) {
		auto i = (komp.begin()), j = i;
		j++;
		if (!i->x() && !j->x()) {
			array<ll, 2> a = {i->a, i->b}, b = {j->a, j->b};
			komp.erase(i);
			komp.erase(j);
			bool fnd = 0;
			for (ll x : a) {
				for (ll y : b) {
					if (xoch(x, y)) {
						fnd = 1;
						ans.push_back({x, y});
						break;
					}
				}
				if (fnd) break;
			}
			d.unite(a[0], b[0]);
			komp.insert(d.fed[d.get(a[0])]);
		} else {
			break;
		}
	}
	if (komp.size() > 1) {
		auto j = (komp.begin());
		j++;
		ll b = j->a;
		vector<ll> a;
		for (ll i = 1; i <= n; i++) {
			if (i != b) a.push_back(i);
		}
		for (ll x : a) {
			if (xoch(x, b)) {
				ans.push_back({x, b});
				goto yes;
			}
		}
		for (ll x : a) {
			for (ll y : g2[x]) {
				if (y != b) {
					ans.push_back({x, b});
					goto yes;
				}
			}
		}
		for (ll i = 0; i < a.size(); i++) {
			ll j = (i + 1) % a.size();
			if (g1[a[i]].find(a[j]) == g1[a[i]].end()) {
				ans.push_back({a[i], a[j]});
				ans.push_back({a[i], b});
				goto yes;
			}
		}
		if (a.size() > 2) {
			ans.push_back({a[0], a[1]});
			ans.push_back({a[0], b});
			goto yes;
		} else {
			cout << "No\n";
			return;
		}

	}
	yes:
	cout << "Yes\n";
	cout << ans.size() << "\n";
	for (auto& p : ans) cout << p[0] << " " << p[1] << "\n";
}
/*
3 0
4 2 3
1 2
3 4
1 3
1 4
2 3


*/