#include <iostream>
#include <vector>
#include <set>
#include <queue>
#include <map>
#include <random>
#include <chrono>
#include <assert.h>
#include <stack>
#include <numeric>

#define pb push_back
#define all(x) (x).begin(), (x).end()

using namespace std;
using ll = long long;
using ii = pair<ll, ll>;
using uid = uniform_int_distribution<int>;
using lst = vector<vector<int>>;

const int type = 1;
const int MAXN = 2e5 + 1;
const int LOGN = 17;
const int MOD = 1e9 + 7;

class SegmentTree {
private:
	int n;
	int size;
	vector<int> t;

	void build(int v, int tl, int tr) {
		if (tl == tr) {
			return;
		}
		int tm = (tl + tr) >> 1;
		build(v << 1, tl, tm);
		build(v << 1 | 1, tm + 1, tr);
	}

public:

	SegmentTree(const int _n)
		:n(_n)
		, size(n << 2)
		, t(size) {
		build(1, 0, n);
	}
};

struct HashTable {
	static const int TABLE_SIZE = 1e7;
	static const int MAX_ELEMENTS = 1e5;
	int head[TABLE_SIZE];
	int next[MAX_ELEMENTS];
	int val[MAX_ELEMENTS];
	int itemCount = 0;

	void update() {
		for (int i = 0;i < MAX_ELEMENTS;i++) {
			next[i] = -1;
		}
		for (int i = 0;i < TABLE_SIZE;i++) {
			head[i] = -1;
		}
		return;
	}

	HashTable() {
		update();
	}

	void add(ll hash) {
		ll h = hash % TABLE_SIZE;
		for (int i = head[h]; i != -1;i = next[i]) {
			if (val[i] == hash) {
				return;
			}
		}
		next[++itemCount] = head[h];
		head[h] = itemCount;
		val[itemCount] = hash;
		return;
	}
};

bool isPrime(ll x) {
	for (int i = 2;i <= x / i;i++) {
		if (x % i == 0) {
			return false;
		}
 	}
	return (x > 1);
}

void genPrime(mt19937& gen, ll& x) {
	while (1) {
		x = uid(1e9, 2e9)(gen);
		if (isPrime(x)) return;
	}
}

int _lg(ll x) {
	int ans = 0;
	for (int i = 0;i < 63;i++) {
		if (x & (1 << i)) ans = i;
	}
	return ans;
}

int _builtin_popcount(ll x) {
	int ans = 0;
	for (int i = 0;i < 63;i++) {
		if (x & (1 << i)) ans++;
	}
	return ans;
}

void solve2() {

}

int getRoot(int v, vector<int>& p) {
	int v0 = v;
	while (p[v] >= 0) {
		v = p[v];
	}
	int root = v;
	v = v0;
	while (p[v] >= 0) {
		int par = p[v];
		p[v] = root;
		v = par;
	}
	return root;
}

bool join(int a, int b, vector<int>& p) {
	a = getRoot(a, p), b = getRoot(b, p);
	if (a == b) return false;
	if (p[a] > p[b]) swap(a, b);
	p[a] += p[b];
	p[b] = a;
	return true;
}

bool tryjoin(int a, int b, vector<int>& p) {  // можно ли соединить?
	a = getRoot(a, p), b = getRoot(b, p);
	if (a == b) return false;
	return true;
}

int tin[MAXN], up[MAXN];

void findBridges(int v, lst& adj, set<ii>& bridges, int t = 0, int p = -1) {
	tin[v] = up[v] = ++t;
	for (int u : adj[v]) {
		if (u == p) continue;
		if (tin[u] == 0) findBridges(u, adj, bridges, t, v);
		if (tin[u] > tin[v]) up[v] = min(up[u], up[v]);
		if (tin[u] < tin[v]) up[v] = min(up[v], tin[u]); 
	}
	if (p != -1 && tin[v] == up[v]) {
		bridges.insert({min(p, v), max(p,v)});
	}
}

void colorComp(int v, lst& adj, vector<int>& clr, int color) {
	clr[v] = color;
	for (int u : adj[v]) {
		if (clr[u] == -1) colorComp(u, adj, clr, color);
	}
	return;
}

struct Edge {
	int v, u;
};

struct ColEdge {
	int col1, col2, v1, v2;
};

void solve(int group) {
	int nV, nE1, nE2;
	cin >> nV >> nE1 >> nE2;
	vector<int> p(nV, -1);
	vector<vector<int>> adj1(nV);
	set<ii> edges1;
	for (int e = 0;e < nE1;e++) {
		int v, u;
		cin >> v >> u; v--;u--;
		adj1[v].pb(u);
		adj1[u].pb(v);
		if (v > u) swap(v, u);
		edges1.insert({v, u});
	}
	lst adj2(nV);
	set<ii> edges2;
	for (int e = 0; e < nE2;e++) {
		int v, u;
		cin >> v >> u; v--; u--;
		adj2[u].pb(v);
		adj2[v].pb(u);
		edges2.insert({ v,u });
	}
	if (group <= 7) {
		for (int i = 0;i < nV;i++) {
			for (int j = i + 1;j < nV;j++) {
				if (edges1.find({ i,j }) == edges1.end() && edges2.find({ i,j }) == edges2.end()) {
					edges1.insert({ i, j });
					edges2.insert({ i, j });
					adj1[i].pb(j);
					adj1[j].pb(i);
					adj2[i].pb(j);
					adj2[j].pb(i);
				}
			}
		}  // добавили все недостающие в обоих графах рёбра, теперь можно просто найти мосты
		vector<int> clr(nV, -1);
		int color = 0;
		for (int i = 0;i < nV;i++) {
			if (clr[i] == -1) {
				colorComp(i, adj1, clr, color);
				color++;
			}
		}  // покрасили компоненты, теперь нужно выявить все рёбра, между компонентами, которые отсутствуют
		set<ii> bridges;
		findBridges(0, adj2, bridges);  // если все цвета можно соединить друг с другом, то всё ок
		vector<ColEdge> edgesColor(color + 1);
		for (int i = 0; i < nV;i++) {
			for (int j = i + 1; j < nV;j++) {
				if (clr[i] != clr[j] && bridges.find({i,j}) == bridges.end()) {  // если два цвета не совпадают, то это замечательно!
					edgesColor.push_back({ clr[i], clr[j], i, j });  // но кстати говоря, если это не мосты	
				}
			}
		}
		vector<ii> ans;
		vector<int> pCol(color + 1, -1);
		for (auto& [col1,col2, v1,v2] : edgesColor) {
			if (join(col1, col2, pCol)) {
				ans.pb({v1,v2});
			}
		}
		int counter = 0;
		for (int i = 0;i < color;i++) {
			if (p[i] < 0) counter++;
		}
		if (counter > 1 || (nV == 3 && nE1 == 0)) {
			cout << "No\n";
			return;
		}
		cout << "Yes\n";
		cout << ans.size() << "\n";
		for (auto [u, v] : ans) {
			cout << u + 1 << " " << v + 1;
		}
		return;
	}
}

int main() {
	cin.tie(0)->ios::sync_with_stdio(false);
	//freopen("input.txt", "r", stdin);
	//freopen("output.txt", "w", stdout);
	int nT = 1; int g;
	if (type) cin >> nT >> g;
	for (int t = 0;t < nT;t++) {
		solve(g);
		if (t != nT - 1) std::cout << "\n";
	}
	return 0;
}