#define _CRT_SECURE_NO_WARNINGS

#include <iostream>
#include <iomanip>
#include <algorithm>
#include <cmath>
#include <vector>
#include <string>
#include <random>
#include <set>

using namespace std;
using ll = long long;
using ld = long double;
mt19937 rd(2346927423);
const ll INF = 2e18;
const int SZ = 1e6 + 1;

struct DDnode {
    DDnode* l;
    DDnode* r;
    ll val, prior;
    DDnode(ll val = 0) : val(val) {
        l = nullptr;
        r = nullptr;
        prior = rd();
    }
};
DDnode* mer(DDnode* l, DDnode* r) {
    if (l == nullptr)
        return r;
    if (r == nullptr)
        return l;

    if (l->prior > r->prior) {
        l->r = mer(l->r, r);
        return l;
    }
    else {
        r->l = mer(l, r->l);
        return l;
    }
}
pair<DDnode*, DDnode*> split(DDnode* v, ll x) {
    return { v, v };
}

struct pt {
    ll x, y;
    pt(ll x = 0, ll y = 0) : x(x), y(y) {}
};
pt operator+(pt a, pt b) {
    return pt(a.x + b.x, a.y + b.y);
}
pt operator-(pt a, pt b) {
    return pt(a.x - b.x, a.y - b.y);
}
ll operator*(pt a, pt b) {
    return a.x * b.x + a.y * b.y;
}
ll operator%(pt a, pt b) {
    return a.x * b.y - a.y * b.x;
}

vector<int> pr(SZ), sz(SZ);
set<int> g1[SZ], g2[SZ];
vector<pair<int, int>> ans;

int get_root(int v) {
    if (pr[v] == v)
        return v;
    return pr[v] = get_root(pr[v]);
}

bool mer(int u, int v) {
    u = get_root(u);
    v = get_root(v);
    if (u == v)
        return 0;
    if (sz[u] < sz[v]) {
        pr[u] = v;
        sz[v] += sz[u];
    }
    else {
        pr[v] = u;
        sz[u] += sz[v];
    }
    return 1;
}

vector<bool> used(SZ, 0);
vector<bool> col(SZ, 0);
vector<int> rnk(SZ, 0);
bool dfs(int v = 1, int p = 1) {
    used[v] = 1;
    if (p != 1 && get_root(1) != get_root(v)) {
        //f1[1][v] = !f1[1][v];
        //f1[v][1] = !f1[v][1];
        //f2[1][v] = !f2[1][v];
        //f2[v][1] = !f2[v][1];
        ans.push_back({ 1, v });
        mer(1, v);
    }

    for (auto u : g2[v]) {
        if (used[u])
            continue;
        dfs(u, v);
    }

    if (v == 1) {
        int vert = -1;
        for (auto u : g2[v]) {
            if (vert == -1 || get_root(u) == get_root(vert)) {
                vert = u;
            }
            else {
                //f1[u][vert] = !f1[u][vert];
                //f1[vert][u] = !f1[vert][u];
                //f2[u][vert] = !f2[u][vert];
                //f2[vert][u] = !f2[vert][u];
                ans.push_back({ vert, u });
                mer(u, vert);
            }
        }
    }
    return 1;
}

int get_most(int v = 1, int p = 0) {
    used[v] = 1;
    rnk[v] = rnk[p] + 1;
    int up1 = rnk[v];
    for (auto u : g2[v]) {
        if (used[u] && u != p) {
            up1 = min(up1, rnk[u]);
            continue;
        }
        if (used[u])
            continue;
        up1 = min(up1, get_most(u, v));
        if (up1 <= -1)
            return up1;
    }
    if (up1 < rnk[v] && col[p] != col[v]) {
        ans.push_back({p, v});
        return -1;
    }
    return up1;
}

int main()
{
#ifdef _DEBUG
    freopen("input.txt", "r", stdin);
#endif
    ios_base::sync_with_stdio(false);
    cin.tie(0); cout.tie(0);
    int t, G;
    cin >> t >> G;
    while (t > 0) {
        t--;
        ans.clear();
        int n, m1, m2;
        cin >> n >> m1 >> m2;
        for (int i = 1; i <= n; i++) {
            pr[i] = i;
            sz[i] = 1;
            g1[i].clear();
            g2[i].clear();
            used[i] = 0;
        }

        for (int i = 0; i < m1; i++) {
            int u, v;
            cin >> u >> v;
            g1[u].insert(v);
            g1[v].insert(u);
            mer(u, v);
        }
        for (int i = 0; i < m2; i++) {
            int u, v;
            cin >> u >> v;
            g2[u].insert(v);
            g2[v].insert(u);
        }

        dfs();

        for (auto p : ans) {
            int u = p.first, v = p.second;
            if (g1[u].find(v) == g1[u].end()) {
                g1[u].insert(v);
            }
            else {
                g1[u].erase(v);
            }
            if (g1[v].find(u) == g1[v].end()) {
                g1[v].insert(u);
            }
            else {
                g1[v].erase(u);
            }
        }

        vector<int> col1, col2;
        for (int i = 1; i <= n; i++) {
            used[i] = 0;
            if (get_root(i) == get_root(1)) {
                col[i] = 0;
                col1.push_back(i);
            }
            else {
                col[i] = 1;
                col2.push_back(i);
            }
        }

        bool tf = (col1.size() == n);
        for (auto u : col1) {
            for (auto v : col2) {
                if (g2[u].find(v) == g2[u].end()) {
                    tf = 1;
                    ans.push_back({ u, v });
                    break;
                }
            }
            if (tf)
                break;
        }

        if (!tf) {
            if (get_most() != -1) {
                cout << "No" << endl;
                continue;
            }
        }

        cout << "Yes\n";
        for (auto p : ans) {
            cout << p.first << ' ' << p.second << endl;
        }
    }
}