#include <bits/stdc++.h>
#define int long long

using namespace std;
using ll = long long;
using db = double;

struct Edge {
    int u, v;
};

bool operator==(const Edge& a, const Edge& b) {
    return a.u == b.u && a.v == b.v;
}

struct Input {
    int n, m1, m2;
    vector<Edge> e1, e2;
};

Input input() {
    int n, m1, m2; cin >> n >> m1 >> m2;
    vector<Edge> e1(m1), e2(m2);
    for (int i = 0; i < m1; ++i) {
        int u, v; cin >> u >> v; u--; v--;
        e1[i] = {u, v};
    }
    for (int i = 0; i < m2; ++i) {
        int u, v; cin >> u >> v; u--; v--;
        e2[i] = {u, v};
    }
    return {n, m1, m2, e1, e2};
}

void dfs(int v, vector<int>& used, const vector<vector<int>>& g, vector<vector<int>>& g2) {
    used[v] = 1;
    for (auto u : g[v]) {
        if (used[u]) continue;
        pair<int, int> tmp = {v, u};
        if (tmp.first > tmp.second) swap(tmp.first, tmp.second);
        g2[v][u] = g2[u][v] = 0;
        dfs(u, used, g, g2);
    }
}

mt19937 rnd(57);

void dfs2(int v, vector<int>& used, const vector<vector<int>>& g, vector<Edge>& e) {
    used[v] = 1;
    vector<int> p(g.size());
    iota(p.begin(), p.end(), 0);
    shuffle(p.begin(), p.end(), rnd);
    for (int ind = 0; ind < (int)g.size(); ++ind) {
        int i = p[ind];
        if (!g[v][i]) continue;
        if (used[i]) continue;
        e.push_back({v, i});
        dfs2(i, used, g, e);
    }
}

pair<int, vector<Edge>> kukarekuSolve(Input inp) {
    auto [n, m1, m2, e1, e2] = std::move(inp);
    vector<vector<int>> g1(n);
    for (auto [u, v] : e2) {
        g1[u].push_back(v);
        g1[v].push_back(u);
    }
    for (int t = 0; t < 100; ++t) {
        for (int i = 0; i < n; ++i) {
            shuffle(g1[i].begin(), g1[i].end(), rnd);
        }
        vector<int> used(n);
        set<pair<int, int>> was;
        for (auto [u, v]: e1) {
            if (u > v) swap(u, v);
            was.insert({u, v});
        }
        vector<vector<int>> g2(n, vector<int>(n, 1));
        dfs(0, used, g1, g2);
        used.assign(n, 0);
        vector<Edge> eNeed;
        dfs2(0, used, g2, eNeed);
        if ((int) eNeed.size() < n - 1) continue;
        vector<Edge> ans;
        for (auto [u, v]: eNeed) {
            if (u > v) swap(u, v);
            if (was.find({u, v}) == was.end()) {
                ans.push_back({u, v});
            }
        }
        return {1, ans};
    }
    return {0, {}};
}

void output(const pair<int, vector<Edge>>& e) {
    if (e.first == 0) {
        cout << "No\n";
        return;
    }
    cout << "Yes\n";
    cout << e.second.size() << '\n';
    for (auto [u, v] : e.second) {
        cout << u + 1 << ' ' << v + 1 << '\n';
    }
}

struct DSU {
    vector<int> p;

    DSU(int n) {
        p.resize(n);
        iota(p.begin(), p.end(), 0);
    }

    int getRoot(int v) {
        if (p[v] == v) return v;
        p[v] = getRoot(p[v]);
        return p[v];
    }

    void unio(int a, int b) {
        a = getRoot(a);
        b = getRoot(b);
        p[a] = b;
    }
};

bool isCon(int n, vector<Edge> e) {
    DSU d(n);
    int cnt = n;
    for (auto [u, v] : e) {
        if (d.getRoot(u) != d.getRoot(v)) {
            cnt--;
            d.unio(u, v);
        }
    }
    return cnt == 1;
}

pair<int, vector<Edge>> stupidSolve(Input inp) {
    auto [n, m1, m2, e1, e2] = std::move(inp);
    for (int i = 0; i < m1; ++i) {
        if (e1[i].u > e1[i].v) {
            swap(e1[i].u, e1[i].v);
        }
    }
    for (int i = 0; i < m2; ++i) {
        if (e2[i].u > e2[i].v) {
            swap(e2[i].u, e2[i].v);
        }
    }
    vector<Edge> e;
    for (int u = 0; u < n; ++u) {
        for (int v = u + 1; v < n; ++v) {
            e.push_back({u, v});
        }
    }
    int m = (int)e.size();
    for (int mask = 0; mask < (1 << m); ++mask) {
        vector<Edge> eAdd;
        for (int i = 0; i < m; ++i) {
            if (mask & (1 << i)) {
                eAdd.push_back(e[i]);
            }
            if (eAdd.size() > n) break;
        }
        if (eAdd.size() > n) continue;
        vector<int> p(eAdd.size());
        iota(p.begin(), p.end(), 0);
        do {
            vector<Edge> ans;
            vector<Edge> ne1 = e1, ne2 = e2;
            for (int x = 0; x < (int)eAdd.size(); ++x) {
                auto ed = eAdd[p[x]];
                auto it = find(ne1.begin(), ne1.end(), ed);
                if (it == ne1.end()) {
                    ne1.push_back(ed);
                } else {
                    ne1.erase(it);
                }
                it = find(ne2.begin(), ne2.end(), ed);
                if (it == ne2.end()) {
                    ne2.push_back(ed);
                } else {
                    ne2.erase(it);
                }
                ans.push_back(ed);
                if (!isCon(n, ne2)) break;
                if (isCon(n, ne1)) {
                    return {1, ans};
                }
            }
        } while (next_permutation(p.begin(), p.end()));
    }
    return {0, {}};
}

void run() {
    int t, G; cin >> t >> G;
    for (int i = 0; i < t; ++i) {
        auto test = input();
        if (test.n <= 7) {
            output(stupidSolve(test));
        } else {
            output(kukarekuSolve(test));
        }
    }
}

signed main() {
    ios_base::sync_with_stdio(false);
    cin.tie(nullptr); cout.tie(nullptr);
#ifdef LOCAL
    freopen("input.txt", "r", stdin);
    freopen("output.txt", "w", stdout);
#endif
    run();
    return 0;
}
