#define _CRT_SECURE_NO_WARNINGS
#include <iostream>
#include <algorithm>
#include <string>
#include <vector>
#include <set>
#include <iomanip>
#include <numeric>
#include <fstream>
#include <unordered_map>
#include <random>

using namespace std;

#define int long long

void solve();

signed main() {
#ifdef _DEBUG
	freopen("input.txt", "r", stdin);
#else
	ios_base::sync_with_stdio(0);
	cin.tie(0);
#endif

	solve();
}

using ll = long long;
using pint = pair<int, int>;

random_device rd;
mt19937 rnd(rd());

struct Edge {
	int a, b;
	bool most, used;
};

struct DSU {
	vector<int> p, sz;

	DSU(int n) {
		p.resize(n + 1);
		sz.resize(n + 1, 1);
		iota(p.begin(), p.end(), 0);
	}

	int get(int x) {
		if (x == p[x]) {
			return x;
		}
		return p[x] = get(p[x]);
	}

	void unite(int a, int b) {
		a = get(a);
		b = get(b);
		if (a == b) {
			return;
		}
		if (sz[a] > sz[b]) {
			swap(a, b);
		}
		sz[b] += sz[a];
		p[a] = b;
	}
};

void dfs(int v, int p, vector<int>& used, vector<Edge>& e, vector<vector<pint>>& g, vector<int>& dp, vector<int>& h) {
	h[v] = dp[v] = h[p] + 1;
	used[v] = true;
	for (auto& [u, id] : g[v]) {
		if (e[id].used) {
			continue;
		}

		e[id].used = true;
		if (used[u]) {
			dp[v] = min(dp[v], h[u]);
		}
		else {
			dfs(u, v, used, e, g, dp, h);
			dp[v] = min(dp[v], dp[u]);
			if (dp[u] > h[v]) {
				e[id].most = true;
			}
		}
	}
}

void solve() {
	int t, g;
	cin >> t >> g;
	while (t--) {
		int n, m1, m2;
		cin >> n >> m1 >> m2;
		vector<Edge> e1, e2;
		vector<int> used1(n + 1), dp1(n + 1), h1(n + 1);
		vector<int> used2(n + 1), dp2(n + 1), h2(n + 1);
		vector<vector<pint>> g1(n + 1), g2(n + 1);
		DSU d1(n + 1), d2(n + 1);
		for (int i = 0; i < m1; i++) {
			int v, u;
			cin >> v >> u;
			d1.unite(v, u);
			e1.push_back({ v, u, 0, 0 });
			g1[v].push_back({ u, i });
			g1[u].push_back({ v, i });
		}
		for (int i = 0; i < m2; i++) {
			int a, b;
			cin >> a >> b;
			d2.unite(a, b);
			e2.push_back({ a, b, 0, 0 });
			g2[a].push_back({ b, i });
			g2[b].push_back({ a, i });
		}

		vector<pint> ans;
		if (g == 2) {
			set<pint> r;
			for (auto& x : e2) {
				r.insert({ x.a, x.b });
			}
			for (int i = 1; i <= n; i++) {
				for (int j = i + 1; j <= n; j++) {
					if (d1.get(i) != d1.get(j) && r.find({i, j}) == r.end()) {
						ans.push_back({ i, j });
						d1.unite(i, j);
					}
				}
			}
			if (d1.sz[1] == n) {
				cout << "Yes\n";
				cout << ans.size() << '\n';
				for (auto& [a, b] : ans) {
					cout << a << ' ' << b << '\n';
				}
			}
		}
		if (g == 3) {
			dfs(1, 0, used1, e1, g1, dp1, h1);
			dfs(1, 0, used2, e2, g2, dp2, h2);

			set<pint> s_most;
			for (auto& x : e2) {
				if (!x.most) {
					if (d1.get(x.a) != d1.get(x.b)) {
						ans.push_back({ x.a, x.b });
						break;
					}
				}
				else {
					s_most.insert({ x.a, x.b });
				}
			}

			if (ans.empty()) {
				for (int i = 1; i <= n; i++) {
					for (int j = i + 1; j <= n; j++) {
						if (d1.get(i) != d1.get(j) && s_most.find({i, j}) == s_most.end()) {
							ans.push_back({ i, j });
							break;
						}
					}
					if (!ans.empty()) break;
				}
			}
			if (ans.empty()) {
				cout << "No\n";
			}
			else {
				cout << "Yes\n";
				cout << 1 << '\n';
				cout << ans[0].first << ' ' << ans[0].second << '\n';
			}
		}
	}
}