#include <iostream>
#include <random>
#include <bitset>
#include <vector>
using namespace std;

const int MAXN = 3001;
bitset<MAXN> edg0[MAXN];
bitset<MAXN> edg1[MAXN];

void input(int n, int m1, int m2) {
    for (int i = 0; i <= n; ++i)
        edg0[i] = edg1[i] = 0;
    for (int ei = 0; ei < m1; ++ei) {
        int u, v;
        cin >> u >> v;
        --u, --v;
        edg0[u][v] = edg0[v][u] = 1;
    }
    for (int ei = 0; ei < m2; ++ei) {
        int u, v;
        cin >> u >> v;
        --u, --v;
        edg1[u][v] = edg1[v][u] = 1;
    }
}

struct dsu {
    vector<int> rt;
    vector<int> sz;

    void resize(int n) {
        sz.assign(n, 1);
        rt.resize(n);
        for (int i = 0; i < n; ++i)
            rt[i] = i;
    }

    int gst(int i) {
        if (rt[i] == i)
            return i;
        return rt[i] = gst(rt[i]);
    }

    int gsz(int i) {
        return sz[gst(i)];
    }

    void unite(int i, int j) {
        i = gst(i);
        j = gst(j);
        if (i == j)
            return;
        if (sz[i] < sz[j])
            swap(i, j);
        rt[j] = i;
        sz[i] += sz[j];
    }
};

void flip_edge(int i, int j) {
    edg0[i][j] = edg0[j][i] = !edg0[i][j];
    edg1[i][j] = edg1[j][i] = !edg1[i][j];
}

inline pair<int, int> random_edge(mt19937& rng, int n) {
    int u = rng() % n;
    int v = rng() % n;
    while (u == v)
        v = rng() % n;
    return {u, v};
}

vector<pair<int, int>> solve_random(int n, mt19937& rng, int sz) {
    vector<pair<int, int>> flip_history;
    dsu su2;
    int tmr = 0;
    while (flip_history.size() + 1 < n && tmr < 3 * n) {
        pair<int, int> e = random_edge(rng, n);
        flip_edge(e.first, e.second);

        su2.resize(n);
        for (int i = 0; i < n; ++i) for (int j = edg1[i]._Find_first(); j < n; j = edg1[i]._Find_next(j))
            su2.unite(i, j);

        if (su2.gsz(0) < n) {
            ++tmr;
            flip_edge(e.first, e.second);
            continue;
        }
        flip_history.push_back(e);

        su2.resize(n);
        for (int i = 0; i < n; ++i) for (int j = edg0[i]._Find_first(); j < n; j = edg0[i]._Find_next(j))
            su2.unite(i, j);
        if (su2.gsz(0) == n) {
            for (auto e0 : flip_history)
                flip_edge(e0.first, e0.second);
            return flip_history;
        }
    }
    for (auto e : flip_history)
        flip_edge(e.first, e.second);
    return {};
}

vector<pair<int, int>> solve(int n, int m1, int m2) {
    if (n <= 100) {
        mt19937 rng(177011);
        int t = 50000 / n / n + 10;
        while (t--) {
            vector<pair<int, int>> ans = solve_random(n, rng, n);
            if (ans.size())
                return ans;
        }
        return {{-1, 0}};
    }
    vector<pair<int, int>> eflip;
    dsu su;
    su.resize(n);
    for (int i = 0; i < n; ++i) for (int j = 0; j < n; ++j) if (edg0[i][j])
        su.unite(i, j);
    for (int i = 0; i < n; ++i) for (int j = 0; j < n; ++j) if (su.gst(i) != su.gst(j)) {
        if (!edg1[i][j]) {
            eflip.push_back({i, j});
            flip_edge(i, j);
            su.unite(i, j);
        }
    }
    if (su.gsz(0) == n)
        return eflip;
    if (n == 3)
        return {{-1, 0}};
    mt19937 rng(177013);
    int t = 50000 / n / n + 10;
    while (t--) {
        vector<pair<int, int>> ans = solve_random(n, rng, n - eflip.size());
        if (ans.size()) {
            for (auto e : ans)
                eflip.push_back(e);
            return eflip;
        }
    }
    return {{-1, 0}};
}


signed main() {
    int t, g;
    cin >> t >> g;
    while (t--) {
        int n, m1, m2;
        cin >> n >> m1 >> m2;
        input(n, m1, m2);
        vector<pair<int, int>> ans;
        ans = solve(n, m1, m2);
        if (ans.size() && ans[0].first == -1) {
            cout << "No\n";
        } else {
            cout << "Yes\n";
            cout << ans.size() << '\n';
            for (auto p : ans)
                cout << p.first + 1 << ' ' << p.second + 1 << '\n';
        }
    }
}
