#include <bits/stdc++.h>

using namespace std;
using ll = long long;
const int inf = 1e9 + 7;
const ll infll = 1e18;

template<typename T>
istream &operator>>(istream &is, vector<T> &a) {
    for (auto &i : a) {
        is >> i;
    }
    return is;
}

struct DSU {
    int comp;
    vector<int> p;

    DSU() = default;
    DSU(int n) {
        p.resize(n);
        iota(p.begin(), p.end(), 0);
        comp = n;
    }

    int root(int v) {
        if (p[v] == v) {
            return v;
        }
        return p[v] = root(p[v]);
    }

    bool same(int u, int v) {
        return root(u) == root(v);
    }

    bool unite(int u, int v) {
        u = root(u);
        v = root(v);
        if (u == v) {
            return false;
        }
        p[v] = u;
        --comp;
        return true;
    }
};

void solve() {
    int n, m1, m2;
    cin >> n >> m1 >> m2;
    if (m2 == n * 1ll * (n - 1) / 2) {
        DSU d(n);
        for (int i = 0; i < m1; ++i) {
            int u, v;
            cin >> u >> v;
            --u, --v;
            d.unite(u, v);
        }
        for (int i = 0; i < m2; ++i) {
            int u, v;
            cin >> u >> v;
            --u, --v;
        }
        if (n == 3 && m1 == 0) {
            cout << "No\n";
            return;
        }
        vector<pair<int, int>> ans;
        for (int i = 0; i < n - 1; ++i) {
            if (!d.same(i, i + 1)) {
                ans.push_back({i, i + 1});
                d.unite(i, i + 1);
            }
        }
        cout << "Yes\n";
        cout << ans.size() << "\n";
        for (auto [u, v]: ans) {
            cout << u + 1 << " " << v + 1 << "\n";
        }
        return;
    }
    vector<vector<int>> g1(n, vector<int>(n)), g2(n, vector<int>(n));
    for (int i = 0; i < m1; ++i) {
        int u, v;
        cin >> u >> v;
        --u, --v;
        g1[u][v] = 1;
        g1[v][u] = 1;
    }
    for (int i = 0; i < m2; ++i) {
        int u, v;
        cin >> u >> v;
        --u, --v;
        g2[u][v] = 1;
        g2[v][u] = 1;
    }
    vector<pair<int, int>> g;
    for (int i = 0; i < n; ++i) {
        for (int j = i + 1; j < n; ++j) {
            g.push_back({i, j});
        }
    }
    {
        vector<bool> used(n);
        auto dfs = [&](auto dfs, const vector<vector<int>> &g, int v) -> void {
            used[v] = 1;
            for (auto u: g[v]) {
                if (!used[u]) {
                    dfs(dfs, g, u);
                }
            }
        };
        vector<vector<int>> gg1(n);
        for (int i = 0; i < n; ++i) {
            for (int j = 0; j < n; ++j) {
                if (g1[i][j]) {
                    gg1[i].push_back(j);
                }
            }
        }
        used.assign(n, false);
        dfs(dfs, gg1, 0);
        if (count(used.begin(), used.end(), false) == 0) {
            cout << "Yes\n";
            cout << 0 << "\n";
            return;
        }
    }
    vector<int> p(g.size());
    iota(p.begin(), p.end(), 0);
    vector<pair<int, int>> best;
    do {
        auto G1 = g1, G2 = g2;
        for (int k = 0; k < p.size(); ++k) {
            auto [u, v] = g[p[k]];
            G1[u][v] ^= 1;
            G1[v][u] ^= 1;
            G2[u][v] ^= 1;
            G2[v][u] ^= 1;
            DSU d1(n), d2(n);
            for (int i = 0; i < n; ++i) {
                for (int j = 0; j < n; ++j) {
                    if (G1[i][j]) {
                        d1.unite(i, j);
                    }
                    if (G2[i][j]) {
                        d2.unite(i, j);
                    }
                }
            }
            bool ok1 = (d1.comp == 1);
            bool ok2 = (d2.comp == 1);
            if (!ok2) {
                break;
            }
            if (ok1) {
                vector<pair<int, int>> cur;
                for (int i = 0; i <= k; ++i) {
                    auto [u, v] = g[p[i]];
                    cur.push_back({u + 1, v + 1});
                }
                if (best.empty() || cur.size() < best.size()) {
                    best = cur;
                }
                break;
            }
        }
    } while (next_permutation(p.begin(), p.end()));
    if (best.empty() || best.size() > n) {
        cout << "No\n";
        return;
    }
    cout << "Yes\n";
    cout << best.size() << "\n";
    for (auto [u, v] : best) {
        cout << u << " " << v << "\n";
    }
}

int32_t main() {
#ifdef LOCAL
    freopen("D:\\input.txt", "r", stdin);
#else
    ios::sync_with_stdio(false);
    cin.tie(nullptr);
#endif
    int t, _;
    cin >> t >> _;
    while (t--) {
        solve();
    }
    return 0;
}