#include <iostream>
#include <cstdint>
#include <vector>
#include <random>
#include <set>

using namespace std;

int64_t good_sol(int64_t n, const vector<int64_t> &d, const vector<int64_t> &l, const vector<int64_t> &r,
                 int64_t a0, int64_t b0) {
    vector<vector<pair<int64_t, int64_t>>> tr(n + 1);
    tr[0].emplace_back(a0, b0);
    for (int32_t i = 0; i < n; ++i) {
        tr[i + 1].reserve(tr[i].size() * 2);
        for (auto [a, b]: tr[i]) {
            tr[i + 1].emplace_back(min(a + d[i], l[i]), b + d[i]);
            tr[i + 1].emplace_back(a + d[i], min(b + d[i], r[i]));
        }
    }
    int64_t res = 0;
    for (auto [a, b]: tr[n]) res = max(res, a + b);
    return res;
}

int64_t my_sol(int64_t n, const vector<int64_t> &d, const vector<int64_t> &l, const vector<int64_t> &r,
               int64_t a0, int64_t b0) {
    auto cmp = [](const pair<int64_t, int64_t> &a, const pair<int64_t, int64_t> &b) {
        if (a.first + b.first != a.second + b.second) return a.first + b.first < a.second + b.second;
        if (a.first != b.first) return a.first < b.first;
        return a.second < b.second;
    };
    set<pair<int64_t, int64_t>, decltype(cmp)> s1, s2;
    s1.emplace(a0, b0);
    int64_t pre_max_one_param = 0, max_one_param = 0;
    for (int32_t i = 0; i < n; ++i) {
        for (auto [a, b]: s1) {
            if (min(a + d[i], l[i]) + b + d[i] > pre_max_one_param) {
                s2.emplace(min(a + d[i], l[i]), b + d[i]);
                if (min(a + d[i], l[i]) > max_one_param) max_one_param = min(a + d[i], l[i]);
                if (b + d[i] > max_one_param) max_one_param = b + d[i];
            }
            if (a + d[i] + min(b + d[i], r[i]) > pre_max_one_param) {
                s2.emplace(a + d[i], min(b + d[i], r[i]));
                if (a + d[i] > max_one_param) max_one_param = a + d[i];
                if (min(b + d[i], r[i]) > max_one_param) max_one_param = min(b + d[i], r[i]);
            }
        }
        pre_max_one_param = max_one_param, max_one_param = 0;
        swap(s1, s2);
        s2.clear();
    }
    int64_t res = max(a0, b0) + accumulate(d.begin(), d.end(), 0ll);
    for (auto [a, b]: s1) if (a + b > res) res = a + b;
    return res;
}

namespace Test {
    random_device dev;
    uint32_t seed = dev();
    mt19937 rng(seed);

    int randint(int a, int b) {
        return uniform_int_distribution(a, b)(rng);
    }

    void Test() {
        while (true) {
            int64_t n = randint(1, 2);
            vector<int64_t> d(n), l(n), r(n);
            for (auto &v: d) v = randint(0, 2);
            for (auto &v: l) v = randint(0, 2);
            for (auto &v: r) v = randint(0, 2);
            int64_t a0 = randint(0, 5), b0 = randint(0, 2);
            auto my_res = my_sol(n, d, l, r, a0, b0);
            auto good_res = good_sol(n, d, l, r, a0, b0);
            if (my_res != good_res) {
                cout << n << "\n";
                for (auto v: d) cout << v << " ";
                cout << "\n";
                for (auto v: l) cout << v << " ";
                cout << "\n";
                for (auto v: r) cout << v << " ";
                cout << "\n";
                cout << a0 << " " << b0 << "\n";
                cout << "my res: " << my_res << "\n";
                cout << "good res: " << good_res << "\n";
                break;
            }
        }
    }
}

void sol() {
//    Test::Test();
    int64_t n;
    cin >> n;
    vector<int64_t> d(n), l(n), r(n);
    for (auto &v: d) cin >> v;
    for (auto &v: l) cin >> v;
    for (auto &v: r) cin >> v;
    int64_t a0, b0;
    cin >> a0 >> b0;
    cout << my_sol(n, d, l, r, a0, b0);
//    vector<vector<pair<int64_t, int64_t>>> tr(n + 1);
//    tr[0].emplace_back(a0, b0);
//    for (int32_t i = 0; i < n; ++i) {
//        tr[i + 1].reserve(tr[i].size() * 2);
//        for (auto [a, b]: tr[i]) {
//            tr[i + 1].emplace_back(min(a + d[i], l[i]), b + d[i]);
//            tr[i + 1].emplace_back(a + d[i], min(b + d[i], r[i]));
//        }
//    }
//    int64_t res = 0;
//    for (auto [a, b]: tr[n]) res = max(res, a + b);
//    cout << res;
}

int main() {
#ifdef LOCAL
    freopen("input.txt", "r", stdin);
    freopen("output.txt", "w", stdout);
    uint64_t t = 1;
#else
    uint64_t t = 1;
#endif
    ios_base::sync_with_stdio(false);
    cin.tie(nullptr);
    cout.tie(nullptr);
    while (t--) {
        sol();
        cout << "\n";
    }
    return 0;
}
