#include <iostream>
#include <vector>
#include <algorithm>
#include <cmath>
#include <set>
#include <map>
#include <array>
#include <string>
#include <iomanip>
#include <queue>

using namespace std;
typedef long long ll;
typedef long double ld;
#define all(a) a.begin(), a.end()
#define fout(a) cout << fixed << setprecision(a)

const ll INF = 1e16;
const ll MOD = 1e9 + 7;
const ld EPS = 1e-9;

const ll C = (1LL << 47);

struct Node {
	ll l, r;
	ll modmin;
	ll mx;
	Node* lef, *rig;
	Node() {}
	Node(ll l, ll r, ll mx) : l(l), r(r), modmin(INF), mx(mx), lef(nullptr), rig(nullptr) {}
};

ll getmx(Node* v) {
	if (v == nullptr) return -INF;
	return min(v->modmin, v->mx);
}

void sum(Node* a, Node* b, Node* res) {
	*res = Node(min(a->l, b->l), max(a->r, b->r), max(getmx(a), getmx(b)));
	res->lef = a;
	res->rig = b;
}

const ll M = 1.1e7;
Node t[M];
ll tnow = 0;

Node* make(ll l, ll r, ll mx) {
	t[tnow] = Node(l, r, mx);
	return t + (tnow++);
}

void rel(Node* v, ll mn) {
	v->modmin = min(v->modmin, mn);
}

void push(Node* v) {
	ll m = (v->l + v->r) / 2;
	if (v->lef == nullptr) v->lef = make(v->l, m, -INF);
	if (v->rig == nullptr) v->rig = make(m, v->r, -INF);
	rel(v->lef, v->modmin);
	rel(v->rig, v->modmin);
}

Node* root = make(-C, C, -INF);

void change(ll lx, ll rx, Node* v, ll mn) {
	if (lx <= v->l && v->r <= rx) {
		rel(v, mn);
		return;
	}
	if (lx >= v->r || v->l >= rx) {
		return;
	}
	push(v);
	change(lx, rx, v->lef, mn);
	change(lx, rx, v->rig, mn);
	sum(v->lef, v->rig, v);
}

void relmax(ll i, ll mx, Node* v) {
	if (i <= v->l && v->r <= i + 1) {
		*v = Node(i, i + 1, mx);
		return;
	}
	if (i >= v->r || v->l >= i + 1) {
		return;
	}
	push(v);
	relmax(i, mx, v->lef);
	relmax(i, mx, v->rig);
	sum(v->lef, v->rig, v);
}

ll get(ll lx, ll rx, Node* v) {
	if (lx <= v->l && v->r <= rx) {
		return getmx(v);
	}
	if (lx >= v->r || v->l >= rx) {
		return -INF;
	}
	push(v);
	ll ans = max(get(lx, rx, v->lef), get(lx, rx, v->rig));
	sum(v->lef, v->rig, v);
	return ans;
}

ll obx(Node* v) {
	if (v->lef == nullptr && v->rig == nullptr) {
		return getmx(v) + v->r - 1;
	}
	push(v);
	return max(obx(v->lef), obx(v->rig));
}

int main() {
	ios_base::sync_with_stdio(0);
	cin.tie(0);
	cout << sizeof(Node) << "\n";
	ll n;
	cin >> n;
	vector<ll> d(n), l(n), r(n);
	ll a0, b0;
	for (ll i = 0; i < n; i++) cin >> d[i];
	for (ll i = 0; i < n; i++) cin >> l[i];
	for (ll i = 0; i < n; i++) cin >> r[i];
	cin >> a0 >> b0;
	ll sdvx = 0, sdvy = 0;
	relmax(a0, b0, root);
	for (ll i = 0; i < n; i++) {
		sdvx += d[i];
		sdvy += d[i];
		ll gh = get(l[i] - sdvx, C, root);
		relmax(l[i] - sdvx, gh, root);
		change(l[i] - sdvx + 1, C, root, r[i] - sdvy);
	}
	cout << obx(root) + sdvx + sdvy << "\n";
	return 0;
}