#pragma GCC optimize("Ofast")
#include <iostream>
#include <algorithm>
#include <vector>
#include <map>
#include <set>
#include <tuple>
using namespace std;
using ll = long long;
const ll INF = 1e18 + 7;
struct Point
{
	ll x, y;
	Point(int _x = 0, int _y = 0) : x(_x), y(_y) {}
	Point operator-(Point oth)
	{
		return Point(x - oth.x, y - oth.y);
	}
	ll operator*(Point oth)
	{
		return x * oth.x + y * oth.y;
	}
	ll operator%(Point oth)
	{
		return x * oth.y - y * oth.x;
	}
	Point operator+(Point oth)
	{
		return Point(x + oth.x, y + oth.y);
	}
};
ll binpow(ll n, ll p)
{
	if (p == 0) return 1;
	if (p % 2 == 0)
	{
		ll b = binpow(n, p / 2);
		return (b * b) % INF;
	}
	else
	{
		return (binpow(n, p - 1) * n) % INF;
	}
}
struct SegmentTree
{
	struct Node
	{
		ll mx=-INF;
		ll l=-1, r=-1;
		ll push = INF;

		Node(ll val=-INF) : mx(val) {}
	};
	vector<Node> tree;
	void Build()
	{
		tree.push_back(Node());
	}
	void Check(int v)
	{
		if (tree[v].l == -1)
		{
			tree[v].l = tree.size();
			tree.push_back(Node());
		}
		if (tree[v].r == -1)
		{
			tree[v].r = tree.size();
			tree.push_back(Node());
		}
	}
	void Push(int i)
	{
		Check(i);
		Node v = tree[i];
		tree[v.l].push = min(tree[v.l].push, v.push);
		tree[v.l].mx = min(tree[v.l].mx, tree[v.l].push);
		tree[v.r].push = min(tree[v.r].push, v.push);
		tree[v.r].mx = min(tree[v.r].mx, tree[v.r].push);
		tree[i].push = INF;
	}

	Node F(Node l, Node r)
	{
		Node x;
		x.mx = max(l.mx, r.mx);
		return x;
	}

	Node Get(int v, ll l, ll r, ll ql, ll qr)
	{
		if (r <= ql || qr <= l)
		{
			return Node();
		}
		if (ql <= l && r <= qr)
		{
			return tree[v];
		}
		Push(v);
		ll mid = (r + l) / 2;
		return F(Get(tree[v].l, l, mid, ql, qr), Get(tree[v].r,mid,r, ql, qr));
	}
	Node Get(ll l, ll r)
	{
		return Get(0, -INF, INF, l, r);
	}
	void SetMn(int v, ll l, ll r, ll ql, ll qr, ll x)
	{
		if (r <= ql || qr <= l)
		{
			return;
		}
		if (ql <= l && r <= qr)
		{
			tree[v].mx = min(tree[v].mx, x);
			tree[v].push = min(tree[v].push, x);
			return;
		}
		Push(v);
		ll mid = (r + l) / 2;
		SetMn(tree[v].l, l, mid, ql, qr, x); SetMn(tree[v].r, mid, r, ql, qr, x);
		tree[v].mx = F(tree[tree[v].l], tree[tree[v].r]).mx;
	}
	void SetMn(ll l, ll r, ll x)
	{
		SetMn(0, -INF, INF, l, r, x);
	}
	void Set(int v, ll l, ll r, ll ql, ll qr, ll x)
	{
		if (r <= ql || qr <= l)
		{
			return;
		}
		if (ql <= l && r <= qr)
		{
			tree[v].mx = x;
			tree[v].push = x;
			return;
		}
		Push(v);
		ll mid = (r + l) / 2;
		Set(tree[v].l, l, mid, ql, qr, x); Set(tree[v].r, mid, r, ql, qr, x);
		tree[v].mx = F(tree[tree[v].l], tree[tree[v].r]).mx;
	}
	void Set(ll l, ll r, ll x)
	{
		Set(0, -INF, INF, l, r, x);
	}
	ll GetAns(int v, ll l, ll r)
	{
		if (tree[v].mx = -INF) return -INF;
		if (r-l==1)
		{
			return tree[v].mx+l;
		}
		bool L = tree[v].l != -1, R = tree[v].r != -1;
		Push(v);
		ll mid = (r + l) / 2;
		return max(L?GetAns(tree[v].l,l,mid):-INF, R?GetAns(tree[v].r, mid, r):-INF);
	}
};
int main()
{
	ios_base::sync_with_stdio(0);
	cin.tie(0);
	cout.tie(0);
	int n;
	cin >> n;
	vector<ll> d(n + 1), l(n + 1), r(n + 1);
	//vector<vector<ll>> dp1(n + 1, vector<ll>(2)), dp2(n + 1, vector<ll>(2));
	for (int i = 1; i <= n; i++) cin >> d[i];
	for (int i = 1; i <= n; i++) cin >> l[i];
	for (int i = 1; i <= n; i++) cin >> r[i];
	vector<ll> pref(n + 1);
	for (int i = 1; i <= n; i++)
	{
		pref[i] = pref[i - 1] + d[i];
		l[i] -= pref[i];
		r[i] -= pref[i];
	}
	ll a0, b0;
	cin >> a0 >> b0;
	//vector<map<ll, ll>> dp(n + 1);
	SegmentTree tree;
	tree.Build();
	tree.Set(a0, a0 + 1, b0); //dp[0][a0] = b0;
	for (int i = 1; i <= n; i++)
	{
		//dp[i][l[i]] = -INF;
		//for (auto [a, b] : dp[i - 1])
		//{
			//dp[i][a] = min(b, r[i]);
			//dp[i][min(a, l[i])] = max(dp[i][min(a, l[i])], b);
		//}
		tree.Set(l[i], l[i] + 1, tree.Get(l[i], INF).mx);
		tree.SetMn(l[i] + 1, INF, r[i]);
	}
	cout << tree.GetAns(0,-INF,INF) + 2 * pref[n];
	//cout <<  + 2 * pref[n];
}