#include <iostream>
#include <cassert>
#include <algorithm>
#include <random>
#include <vector>
using namespace std;
using ll = long long;

const ll INF = 1e18 + 177013;
const int MAXN = 1e5 + 123;
int d[MAXN];
ll dsm[MAXN];
ll l[MAXN];
ll r[MAXN];

int break_ind[MAXN];

void init(int n) {
    for (int i = 1; i <= n; ++i)
        cin >> d[i];
    for (int i = 1; i <= n; ++i)
        dsm[i] = dsm[i - 1] + d[i];
    for (int i = 1; i <= n; ++i)
        cin >> l[i];
    for (int i = 1; i <= n; ++i)
        cin >> r[i];
    ll a0, b0;
    cin >> a0 >> b0;
    l[0] = a0;
    r[0] = b0;
    vector<pair<ll, int>> cpr(n + 1);
    for (int i = 0; i <= n; ++i)
        cpr[i] = {l[i] - dsm[i], i};
    sort(cpr.begin(), cpr.end());
    for (int i = 0; i <= n; ++i) {
        break_ind[i] = lower_bound(cpr.begin(), cpr.end(), pair<ll, int>(l[i] - dsm[i], i)) - cpr.begin();
    }
}

struct stree {
    int size;
    vector<ll> t;
    vector<ll> mod_add;
    vector<ll> mod_min;

    void resize(int n) {
        size = n;
        t.assign(2 * n - 1, -INF);
        mod_add.assign(2 * n - 1, 0);
        mod_min.assign(2 * n - 1, INF);
    }

    void push(int ind, int lnd, int rnd) {
        if (rnd - lnd == 1)
            return;
        int mnd = (lnd + rnd) / 2;
        int chd = ind + (mnd - lnd) * 2;
        t[ind + 1] = min(mod_min[ind], t[ind + 1] + mod_add[ind]);
        mod_min[ind + 1] = min(mod_min[ind + 1] + mod_add[ind], mod_min[ind]);
        mod_add[ind + 1] += mod_add[ind];

        t[chd] = min(mod_min[ind], t[chd] + mod_add[ind]);
        mod_min[chd] = min(mod_min[chd] + mod_add[ind], mod_min[ind]);
        mod_add[chd] += mod_add[ind];
        
        mod_add[ind] = 0;
        mod_min[ind] = INF;
    }

    ll get(int ind, int lnd, int rnd, int l, int r) {
        if (l <= lnd && rnd <= r)
            return t[ind];
        if (l >= rnd || lnd >= r)
            return -INF;
        int mnd = (lnd + rnd) / 2;
        int chd = ind + (mnd - lnd) * 2;
        return min(max(get(ind + 1, lnd, mnd, l, r), get(chd, mnd, rnd, l, r)) + mod_add[ind], mod_min[ind]);
    }

    ll get(int l, int r) {
        return get(0, 0, size, l, r);
    }

    void add(int ind, int lnd, int rnd, int l, int r, ll x) {
        if (l <= lnd && rnd <= r) {
            t[ind] += x;
            mod_add[ind] += x;
            mod_min[ind] += x;
            return;
        }
        if (l >= rnd || lnd >= r)
            return;
        push(ind, lnd, rnd);
        int mnd = (lnd + rnd) / 2;
        int chd = ind + (mnd - lnd) * 2;
        add(ind + 1, lnd, mnd, l, r, x);
        add(chd, mnd, rnd, l, r, x);
        t[ind] = max(t[ind + 1], t[chd]);
    }

    void add(int l, int r, ll x) {
        add(0, 0, size, l, r, x);
    }

    void mineq(int ind, int lnd, int rnd, int l, int r, ll x) {
        if (l <= lnd && rnd <= r) {
            t[ind] = min(t[ind], x);
            mod_min[ind] = min(mod_min[ind], x);
            return;
        }
        if (l >= rnd || lnd >= r)
            return;
        push(ind, lnd, rnd);
        int mnd = (lnd + rnd) / 2;
        int chd = ind + (mnd - lnd) * 2;
        mineq(ind + 1, lnd, mnd, l, r, x);
        mineq(chd, mnd, rnd, l, r, x);
        t[ind] = max(t[ind + 1], t[chd]);
    }

    void mineq(int l, int r, ll x) {
        mineq(0, 0, size, l, r, x);
    }

    void set(int i, ll x) {
        ll val = get(i, i + 1);
        if (val < x)
            add(i, i + 1, x - val);
    }
};

signed main() {
    ios_base::sync_with_stdio(false);
    cin.tie(nullptr);
    cout.tie(nullptr);
    int n;
    cin >> n;
    init(n);

    stree dp;
    dp.resize(n + 1);
    dp.set(break_ind[0], r[0]);
    for (int i = 0; i < n; ++i) {
        ll ndpval = dp.get(break_ind[i + 1] + 1, n + 1);
        dp.add(0, n + 1, d[i + 1]);
        dp.mineq(break_ind[i + 1] + 1, n + 1, r[i + 1]);
        dp.set(break_ind[i + 1], ndpval + d[i + 1]);
    }
    ll ans = -INF;
    //for (int i = 0; i <= n; ++i) for (int j = i + 1; j <= n; ++j)
        //assert(break_ind[i] != break_ind[j]);
    for (int i = 0; i <= n; ++i) {
        ans = max(ans, dp.get(break_ind[i], break_ind[i] + 1) + l[i] + dsm[n] - dsm[i]);
    }
    cout << ans << endl;
}
