#include <bits/stdc++.h>

#ifdef LOCAL
#define debug(x) cerr << #x << ": " << x << endl
#else
#define debug(x)
#endif

#define int long long

using namespace std;

mt19937 rnd(0);

using ll = long long;
using pi = pair<int, int>;
using vi = vector<int>;
using vpi = vector<pi>;

const int maxN = 1e5;

struct item {
    ll key, prior, val, max, upd;
    item *l, *r;
    item(ll a = 0, ll b = 0) : key(a), prior(rnd()), val(b), max(b), upd(0), l(nullptr), r(nullptr) {}
};

typedef item* pitem;

int n, d[maxN], l[maxN], r[maxN], a0, b0;
pitem treap;

void push(pitem t) {
    if (!t) {
        return;
    }
    t->key += t->upd;
    t->val += t->upd;
    t->max += t->upd;
    if (t->l) {
        t->l->upd += t->upd;
    }
    if (t->r) {
        t->r->upd += t->upd;
    }
    t->upd = 0;
}

void upd(pitem t) {
    if (!t) {
        return;
    }
    t->max = t->val;
    if (t->l) {
        t->max = t->l->max;
    }
}

void merge(pitem& t, pitem l, pitem r) {
    push(l);
    push(r);
    if (!l || !r) {
        t = l ? l : r;
        return;
    }
    if (l->prior > r->prior) {
        t = l;
        merge(t->r, t->r, r);
        push(t->l);
    } else {
        t = r;
        merge(t->l, l, t->l);
        push(t->r);
    }
    upd(t);
}

void split(pitem t, pitem& l, pitem& r, int key) {
    if (!t) {
        l = r = nullptr;
        return;
    }
    push(t);
    if (t->key < key) {
        l = t;
        split(t->r, l->r, r, key);
        push(l->l);
        upd(l);
    } else {
        r = t;
        split(t->l, l, r->l, key);
        push(r->r);
        upd(r);
    }
}

void split1(pitem t, pitem& l, pitem& r, int val) {
    if (!t) {
        l = r = nullptr;
        return;
    }
    push(t);
    if (t->val > val) {
        l = t;
        split1(t->r, l->r, r, val);
        push(l->l);
        upd(l);
    } else {
        r = t;
        split1(t->l, l, r->l, val);
        push(r->r);
        upd(r);
    }
}

void insert(pitem& t, pitem it) {
    if (!t) {
        t = it;
        return;
    }
    push(t);
    if (it->prior > t->prior) {
        split(t, it->l, it->r, it->key);
        t = it;
    } else if (it->key < t->key) {
        insert(t->l, it);
        push(t->r);
    } else {
        push(t->l);
        insert(t->r, it);
    }
    upd(t);
}

ll get(pitem t) {
    if (!t) {
        return 0;
    }
    push(t);
    ll ans = t->key + t->val;
    ans = max(ans, get(t->l));
    ans = max(ans, get(t->r));
    return ans;
}

void print(pitem t) {
    if (!t) {
        return;
    }
    push(t);
    print(t->l);
    print(t->r);
    upd(t);
    if (t->l) {
        assert(t->l->max >= t->val);
    }
    if (t->r) {
        assert(t->val >= t->r->max);
    }
}

void solve() {
    cin >> n;
    for (int i = 0; i < n; i++) {
        cin >> d[i];
    }
    for (int i = 0; i < n; i++) {
        cin >> l[i];
    }
    for (int i = 0; i < n; i++) {
        cin >> r[i];
    }
    cin >> a0 >> b0;
    treap = new item(a0, b0);
    for (int i = 0; i < n; i++) {
        treap->upd += d[i];
        push(treap);
        pitem a, b, c;
        split(treap, a, b, l[i]);
        split1(b, b, c, r[i] - 1);
        merge(treap, a, c);
        if (b) {
            push(b);
            insert(treap, new item(l[i], b->max));
            while (b->r) {
                b = b->r;
                push(b);
            }
            if (b->key != l[i]) {
                insert(treap, new item(b->key, r[i]));
            }
        }
    }
    cout << get(treap) << "\n";
}

signed main() {
    ios_base::sync_with_stdio(false);
    cin.tie(0);
    cout.tie(0);
    int test = 1;
#ifdef LOCAL
    cin >> test;
#endif
    while (test--) {
        solve();
    }
    return 0;
}