// pe_solve.cpp
#include <iostream>
#include <vector>
#include <set>
#include <map>
#include <unordered_map>
#include <unordered_set>
#include <numeric>
#include <deque>
#include <random>
#include <functional>
#include <cassert>
#include <queue>
 
using namespace std;

const int maxn = 1 << 19;
const int inf = 1e9;
int rb[maxn];
int rev[maxn];
int ans[maxn];
int rt[maxn];

namespace dsu {
    int p[maxn];
    int sz[maxn];

    void init() {
        iota(p, p + maxn, 0);
        iota(rb, rb + maxn, 0);
        fill(sz, sz + maxn, 1);
    }

    int c(int v) {
        return (p[v] == v) ? v : p[v] = c(p[v]);
    }

    void unite(int u, int v) {
        u = c(u), v = c(v);
        if (u == v) return;
        if (sz[u] > sz[v]) swap(u, v);
        p[u] = v;
        sz[v] += sz[u];
        rb[v] = max(rb[u], rb[v]);
    }
};

namespace SegTree {
    int tree[maxn * 2];

    void upd(int i, int x) {
        i += maxn;
        tree[i] = x;
        while (i > 1) {
            i >>= 1;
            tree[i] = max(tree[i * 2], tree[i * 2 + 1]);
        }
    }

    int get(int l, int r) {
        l += maxn;
        r += maxn;
        int ans = -inf;
        while (l < r) {
            if (l & 1) ans = max(ans, tree[l++]);
            if (r & 1) ans = max(ans, tree[--r]);
            l >>= 1;
            r >>= 1;
        }
        return ans;
    }
};

namespace mem {
    int tree[maxn * 2];
    
    void upd(int i, int x) {
        i += maxn;
        tree[i] = x;
        while (i > 1) {
            i >>= 1;
            tree[i] = max(tree[i * 2], tree[i * 2 + 1]);
        }
    }

    int get(int pos, int i = 1, int l = 0, int r = maxn) {
        if (r <= pos) return -1;
        if (!tree[i]) return -1;
        if (l + 1 == r) {
            return l;
        }
        int m = (l + r) / 2;
        if (tree[i * 2]) {
            int ans = get(pos, i * 2, l, m);
            if (ans != -1) {
                return ans;
            }
        }
        return get(pos, i * 2 + 1, m, r);
    }
};

struct query {
    int t, u, v, i;
    query() {}
    query(int t, int u, int v, int i): t(t), u(u), v(v), i(i) {}
};

int up[20][maxn];

void solveMid(int l, int m, int r, vector<query> &queries) {
    for (int i = l; i < m; ++i) {
        SegTree::upd(rev[i], i);
    }
    for (int i = l; i < m; ++i) {
        int pos = rev[i];
        int x = SegTree::get(pos, rt[pos]);
        if (x > 0) {
            up[0][pos] = rev[x];
        } else {
            up[0][pos] = -1;
        }
    }
    for (int j = 1; j < 20; ++j) {
        for (int i = l; i < m; ++i) {
            int pos = rev[i];
            if (up[j - 1][pos] != -1)
                up[j][pos] = up[j - 1][up[j - 1][pos]];
            else
                up[j][pos] = -1;
        }
    }
    int x = m;
    for (query& q : queries) {
        while (x <= q.v) {
            mem::upd(rev[x], 1);
            SegTree::upd(rev[x], x);
            ++x;
        }
        if (q.u >= m) continue;
        int u = q.u;
        int pos = rev[u];
        int pt = mem::get(pos);
        for (int j = 19; j >= 0; --j) {
            if (up[j][pos] == -1 || rt[up[j][pos]] > pt) continue;
            ans[q.i] += 1 << j;
            pos = up[j][pos];
        }
        if (pt >= rt[pos]) {
            ++ans[q.i];
            pos = up[0][pos];
            if (pos == -1) {
                ans[q.i] = 0;
                q.u = inf;
                continue;
            }
        }
        int nxt = SegTree::get(pos, rt[pos]);
        if (nxt >= m) {
            ++ans[q.i];
            q.u = nxt;
        } else {
            ans[q.i] = 0;
            q.u = inf;
        }
    }
    for (int i = l; i < r; ++i) {
        SegTree::upd(rev[i], -inf);
    }
    for (int i = m; i < r; ++i) {
        mem::upd(rev[i], 0);
    }
}

void solve(int l, int r, vector<query> &queries) {
    if (queries.empty()) return;
    int m = (l + r) / 2;
    vector<query> ql, qr;
    for (query q : queries) {
        if (q.u >= q.v) continue;
        if (q.v < m) ql.push_back(q);
        else qr.push_back(q);
    }
    solve(l, m, ql);
    solveMid(l, m, r, qr);
    solve(m, r, qr);
}

int32_t main() {
    ios_base::sync_with_stdio(false);
    cin.tie(0);
    cout.tie(0);
    int n, d, group;
    cin >> n >> d >> group;
    vector<int> a(n);
    vector<pair<int, int>> dt;
    for (int& i : a) {
        cin >> i;
    }
    dsu::init();
    for (int i = 0; i < n; ++i) {
        dt.push_back({ a[i], i });
    }
    sort(dt.rbegin(), dt.rend());
    fill(rt, rt + maxn, n);
    vector<int> arr(n, 0);
    set<int> ends;
    for (auto [x, i] : dt) {
        arr[i] = 1;
        if (i > 0 && arr[i - 1]) {
            ends.erase(i - 1);
            dsu::unite(i - 1, i);
        }
        if (i + 1 < n && arr[i + 1]) {
            dsu::unite(i, i + 1);
            if (dsu::sz[dsu::c(i)] >= d) {
                ends.insert(rb[dsu::c(i)]);
            }
        } else {
            if (dsu::sz[dsu::c(i)] >= d) {
                ends.insert(i);
            }
        }
        auto ptr = ends.lower_bound(i + d);
        if (ptr != ends.end()) {
            int id = *ptr;
            int bgn = max(i + 1, rb[dsu::c(id)] - dsu::sz[dsu::c(id)] + 1);
            rt[i] = bgn + d;
        }
    }
    for (int i = 0; i < n; ++i) {
        rev[a[i]] = i;
    }
    int q;
    cin >> q;
    vector<query> queries;
    for (int i = 0; i < q; ++i) {
        int t, u, v;
        cin >> t >> u >> v;
        queries.emplace_back(t, a[u - 1], a[v - 1], i);
    }
    sort(queries.begin(), queries.end(), [&] (query a, query b) {
        if (a.v == b.v) return a.i < b.i;
        return a.v < b.v;
    });
    solve(1, n + 1, queries);
    for (query q : queries) {
        if (q.t == 1) {
            ans[q.i] = (bool) ans[q.i];
        }
    }
    for (int i = 0; i < q; ++i) {
        cout << ans[i] << '\n';
    }
}
