// n2-diff-better.cpp
#include <iostream>
#include <vector>
#include <set>
#include <map>
#include <unordered_map>
#include <unordered_set>
#include <numeric>
#include <deque>
#include <random>
#include <functional>
#include <cassert>
#include <queue>

using namespace std;

const int lg = 9;
const int maxn = 300'000;
const int inf = 1e9;

/*  simple dsu  */

int p[maxn];
int sz[maxn];
int r[maxn];

void init() {
    iota(p, p + maxn, 0);
    iota(r, r + maxn, 0);
    fill(sz, sz + maxn, 1);
}

int c(int v) {
    return (p[v] == v) ? v : p[v] = c(p[v]);
}

void unite(int u, int v) {
    u = c(u), v = c(v);
    if (u == v) return;
    if (sz[u] > sz[v]) swap(u, v);
    p[u] = v;
    sz[v] += sz[u];
    r[v] = max(r[u], r[v]);
}

/*  jumps  */

int go[maxn];
int jump[maxn];
int goGood[maxn];
vector<int> g[maxn];

const int rebuildSize = 1500;
const int batchSize = rebuildSize;

/*  jumps calc  */

void dfs(int v, vector<int> &s) {
    jump[v] = v;
    if (s.size() >= batchSize) {
        jump[v] = s[s.size() - batchSize];
    }
    s.push_back(v);
    for (int u : g[v]) {
        dfs(u, s);
    }
    s.pop_back();
}

/*  fast get-min  */

struct mndsu {
    int p[maxn];
    int sz[maxn];
    int mx[maxn];
    vector<int> st;
    vector<int> ids;

    void reset() {
        iota(p, p + maxn, 0);
        fill(sz, sz + maxn, 1);
        iota(mx, mx + maxn, 0);
        st.clear();
        ids.clear();
    }

    int c(int v) {
        return (p[v] == v) ? v : p[v] = c(p[v]);
    }

    void unite(int u, int v) {
        u = c(u), v = c(v);
        if (u == v) return;
        mx[v] = mx[u];
        if (sz[u] > sz[v]) swap(u, v);
        p[u] = v;
        sz[v] += sz[u];
    }
    
    void upd(int i, int x) {
        while (!st.empty() && st.back() <= x) {
            unite(i, ids.back());
            ids.pop_back();
            st.pop_back();
        }
        st.push_back(x);
        ids.push_back(i);
    }
    
    int get(int i) {
        return mx[c(i)];
    }
};

mndsu ds;

int32_t main() {
    ios_base::sync_with_stdio(false);
    cin.tie(0);
    cout.tie(0);
    int n, d, group;
    cin >> n >> d >> group;
    vector<int> a(n);
    vector<pair<int, int>> dt;
    for (int& i : a) {
        cin >> i;
    }
    init();
    for (int i = 0; i < n; ++i) {
        dt.push_back({ a[i], i });
    }
    sort(dt.rbegin(), dt.rend());
    vector<int> rt(n, n);
    vector<int> arr(n, 0);
    set<int> ends;
    for (auto [x, i] : dt) {
        arr[i] = 1;
        if (i > 0 && arr[i - 1]) {
            ends.erase(i - 1);
            unite(i - 1, i);
        }
        if (i + 1 < n && arr[i + 1]) {
            unite(i, i + 1);
            if (sz[c(i)] >= d) {
                ends.insert(r[c(i)]);
            }
        } else {
            if (sz[c(i)] >= d) {
                ends.insert(i);
            }
        }
        auto ptr = ends.lower_bound(i + d);
        if (ptr != ends.end()) {
            int id = *ptr;
            int bgn = max(i + 1, r[c(id)] - sz[c(id)] + 1);
            rt[i] = bgn + d;
        }
    }
    int q;
    cin >> q;
    vector<vector<pair<pair<int, int>, int>>> queries(n);
    for (int i = 0; i < q; ++i) {
        int t, u, v;
        cin >> t >> u >> v;
        --u; --v;
        if (a[u] > a[v]) continue;
        queries[v].push_back({ { t, u }, i });
    }
    vector<int> ans(q, 0);
    int id = 0;
    vector<int> vv;
    vector<int> bad;
    vector<int> b = a;
    ds.reset();
    for (int i = n - 1; i >= 0; --i) {
        ds.upd(i, b[i]);
        goGood[i] = ds.get(rt[i] - 1);
    }
    for (auto [x, i] : dt) {
        if (id % rebuildSize == 0) {
            bad.clear();
            for (int i = 0; i < n; ++i) {
                g[i].clear();
            }
            memcpy(go, goGood, maxn * sizeof(int));
            for (int i = n - 1; i >= 0; --i) {
                if (go[i] != i) {
                    g[go[i]].push_back(i);
                }
            }
            for (int i = 0; i < n; ++i) {
                if (go[i] == i) {
                    dfs(i, vv);
                }
            }
            for (int j = id; j < n && j < id + rebuildSize; ++j) {
                bad.push_back(dt[j].second);
                b[dt[j].second] = -inf;
            }
            ds.reset();
            for (int i = n - 1; i >= 0; --i) {
                ds.upd(i, b[i]);
                goGood[i] = ds.get(rt[i] - 1);
            }
            for (int j : bad) {
                b[j] = a[j];
            }
            sort(bad.begin(), bad.end());
        }
        ++id;
        for (auto [p, j] : queries[i]) {
            auto [t, u] = p;
            int res = 0;
            bool ok = false;
            int ptrR = -1;
            deque<pair<int, int>> s;
            for (int step = 0; step < rebuildSize; ++step) {
                while (jump[u] != u && a[jump[u]] < a[i]) {
                    u = jump[u];
                    res += batchSize;
                }
                while (go[u] != u && a[go[u]] < a[i]) {  ///TODO: optimize this
                    ++res;
                    u = go[u];
                }
                int goBad = -1;
                while (ptrR + 1 < bad.size() && bad[ptrR + 1] < rt[u]) {
                    if (a[bad[ptrR + 1]] <= a[i]) {
                        while (!s.empty() && s.back().first <= a[bad[ptrR + 1]]) s.pop_back();
                        s.push_back({ a[bad[ptrR + 1]], bad[ptrR + 1] });
                    }
                    ++ptrR;
                }
                while (!s.empty() && s[0].second < u) s.pop_front();
                if (!s.empty()) {
                    goBad = s[0].second;
                }
                if (goBad != -1 && a[goGood[u]] < a[goBad]) {
                    u = goBad;
                } else {
                    u = goGood[u];
                }
                ++res;
                if (u == i) {
                    ok = true;
                    break;
                }
            }
            if (ok) {
                if (t == 2) {
                    ans[j] = res;
                } else {
                    ans[j] = 1;
                }
            }
        }
        b[i] = -inf;
    }
    for (int j : ans) {
        cout << j << '\n';
    }
}
/*
 7 1
 0 5 6 1 2 3 4
 10
 1 3 6
 1 1 2
 1 2 6
 1 3 5
 1 6 7
 1 2 3
 1 1 2
 1 2 5
 1 2 6
 1 1 7
 */
