// mangooste_ok.cpp
#include <bits/stdc++.h>
using namespace std;

using ll = long long;
using ld = long double;

#define all(a) begin(a), end(a)
#define len(a) int((a).size())

template<typename Fun>
struct y_combinator {
    const Fun fun;

    explicit y_combinator(const Fun&& fun) : fun(std::forward<const Fun>(fun)) {}

    template<typename... Args>
    auto operator()(Args&&... args) const {
        return fun(std::ref(*this), std::forward<Args>(args)...);
    }
};

struct disjoin_set_union {
    vector<int> par;

    disjoin_set_union(int n) : par(n) {
        iota(all(par), 0);
    }

    int root(int v) {
        return par[v] == v ? v : (par[v] = root(par[v]));
    }

    void unite(int v, int u) {
        par[v] = max(par[v], u);
    }
};

struct segtree {
    int n;
    vector<int> tree;

    segtree(int sz) : n(1) {
        while (n < sz) {
            n *= 2;
        }
        tree.resize(2 * n, -1);
    }

    void update(int pos, int val) {
        pos += n;
        tree[pos] = val;
        for (; pos > 1; pos >>= 1) {
            tree[pos >> 1] = max(tree[pos], tree[pos ^ 1]);
        }
    }

    int query(int l, int r) {
        int res = -1;
        for (l += n, r += n; l < r; l >>= 1, r >>= 1) {
            if (l & 1) {
                res = max(res, tree[l++]);
            }
            if (r & 1) {
                res = max(res, tree[--r]);
            }
        }
        return res;
    }

    int find_first(int v, int vl, int vr, int from) {
        if (tree[v] == -1 || vr <= from) {
            return -1;
        }
        if (vr - vl == 1) {
            return vl;
        }
        int vm = (vl + vr) >> 1;
        int res = find_first(v << 1, vl, vm, from);
        if (res == -1) {
            res = find_first(v << 1 | 1, vm, vr, from);
        }
        return res;
    }

    int find_first(int from) {
        return find_first(1, 0, n, from);
    }
};

int main() {
    cin.tie(nullptr)->sync_with_stdio(false);

    int n, d, group;
    cin >> n >> d >> group;
    vector<int> a(n);
    vector<int> inv_a(n);
    for (int i = 0; i < n; i++) {
        cin >> a[i];
        a[i]--;
        inv_a[a[i]] = i;
    }

    disjoin_set_union dsu(n);
    set<int> active;
    vector<int> max_right(n);
    for (auto pos : inv_a) {
        auto it = active.insert(pos).first;
        if (next(it) != active.end()) {
            int nxt = *next(it);
            if (nxt - pos <= d) {
                dsu.unite(pos, nxt);
            }
        }
        if (it != active.begin()) {
            int prv = *prev(it);
            if (pos - prv <= d) {
                dsu.unite(prv, pos);
            }
        }
        max_right[pos] = min(n - 1, dsu.root(pos) + d) + 1;
    }

    struct query {
        int from, to, type, id;
    };
    vector<query> queries;

    int q;
    cin >> q;
    vector<int> ans(q);
    for (int i = 0; i < q; i++) {
        int type, l, r;
        cin >> type >> l >> r;
        l--, r--;
        if (a[l] < a[r]) {
            queries.push_back({a[l], a[r], type, i});
        }
    }
    sort(all(queries), [&](const auto &a, const auto &b) {
        return a.to < b.to;
    });

    segtree tree_jump(n);
    constexpr int LG = 19;
    vector<array<int, LG>> par(n);

    y_combinator([&](auto solve, int l, int r, vector<query> queries) -> void {
        if (r - l == 1 || queries.empty()) {
            return;
        }
        int mid = (l + r) / 2;
        for (int i = mid - 1; i >= l; i--) {
            int value = tree_jump.query(inv_a[i], max_right[inv_a[i]]);
            if (value != -1) {
                par[i][0] = value;
                for (int level = 1; level < LG; level++) {
                    par[i][level] = par[par[i][level - 1]][level - 1];
                }
            } else {
                par[i].fill(i);
            }
            tree_jump.update(inv_a[i], i);
        }
        for (int i = l; i < mid; i++) {
            tree_jump.update(inv_a[i], -1);
        }

        int ptr = 0;
        for (int i = mid; i < r; i++) {
            tree_jump.update(inv_a[i], i);
            for (; ptr < len(queries) && queries[ptr].to <= i; ptr++) {
                if (queries[ptr].to != i || queries[ptr].from >= mid) {
                    continue;
                }
                auto &[from, to, type, id] = queries[ptr];
                int f = tree_jump.find_first(inv_a[from]);
                assert(f != -1);

                for (int level = LG - 1; level >= 0; level--) {
                    int j = par[from][level];
                    if (max_right[inv_a[j]] <= f) {
                        from = j;
                        ans[id] += (1 << level);
                    }
                }
                if (max_right[inv_a[from]] <= f) {
                    from = par[from][0];
                    ans[id]++;
                }

                if (max_right[inv_a[from]] <= f) {
                    ans[id] = 0;
                    from = -1;
                    continue;
                }

                ans[id]++;
                from = tree_jump.query(inv_a[from], min(inv_a[to] + 1, max_right[inv_a[from]]));
                assert(from != -1);
            }
        }
        for (int i = mid; i < r; i++) {
            tree_jump.update(inv_a[i], -1);
        }

        vector<query> left;
        vector<query> right;
        left.reserve(len(queries));
        right.reserve(len(queries));
        for (auto &q : queries) {
            if (q.from != -1) {
                (q.to < mid ? left : right).push_back(q);
            }
        }

        solve(l, mid, left);
        solve(mid, r, right);
    })(0, n, queries);

    for (auto &q : queries) {
        if (q.type == 1) {
            ans[q.id] = min(ans[q.id], 1);
        }
    }
    for (auto x : ans) {
        cout << x << '\n';
    }
}
