#include "bits/stdc++.h"

using namespace std;

using ll = long long;

struct Node {
    int x;

    Node() : x(1e9) {};

    Node(int x) : x(x) {};

    friend Node operator+(Node L, Node R) {
        return Node(min(L.x, R.x));
    }
};

template<typename Node>
class Segtree {
public:
    template<typename Input>
    Segtree(int n, vector<Input> a) : n(n) {
        t.resize(4 * n + 1);
        build(0, 0, n, a);
    }

    template<typename Input>
    void update(int i, Input x) {
        update(0, 0, n, i, x);
    }

    Node get(int l, int r) {
        return get(0, 0, n, l, r);
    }

private:
    template<typename Input>
    void build(int v, int l, int r, vector<Input> &a) {
        if (r - l == 1) {
            t[v] = Node(a[l]);
            return;
        }
        int m = (l + r) / 2;
        build(2 * v + 1, l, m, a);
        build(2 * v + 2, m, r, a);
        t[v] = t[2 * v + 1] + t[2 * v + 2];
    }

    template<typename Input>
    void update(int v, int l, int r, int i, Input x) {
        if (r - l == 1) {
            t[v] = Node(x);
            return;
        }
        int m = (l + r) / 2;
        if (m > i) update(2 * v + 1, l, m, i, x);
        else update(2 * v + 2, m, r, i, x);
        t[v] = t[2 * v + 1] + t[2 * v + 2];
    }

    Node get(int v, int l, int r, int ql, int qr) {
        if (ql <= l && r <= qr) return t[v];
        if (l >= qr || r <= ql) return Node();

        int m = (l + r) / 2;
        return get(2 * v + 1, l, m, ql, qr) + get(2 * v + 2, m, r, ql, qr);
    }

    int n;
    vector<Node> t;
};

signed main() {
#ifdef LOCAL
    freopen("input.txt", "r", stdin);
    freopen("output.txt", "w", stdout);
#else
    ios_base::sync_with_stdio(false);
    cin.tie(nullptr);
#endif

    int n, D, group;
    cin >> n >> D >> group;

    assert(group == 0 || n == 100);

    vector<int> a(n);
    for (int i = 0; i < n; ++i) {
        cin >> a[i];
    }

    vector<vector<int>> distmove(n, vector<int>(n, 1e9));
    vector<vector<int>> dist(n, vector<int>(n, 1e9));
    for (int start = 0; start < n; ++start) {
        distmove[start][start] = 0;
        dist[start][start] = 0;
        deque<int> q = {start};
        while (!q.empty()) {
            int v = q.front();
            q.pop_front();

            for (int i = 1; i <= D; ++i) {
                if (start + i >= n) break;
                if (a[start + i] >= a[start] || distmove[start][start + i] != 1e9) continue;
                distmove[start][start + i] = distmove[start][v] + 1;
                q.push_back(start + i);
            }
        }

        Segtree<Node> tree(n, distmove[start]);
        for (int i = start + 1; i < n; ++i) {
            if (a[i] < a[start]) continue;
            dist[start][i] = min<int>(dist[start][i], (tree.get(max(0, i - D), i).x == 1e9 ? 1e9 : 0) + 1);
        }
    }

    vector<vector<int>> g(n);
    for (int i = 0; i < n; ++i) {
        for (int j = i + 1; j < n; ++j) {
            if (dist[i][j] != 1e9){
                g[i].push_back(j);
                cerr << i + 1 << " " << j + 1 << "\n";
            }
        }
    }

    vector<vector<int>> d(n, vector<int>(n, 1e9));
    for (int i = 0; i < n; ++i) {
        d[i][i] = 0;
        deque<int> q = {i};
        while (!q.empty()){
            int v = q.front();
            q.pop_front();

            for (auto u: g[v]) {
                if (d[i][u] == 1e9){
                    d[i][u] = d[i][v] + 1;
                    q.push_back(u);
                }
            }
        }
    }

    int q;
    cin >> q;

    while (q--){
        int t, u, v;
        cin >> t >> u >> v;

        if (t==1) cout << (d[u - 1][v - 1] == 1e9 ? 0 : 1) << "\n";
        else cout << (d[u - 1][v - 1] == 1e9 ? 0 : d[u - 1][v - 1]) << "\n";
    }

}