#include <algorithm>
#include <iostream>
#include <vector>

using namespace std;

struct seg_tree1 {
    struct node {
        int ans = 0;
        int pref = 0;
        int suf = 0;
        int len = 0;
    };
    static node pull(node a, node b) {
        node c;
        c.len = a.len + b.len;
        if (a.ans == a.len && b.ans == b.len) {
            c.ans = c.pref = c.suf = a.len + b.len;
            return c;
        }
        if (a.ans == a.len) {
            c.pref = a.len + b.pref;
            c.suf = b.suf;
            c.ans = max({a.ans, b.ans, a.pref, a.suf, a.suf + b.pref});
            return c;
        }
        if (b.ans == b.len) {
            c.pref = a.pref;
            c.suf = b.len + a.suf;
            c.ans = max({a.ans, b.ans, a.pref, a.suf, a.suf + b.pref});
            return c;
        }
        c.pref = a.pref;
        c.suf = b.suf;
        c.ans = max({a.ans, b.ans, a.suf + b.pref});
        return c;
    }
    vector<node> t;
    void init(int n) {
        t.assign(n << 2, node());
    }
    void build(int v, int tl, int tr) {
        if (tl + 1 == tr) {
            t[v] = {1, 1, 1, 1};
            return;
        }
        int tm = (tl + tr) >> 1;
        build(2 * v + 1, tl, tm);
        build(2 * v + 2, tm, tr);
        t[v] = pull(t[2 * v + 1], t[2 * v + 2]);
    }
    void upd(int v, int tl, int tr, int i) {
        if (tl + 1 == tr) {
            t[v] = {0, 0, 0, 1};
            return;
        }
        int tm = (tl + tr) >> 1;
        if (i < tm) {
            upd(2 * v + 1, tl, tm, i);
        } else {
            upd(2 * v + 2, tm, tr, i);
        }
        t[v] = pull(t[2 * v + 1], t[2 * v + 2]);
    }
    node get(int v, int tl, int tr, int l, int r) {
        if (tr <= l || r <= tl) {
            return {0, 0, 0, 0};
        }
        if (l <= tl && tr <= r) {
            return t[v];
        }
        int tm = (tl + tr) >> 1;
        return pull(get(2 * v + 1, tl, tm, l, r), get(2 * v + 2, tm, tr, l, r));
    }
};

struct seg_tree2 {
    vector<int> a;
    vector<pair<int, int>> t;
    void init(int n) {
        a.assign(n, 0);
        t.assign(n << 2, {-1, -1});
    }
    void build(int v, int tl, int tr) {
        if (tl + 1 == tr) {
            t[v] = {a[tl], tl};
            return;
        }
        int tm = (tl + tr) >> 1;
        build(2 * v + 1, tl, tm);
        build(2 * v + 2, tm, tr);
        t[v] = max(t[2 * v + 1], t[2 * v + 2]);
    }
    pair<int, int> get(int v, int tl, int tr, int l, int r) {
        if (tr <= l || r <= tl) {
            return {-1, -1};
        }
        if (l <= tl && tr <= r) {
            return t[v];
        }
        int tm = (tl + tr) >> 1;
        return max(get(2 * v + 1, tl, tm, l, r), get(2 * v + 2, tm, tr, l, r));
    }
};

signed main() {
    ios_base::sync_with_stdio(false);
    cin.tie(nullptr);
    int n, d, g;
    cin >> n >> d >> g;
    vector<int> a(n);
    vector<vector<int>> pos(n);
    for (int i = 0; i < n; i++) {
        cin >> a[i], a[i]--;
        pos[a[i]].push_back(i);
    }
    seg_tree1 T1;
    seg_tree2 T2;
    T1.init(n);
    T2.init(n);
    T2.a = a;
    T1.build(0, 0, n);
    T2.build(0, 0, n);
    vector<int> link(n), max_jump(n);
    for (int c = 0; c < n; c++) {
        for (int i : pos[c]) {
            int l = i - 1, r = n;
            while (r - l > 1) {
                int m = (l + r) >> 1;
                int x = T1.get(0, 0, n, i + 1, m + 1).ans;
                if (x <= d) {
                    l = m;
                } else {
                    r = m;
                }
            }
            auto [x, j] = T2.get(0, 0, n, i + 1, l + 1);
            max_jump[i] = l;
            if (x <= a[i]) {
                link[i] = i;
            } else {
                link[i] = j;
            }
        }
        for (int i : pos[c]) {
            T1.upd(0, 0, n, i);
        }
    }
//    for (int i = 0; i < n; i++) {
//        cerr << "! " << i + 1 << ' ' << link[i] + 1 << '\n';
//    }
    vector<vector<int>> go(n, vector<int>(20));
    for (int i = 0; i < n; i++) {
        go[i][0] = link[i];
    }
    for (int c = 1; c < 20; c++) {
        for (int i = 0; i < n; i++) {
            go[i][c] = go[go[i][c - 1]][c - 1];
        }
    }
    int q;
    cin >> q;
    while (q--) {
        int t, u, v;
        cin >> t >> u >> v, u--, v--;
        if (go[u][19] < v || a[u] >= a[v]) {
            cout << "0\n";
            continue;
        }
        int l = -1, r = n + 1;
        while (r - l > 1) {
            int m = (l + r) >> 1;
            int uu = u;
            for (int c = 19; c >= 0; c--) {
                if (m & (1 << c)) {
                    uu = go[uu][c];
                }
            }
            if (max_jump[uu] >= v) {
                r = m;
            } else {
                l = m;
            }
        }
        cout << (t == 1 ? 1 : r + 1) << '\n';
    }
    return 0;
}
/*

6 1 0
2 1 3 4 5 6
6
2 1 3
2 1 2
1 1 4
2 1 5
2 1 6
1 2 6

6 2 0
1 2 3 4 5 6
6
2 1 5
2 2 5
2 1 6
2 2 6
2 1 4
2 2 4

10 2 0
2 1 4 3 5 6 8 7 10 9
10
2 1 5
1 2 5
2 3 5
2 1 9
2 5 8
2 3 9
2 1 8
1 1 2
2 3 8
2 1 9

 */