#define _CRT_SECURE_NO_WARNINGS

#include <iostream>
#include <iomanip>
#include <algorithm>
#include <cmath>
#include <vector>
#include <string>
#include <random>
#include <set>

using namespace std;
using ll = long long;
using ld = long double;
mt19937 rd(2346927423);
const ll INF = 2e18;
const int SZ = 1e6 + 1;

struct DDnode {
    DDnode* l;
    DDnode* r;
    ll val, prior;
    DDnode(ll val = 0) : val(val) {
        l = nullptr;
        r = nullptr;
        prior = rd();
    }
};
DDnode* mer(DDnode* l, DDnode* r) {
    if (l == nullptr)
        return r;
    if (r == nullptr)
        return l;

    if (l->prior > r->prior) {
        l->r = mer(l->r, r);
        return l;
    }
    else {
        r->l = mer(l, r->l);
        return l;
    }
}
pair<DDnode*, DDnode*> split(DDnode* v, ll x) {
    return { v, v };
}

struct pt {
    ll x, y;
    pt(ll x = 0, ll y = 0) : x(x), y(y) {}
};
pt operator+(pt a, pt b) {
    return pt(a.x + b.x, a.y + b.y);
}
pt operator-(pt a, pt b) {
    return pt(a.x - b.x, a.y - b.y);
}
ll operator*(pt a, pt b) {
    return a.x * b.x + a.y * b.y;
}
ll operator%(pt a, pt b) {
    return a.x * b.y - a.y * b.x;
}

vector<vector<int>> g, rg;

struct tr {
    ll a;
    int v, mx;
    tr(ll a = 0, int v = 0, int mx = 0) : a(a), v(v), mx(mx) {}
};

int main()
{
#ifdef _DEBUG
    freopen("input.txt", "r", stdin);
#endif
    ios_base::sync_with_stdio(false);
    cin.tie(0); cout.tie(0);
    
    int n, d, G;
    cin >> n >> d >> G;
    vector<int> a(n);
    for (int i = 0; i < n; i++)
        cin >> a[i];

    vector<tr> vec;
    vector<int> pos(n, -1);
    g.assign(n, {});
    rg.assign(n, {});

    for (int i = n - 1; i >= 0; i--) {
        int p = i;
        for (int j = i + 1; j < n; j++) {
            if (j - p > d)
                break;
            if (a[j] > a[i]) {
                g[i].push_back(j);
                rg[j].push_back(i);
            }
            else {
                p = j;
            }
        }
        vec.push_back({ a[i], i });
    }

    vector<vector<ll>> dp(n, vector<ll>(n, INF));

    for (int i = 0; i < n; i++) {
        dp[i][i] = 0;
        for (int j = i + 1; j < n; j++) {
            for (auto u : rg[j]) {
                dp[i][j] = min(dp[i][j], dp[i][u] + 1);
            }
        }
    }

    int t;
    cin >> t;
    vector<ll> ans(t);
    for (int q = 0; q < t; q++) {
        int ty, u, v;
        cin >> ty >> u >> v;
        u--; v--;

        if (dp[u][v] >= INF) {
            dp[u][v] = 0;
        }
        else {
            if (ty == 1) {
                ans[q] = 1;
            }
            else {
                ans[q] = dp[u][v];
            }
        }
    }

    for (auto x : ans) {
        cout << x << '\n';
    }
}

/*vector<int> pr(SZ), sz(SZ);
set<int> g1[SZ], g2[SZ];
vector<pair<int, int>> ans;

int get_root(int v) {
    if (pr[v] == v)
        return v;
    return pr[v] = get_root(pr[v]);
}

bool mer(int u, int v) {
    u = get_root(u);
    v = get_root(v);
    if (u == v)
        return 0;
    if (sz[u] < sz[v]) {
        pr[u] = v;
        sz[v] += sz[u];
    }
    else {
        pr[v] = u;
        sz[u] += sz[v];
    }
    return 1;
}

vector<bool> used(SZ, 0);
vector<bool> col(SZ, 0);
vector<int> rnk(SZ, 0);
bool dfs(int v = 1, int p = 1) {
    used[v] = 1;
    if (p != 1 && get_root(1) != get_root(v)) {
        //f1[1][v] = !f1[1][v];
        //f1[v][1] = !f1[v][1];
        //f2[1][v] = !f2[1][v];
        //f2[v][1] = !f2[v][1];
        ans.push_back({ 1, v });
        mer(1, v);
    }

    for (auto u : g2[v]) {
        if (used[u])
            continue;
        dfs(u, v);
    }

    if (v == 1) {
        int vert = -1;
        for (auto u : g2[v]) {
            if (vert == -1 || get_root(u) == get_root(vert)) {
                vert = u;
            }
            else {
                //f1[u][vert] = !f1[u][vert];
                //f1[vert][u] = !f1[vert][u];
                //f2[u][vert] = !f2[u][vert];
                //f2[vert][u] = !f2[vert][u];
                ans.push_back({ vert, u });
                mer(u, vert);
            }
        }
    }
    return 1;
}

int get_most(int v = 1, int p = 0) {
    rnk[v] = rnk[p] + 1;
    int up1 = rnk[v];
    for (auto u : g2[v]) {
        if (used[u] && u != p) {
            up1 = min(up1, rnk[u]);
            continue;
        }
        up1 = min(up1, get_most(u, v));
    }
    if (up1 < rnk[v] && col[p] != col[v]) {
        ans.push_back(v);
    }
}*/