#include <iostream>
#include <bitset>
#include <set>
#include <vector>
#include <algorithm>
using namespace std;
using ll = long long;

const int MAXN = 3e5 + 123;
int a[MAXN];
int edg_terminal[MAXN];

struct Query {
    int t;
    int l;
    int r;
    int qi;

    bool operator<(const Query& q) const {
        return a[r] < a[q.r];
    }
};

void init_terminal(int n, int d) {
    vector<int> ra(n);
    for (int i = 0; i < n; ++i)
        ra[a[i]] = i;
    set<int> terminal_vertecies;
    bitset<MAXN> active;
    for (int i = 0; i < n; ++i)
        terminal_vertecies.insert(i);
    for (int rai = 0; rai < n; ++rai) {
        int i = ra[rai];
        active[i] = true;
        while (true) {
            auto it = terminal_vertecies.lower_bound(i - d);
            if (it == terminal_vertecies.end() || *it >= i)
                break;
            //cout << "ERASE " << *it << endl;
            terminal_vertecies.erase(it);
        }
        //edg_terminal[i] = 
        auto it = terminal_vertecies.lower_bound(i);
        if (it == terminal_vertecies.end())
            edg_terminal[i] = n;
        else
            edg_terminal[i] = *it + d;
    }
}

struct stree {
    int size;
    vector<int> t;
    
    void resize(int n) {
        size = n;
        t.assign(2 * n - 1, -1);
    }

    void set(int ind, int lnd, int rnd, int i, int x) {
        if (rnd - lnd == 1) {
            t[ind] = x;
            return;
        }
        int mnd = (lnd + rnd) / 2;
        int chd = ind + (mnd - lnd) * 2;
        if (i < mnd)
            set(ind + 1, lnd, mnd, i, x);
        else
            set(chd, mnd, rnd, i, x);
        t[ind] = max(t[ind + 1], t[chd]);
    }

    int get(int ind, int lnd, int rnd, int l, int r) {
        if (l <= lnd && r <= rnd)
            return t[ind];
        if (l >= rnd || lnd >= r)
            return -1;
        int mnd = (lnd + rnd) / 2;
        int chd = ind + (mnd - lnd) * 2;
        return max(get(ind + 1, lnd, mnd, l, r), get(chd, mnd, rnd, l, r));
    }
};

signed main() {
    ios_base::sync_with_stdio(false);
    cin.tie(nullptr);
    cout.tie(nullptr);
    int n, d, g;
    cin >> n >> d >> g;
    for (int i = 0; i < n; ++i) {
        cin >> a[i];
        --a[i];
    }
    vector<int> ra(n);
    for (int i = 0; i < n; ++i)
        ra[a[i]] = i;
    init_terminal(n, d);

    int q;
    cin >> q;
    vector<Query> qrs(q);
    for (int qi = 0; qi < q; ++qi) {
        cin >> qrs[qi].t >> qrs[qi].l >> qrs[qi].r;
        --qrs[qi].l;
        --qrs[qi].r;
        qrs[qi].qi = qi;
    }
    sort(qrs.begin(), qrs.end());

    stree tr;
    tr.resize(n);
    set<int> terminal_vertecies;
    bitset<MAXN> active;
    for (int i = 0; i < n; ++i)
        terminal_vertecies.insert(i);
    vector<int> qrans(q);
    int rai = 0;

    for (auto qr : qrs) {
        if (a[qr.l] >= a[qr.r] || qr.l >= qr.r) {
            qrans[qr.qi] = 0;
            continue;
        }
        while (rai < n && rai <= a[qr.r]) {
            int i = ra[rai];
            tr.set(0, 0, n, i, rai);
            active[i] = true;
            while (true) {
                auto it = terminal_vertecies.lower_bound(i - d);
                if (it == terminal_vertecies.end() || *it >= i)
                    break;
                //cout << "ERASE " << *it << endl;
                terminal_vertecies.erase(it);
            }
            ++rai;
        }
        auto lit = terminal_vertecies.lower_bound(qr.l);
        if (lit != terminal_vertecies.end() && *lit < qr.r - d) {
            qrans[qr.qi] = 0;
            continue;
        }
        if (qr.t == 1) {
            qrans[qr.qi] = 1;
            continue;
        }

        int movi = qr.l;
        while (qr.l < qr.r) {
            if (edg_terminal[qr.l] >= qr.r) {
                ++qrans[qr.qi];
                break;
            }
            int besti = ra[tr.get(0, 0, n, qr.l, edg_terminal[qr.l] + 1)];
            if (besti == qr.l) {
                qrans[qr.qi] = 0;
                break;
            }
            ++qrans[qr.qi];
            qr.l = besti;
        }
        if (qr.t == 1 && qrans[qr.qi] > 0) {
            qrans[qr.qi] = 1;
        }
        //cout << qr.qi << ": " << qr.l << ' ' << qr.r << " | ";
        //for (auto x : terminal_vertecies)
            //cout << x << ' ';
        //for (int i = 0; i < n; ++i)
            //cout << active[i];
        //cout << endl;
    }
    for (int i = 0; i < q; ++i) {
        cout << qrans[i] << '\n';
    }
}
