#include <iostream>
#include <bitset>
#include <set>
#include <vector>
#include <algorithm>
using namespace std;
using ll = long long;

const int MAXN = 3e5 + 123;
int a[MAXN];
int edg_terminal[MAXN];
int ra[MAXN];

struct Query {
    int t;
    int l;
    int r;
    int qi;

    bool operator<(const Query& q) const {
        return a[r] < a[q.r];
    }
};

void init_terminal(int n, int d) {
    for (int i = 0; i < n; ++i)
        ra[a[i]] = i;
    set<int> terminal_vertecies;
    bitset<MAXN> active;
    for (int i = 0; i < n; ++i)
        terminal_vertecies.insert(i);
    for (int rai = 0; rai < n; ++rai) {
        int i = ra[rai];
        active[i] = true;
        while (true) {
            auto it = terminal_vertecies.lower_bound(i - d);
            if (it == terminal_vertecies.end() || *it >= i)
                break;
            //cout << "ERASE " << *it << endl;
            terminal_vertecies.erase(it);
        }
        //edg_terminal[i] = 
        auto it = terminal_vertecies.lower_bound(i);
        if (it == terminal_vertecies.end())
            edg_terminal[i] = n;
        else
            edg_terminal[i] = *it + d;
    }
}

struct stree {
    int size;
    vector<int> t;
    
    void resize(int n) {
        size = n;
        t.assign(2 * n - 1, -1);
    }

    void set(int ind, int lnd, int rnd, int i, int x) {
        if (rnd - lnd == 1) {
            t[ind] = x;
            return;
        }
        int mnd = (lnd + rnd) / 2;
        int chd = ind + (mnd - lnd) * 2;
        if (i < mnd)
            set(ind + 1, lnd, mnd, i, x);
        else
            set(chd, mnd, rnd, i, x);
        t[ind] = max(t[ind + 1], t[chd]);
    }

    int get(int ind, int lnd, int rnd, int l, int r) {
        if (l <= lnd && rnd <= r)
            return t[ind];
        if (l >= rnd || lnd >= r)
            return -1;
        int mnd = (lnd + rnd) / 2;
        int chd = ind + (mnd - lnd) * 2;
        return max(get(ind + 1, lnd, mnd, l, r), get(chd, mnd, rnd, l, r));
    }

    void set(int i, int x) {
        set(0, 0, size, i, x);
    }

    int get(int l, int r) {
        return get(0, 0, size, l, r);
    }
};

stree tr;
int depth[MAXN];

const int SQN = 400;

struct sqrt_decomp {
    int size;
    bitset<MAXN> active;
    vector<pair<int, int>> last_inside_block;

    void resize(int n) {
        size = n;
        last_inside_block.resize(n);
        for (int i = 0; i < n; ++i)
            last_inside_block[i] = {i, 0};
    }

    
};


signed main() {
    ios_base::sync_with_stdio(false);
    cin.tie(nullptr);
    cout.tie(nullptr);
    int n, d, g;
    cin >> n >> d >> g;
    for (int i = 0; i < n; ++i) {
        cin >> a[i];
        --a[i];
    }
    init_terminal(n, d);

    for (int i = 0; i < n; ++i)
        last_edge[i] = i;

    int q;
    cin >> q;
    vector<Query> qrs(q);
    for (int qi = 0; qi < q; ++qi) {
        cin >> qrs[qi].t >> qrs[qi].l >> qrs[qi].r;
        --qrs[qi].l;
        --qrs[qi].r;
        qrs[qi].qi = qi;
    }
    sort(qrs.begin(), qrs.end());

    tr.resize(n);
    set<int> terminal_vertecies;
    bitset<MAXN> active;
    for (int i = 0; i < n; ++i)
        terminal_vertecies.insert(i);
    vector<int> qrans(q);
    int rai = 0;

    for (auto qr : qrs) {
        if (a[qr.l] >= a[qr.r] || qr.l >= qr.r) {
            qrans[qr.qi] = 0;
            continue;
        }
        while (rai < n && rai <= a[qr.r]) {
            int i = ra[rai];
            tr.set(i, rai);
            active[i] = true;
            while (true) {
                auto it = terminal_vertecies.lower_bound(i - d);
                if (it == terminal_vertecies.end() || *it >= i)
                    break;
                //cout << "ERASE " << *it << endl;
                terminal_vertecies.erase(it);
            }
            ++rai;
        }

        auto lit = terminal_vertecies.lower_bound(qr.l);
        if (lit != terminal_vertecies.end() && *lit < qr.r - d) {
            qrans[qr.qi] = 0;
            continue;
        }
        if (qr.t == 1) {
            qrans[qr.qi] = 1;
            continue;
        }

        if (g == 3 || g == 5) {
            g = -1;
            for (int v = n - 2; v >= 0; --v) if (a[v] < a[n - 1]) {
                int u = ra[tr.get(v, edg_terminal[v] + 1)];
                depth[v] = depth[u] + 1;
            }
        }
        if (g == -1) {
            qrans[qr.qi] = depth[qr.l];
            continue;
        }

        while (qr.l < qr.r) {
            if (edg_terminal[qr.l] >= qr.r) {
                ++qrans[qr.qi];
                break;
            }
            qr.l = ra[tr.get(qr.l, edg_terminal[qr.l] + 1)];
            ++qrans[qr.qi];
        }
    }
    for (int i = 0; i < q; ++i) {
        cout << qrans[i] << '\n';
    }
}
