#include <iostream>
#include <vector>
#include <map>
#include <set>
#include <iomanip>
#include <string>
#include <algorithm>
#include <deque>
#include <queue>
#include <math.h>
#include <fstream>
#include <random>
#include <cassert>
#include <hash_map>
#include <unordered_map>
#include <unordered_set>

using namespace std;

using ll = long long;
using ld = long double;

#pragma GCC optimize("03")
#pragma GCC target("avx2")

#define vec vector
#define all(x) x.begin(), x.end()
#define rall(x) x.rbegin(), x.rend()
#define pb push_back
//#define int long long

random_device rd;
mt19937 rnd(rd());

const int INF = 2e18;

struct Fenwick{
    vec<int> t;
    int n = 0;

    void init(int n_) {
        n = n_;
        t.assign(n, 0);
    }

    void upd(int a, int x) {
        for (; a < n; a += a & -a) {
            t[a] += x;
        }
    }

    int get(int r) {
        int s = 0;
        for (; r > 0; r -= r & -r) {
            s += t[r];
        }
        return s;
    }

    int get(int l, int r) {
        return get(r) - get(l - 1);
    }
};

struct Vector{
    int x, y;

    Vector(int x_ = 0, int y_ = 0) {
        x = x_, y = y_;
    }

    friend Vector operator+(Vector& a, Vector& b) {
        return (*new Vector(a.x + b.x, a.y + b.y));
    }

    friend Vector operator-(Vector& a, Vector& b) {
        return (*new Vector(a.x - b.x, a.y - b.y));
    }

    friend int operator*(Vector& a, Vector& b) {
        return a.x * b.x + a.y * b.y;
    }

    friend int operator^(Vector& a, Vector& b) {
        return a.x * b.y - a.y * b.x;
    }

    ld len() {
        return x * x + y + y;
    }

    ld len2() {
        return sqrt(x * x + y * y);
    }
};

vec<int> p(3e5), s(3e5);
vec<vec<int>> sv(3e5);

void make_set(int v) {
    p[v] = v;
    s[v] = 1;
    sv[v] = {v};
}

int find_set(int v) {
    if (v == p[v]) return v;
    return p[v] = find_set(p[v]);
}

void union_sets(int a, int b) {
    a = find_set(a);
    b = find_set(b);
    if (a != b) {
        if (s[a] < s[b]) swap(a, b);
        p[b] = a;
        s[a] += s[b];
        for (int& i : sv[b]) {
            sv[a].push_back(b);
        }
    }
}

vec<vec<int>> g;
vec<vec<int>> dist;
int n, d, gg;

void bfs(int v) {
    dist[v][v] = 0;
    deque<int> q;
    q.push_back(v);
    vec<bool> used(n);
    while ((int)q.size() > 0) {
        int v2 = q.front();
        q.pop_front();
        for (int& u : g[v2]) {
            if (!used[u]) {
                used[u] = 1;
                q.push_back(u);
                dist[v][u] = dist[v][v2] + 1;
            }
        }
    }
}

void solve() {
    cin >> n >> d >> gg;
    vec<int> a(n);
    g.assign(n, vec<int>());
    dist.assign(n, vec<int>(n, INF));
    for (int i = 0; i < n; ++i) {
        cin >> a[i];
    }
    for (int i = 0; i < n; ++i) {
        int pred = i;
        for (int j = i + 1; j < n; ++j) {
            if (j - pred > d) break;
            if (a[j] < a[i]) {
                pred = j;
            }
            else {
                g[i].push_back(j);
            }
        }
    }
    for (int i = 0; i < n; ++i) {
        bfs(i);
    }
    int q; cin >> q;
    for (int i = 0; i < q; ++i) {
        int type, v, u;
        cin >> type >> v >> u;
        v--, u--;
        if (dist[v][u] != INF) {
            if (type == 1) cout << 1 << endl;
            else cout << dist[v][u] << endl;
        }
        else {
            cout << 0 << endl;
        }
    }
}

signed main() {
    //freopen("C:\\Users\\olymp\\CLionProjects\\untitled\\input.txt", "r", stdin);
    //freopen("C:\\Users\\olymp\\CLionProjects\\untitled\\output.txt", "w", stdout);
    ios_base::sync_with_stdio(0);
    cin.tie(0);
    int t = 1;
    //cin >> t >> g;
    for (; t > 0; --t) {
        solve();
    }
}