#include <bits/stdc++.h>

using namespace std;

mt19937 rnd(57);

struct Node {
    pair<int, int> key;
    int prior;
    int sz;
    int vl, cnt;
    bool push;
    Node *l, *r;

    Node(pair<int, int> _key) {
        key = _key;
        prior = rnd();
        sz = 1;
        l = r = nullptr;
        vl = cnt = 0;
        push = false;
    }
};

int sz(Node *root) {
    if (root == nullptr) {
        return 0;
    }
    return root->sz;
}

int cnt(Node *root) {
    if (root == nullptr) {
        return 0;
    }
    return root->cnt;
}

void relax(Node *root) {
    root->sz = sz(root->l) + 1 + sz(root->r);
    root->cnt = cnt(root->l) + root->vl + cnt(root->r);
}

void push(Node *root) {
    if (root->push) {
        if (root->l) {
            root->l->vl = 1;
            root->l->push = true;
            relax(root->l);
        }
        if (root->r) {
            root->r->vl = 1;
            root->r->push = true;
            relax(root->r);
        }
    }
    root->push = false;
}

Node *merge(Node *root1, Node *root2) {
    if (root1 == nullptr) {
        return root2;
    }
    if (root2 == nullptr) {
        return root1;
    }
    push(root1);
    push(root2);
    if (root1->prior > root2->prior) {
        root1->r = merge(root1->r, root2);
        relax(root1);
        return root1;
    } else {
        root2->l = merge(root1, root2->l);
        relax(root2);
        return root2;
    }
}

pair<Node*, Node*> split_val(Node *root, pair<int, int> vl) {
    if (root == nullptr) {
        return {nullptr, nullptr};
    }
    push(root);
    if (root->key >= vl) {
        auto [res1, res2] = split_val(root->l, vl);
        root->l = res2;
        relax(root);
        return {res1, root};
    } else {
        auto [res1, res2] = split_val(root->r, vl);
        root->r = res1;
        relax(root);
        return {root, res2};
    }
}

pair<Node*, Node*> split_sz(Node *root, int sz_l) {
    if (root == nullptr) {
        return {nullptr, nullptr};
    }
    push(root);
    if (sz(root->l) >= sz_l) {
        auto [res1, res2] = split_sz(root->l, sz_l);
        root->l = res2;
        relax(root);
        return {res1, root};
    } else {
        auto [res1, res2] = split_sz(root->r, sz_l - sz(root->l) - 1);
        root->r = res1;
        relax(root);
        return {root, res2};
    }
}

int get_pos(Node *root, pair<int, int> val) {
    auto [res1, res2] = split_val(root, val);
    int ans = sz(res1);
    merge(res1, res2);
    return ans;
}

pair<int, int> get_val(Node *root, int pos) {
    auto [res1, res2] = split_sz(root, pos);
    auto [res3, res4] = split_sz(res2, 1);
    pair<int, int> ans = res3->key;
    merge(res1, merge(res3, res4));
    return ans;
}

int get_len(Node *root, int pos) {
    int l = 0, r = pos + 1;
    while (l < r - 1) {
        int m = (l + r) / 2;
        auto [res1, res2] = split_sz(root, pos);
        auto [res3, res4] = split_sz(res1, pos - m);
        if (cnt(res4) == sz(res4)) {
            l = m;
        } else {
            r = m;
        }
        merge(merge(res3, res4), res2);
    }
    return l;
}

int get_len_r(Node *root, int pos) {
    int l = 0, r = sz(root) - pos + 1;
    while (l < r - 1) {
        int m = (l + r) / 2;
        auto [res1, res2] = split_sz(root, pos);
        auto [res3, res4] = split_sz(res2, m);
        if (cnt(res3) == sz(res3)) {
            l = m;
        } else {
            r = m;
        }
        merge(res1, merge(res3, res4));
    }
    return l;
}

void assign(Node *root, int l, int r) {
    auto [res1, res2] = split_sz(root, r);
    auto [res3, res4] = split_sz(res1, l);
    if (res4) {
        res4->vl = 1;
        res4->push = 1;
        relax(res4);
    }
    merge(merge(res3, res4), res2);
}

void solve() {
    int n;
    cin >> n;
    vector<int> a(n), b(n);
    for (int i = 0; i < n; ++i) {
        cin >> a[i];
    }
    for (int i = 0; i < n; ++i) {
        cin >> b[i];
    }
    Node *root_a = nullptr, *root_b = nullptr;
    long long ans = 0, sum = 0;
    for (int i = 0; i < n; ++i) {
        // long long ans = 0;
        // vector<int> a_nw, b_nw;
        // for (int i = 0; i < pr; ++i) {
        //     ans += a[i];
        //     ans -= b[i];
        //     a_nw.push_back(a[i]);
        //     b_nw.push_back(b[i]);
        // }
        ans += a[i] - b[i];
        sum += a[i] - b[i];
        int pos_a = get_pos(root_a, {a[i], -i}), pos_b = get_pos(root_b, {b[i], i});
        auto [res1, res2] = split_sz(root_a, pos_a);
        auto nw_a = new Node({a[i], -i});
        root_a = merge(res1, merge(nw_a, res2));

        auto [res3, res4] = split_sz(root_b, pos_b);
        auto nw_b = new Node({b[i], i});
        root_b = merge(res3, merge(nw_b, res4));

        if (pos_a == pos_b) {
            for (int j = max(0, pos_a - 1); j <= pos_a && j < i; ++j) {
                if (get_val(root_a, j).first >= get_val(root_b, j + 1).first) {
                    assign(root_a, j, j + 1);
                }
            }
            for (int j = max(0, pos_a - 1); j <= pos_a; ++j) {
                int len = get_len(root_a, j);
                ans = min(ans, sum - (get_val(root_a, j).first - get_val(root_b, j - len).first));
            }
            int pos = pos_a + get_len_r(root_a, pos_a);
            if (pos == i + 1) {
                pos = i;
            }
            int len = get_len(root_a, pos);
            ans = min(ans, sum - (get_val(root_a, pos).first - get_val(root_b, pos - len).first));
        } else if (pos_b < pos_a) {
            assign(root_a, pos_b, pos_a);
            for (int j = max(pos_b - 1, 0); j <= pos_b && j < i; ++j) {
                if (get_val(root_a, j).first >= get_val(root_b, j + 1).first) {
                    assign(root_a, j, j + 1);
                }
            }
            for (int j = max(0, pos_a - 1); j <= pos_a && j < i; ++j) {
                if (get_val(root_a, j).first >= get_val(root_b, j + 1).first) {
                    assign(root_a, j, j + 1);
                }
            }

            for (int j = max(pos_b - 1, 0); j <= pos_b + 1 && j <= i; ++j) {
                int len = get_len(root_a, j);
                ans = min(ans, sum - (get_val(root_a, j).first - get_val(root_b, j - len).first));
            }
            for (int j = max(0, pos_a - 1); j <= pos_a + 1 && j <= i; ++j) {
                int len = get_len(root_a, j);
                ans = min(ans, sum - (get_val(root_a, j).first - get_val(root_b, j - len).first));
            }

            int pos = pos_a + get_len_r(root_a, pos_a);
            if (pos == i + 1) {
                pos = i;
            }
            int len = get_len(root_a, pos);
            ans = min(ans, sum - (get_val(root_a, pos).first - get_val(root_b, pos - len).first));

            if (pos_b + 1 <= i ) {
                pos = pos_b + 1 + get_len_r(root_a, pos_b + 1);
                if (pos == i + 1) {
                    pos = i;
                }
                len = get_len(root_a, pos);
                ans = min(ans, sum - (get_val(root_a, pos).first - get_val(root_b, pos - len).first));
            }

 
            for (int j = pos_a; j <= i; ++j) {
                int len = get_len(root_a, j);
                ans = min(ans, sum - (get_val(root_a, j).first - get_val(root_b, j - len).first));
            }
        } else {
            for (int j = 0; j < i; ++j) {
                if (get_val(root_a, j).first >= get_val(root_b, j + 1).first) {
                    assign(root_a, j, j + 1);
                }
            }
            for (int j = 0; j <= i; ++j) {
                int len = get_len(root_a, j);
                ans = min(ans, sum - (get_val(root_a, j).first - get_val(root_b, j - len).first));
            }
        }
        // for (int pos = 0; pos < pr; ++pos) {
        //     long long ans_nw = 0;
        //     int p = pr - 1;
        //     for (int i = pr - 1; i >= 0; --i) {
        //         if (i == pos) {
        //             continue;
        //         }
        //         while (a_nw[i] < b_nw[p]) {
        //             p--;
        //         }
        //         ans_nw += a_nw[i] - b_nw[p];
        //         p--;
        //     }
        //     ans = min(ans, ans_nw);
        // }
        cout << ans << ' ';
    }
    cout << '\n';
}

signed main() {
    ios_base::sync_with_stdio(false);
    cin.tie(nullptr);
    int t = 1;
    cin >> t;
    while (t--) {
        solve();
    }
    return 0;
}
