#include <iostream>
#include <vector>
#include <string>
#include <algorithm>
#include <set>
#include <map>
#include <iomanip>
#include <cmath>
#include <cassert>
#include <random>

using namespace std;

#define F first
#define S second
#define pb push_back
#define mp make_pair
#define all(x) x.begin(), x.end()
#define rall(x) x.rbegin(), x.rend()
typedef long long ll;

struct Segment_Tree {
    int n; 
    vector<int> a, t, md;

    void build(int p, int l, int r) {
        if (l + 1 == r) {
            t[p] = a[l];
            return;
        }
        int m = (l + r) / 2;
        build(2 * p + 1, l, m);
        build(2 * p + 2, m, r);
        t[p] = t[2 * p + 1] + t[2 * p + 2];
    }

    Segment_Tree(vector<int> a) : a(a), n(a.size()) {
        t.resize(4 * n);
        md.resize(4 * n);
        build(0, 0, n);
    }

    void push(int p, int l, int r) {
        if (l + 1 == r) return;
        int m = (l + r) / 2;
        t[2 * p + 1] += (m - l) * md[p];
        t[2 * p + 2] += (r - m) * md[p];
        md[2 * p + 1] += md[p];
        md[2 * p + 2] += md[p];
        md[p] = 0;
    }

    void update(int p, int l, int r, int ql, int qr, int val) {
        if (r <= ql || qr <= l) return;
        push(p, l, r);
        if (ql <= l && r <= qr) {
            t[p] += (r - l) * val;
            md[p] += val;
            return;
        }   
        int m = (l + r) / 2;
        update(2 * p + 1, l, m, ql, qr, val);
        update(2 * p + 2, m, r, ql, qr, val);
        t[p] = t[2 * p + 1] + t[2 * p + 2];
    }   
    
    int get(int p, int l, int r, int ql, int qr) {
        if (r <= ql || qr <= l) return 0;
        push(p, l, r);
        if (ql <= l && r <= qr) {
            return t[p];
        }   
        int m = (l + r) / 2;
        return get(2 * p + 1, l, m, ql, qr) + get(2 * p + 2, m, r, ql, qr);
    }
};

int main() {
    ios_base::sync_with_stdio(0);
    cin.tie(0);
    int tt; cin >> tt;
    while (tt--) {
        int n; cin >> n;
        vector<ll> a(n), b(n);
        for (int i = 0; i < n; i++) cin >> a[i];
        for (int i = 0; i < n; i++) cin >> b[i];
       
        ll sumA = 0, sumB = 0, mx = 0;
        multiset<ll> A, B;
        vector<int> cnt;
        for (int i = 0; i < n; i++) {
            sumA += a[i];
            sumB += b[i];
            A.insert(a[i]);
            B.insert(b[i]);
            if (A.find(b[i]) != A.end()) {
                A.erase(A.find(b[i]));
                B.erase(B.find(b[i]));
                sumB -= b[i];
                sumA -= b[i];
            } 

            cnt.clear();
            auto it = B.begin(); 
            int id = 0;
            for (auto j : A) {
                while (it != B.end() j >= *it) it++, id++;
                cnt.pb(id);
            }

            int sz = cnt.size();
            int last = sz - 1;
            for (int j = sz - 2; j >= 0; j--) {
                if (cnt[j] < j + 2) {
                    break;
                }
                last = j;
            }

            id = 0;
            for (auto j : B) {
                if (id == last) {
                    mx = max(mx, *A.rbegin() - j);
                    break;
                }
                id++;
            }
            
            cout << sumA - sumB - mx << " ";
        }
        cout << "\n";
    }
}