#include <iostream>
#include <vector>
#include <set>
#include <queue>
#include <map>
#include <random>
#include <chrono>
#include <assert.h>
#include <stack>
#include <numeric>

#define pb push_back
#define all(x) (x).begin(), (x).end()

using namespace std;
using ll = long long;
using ii = pair<ll, ll>;
using uid = uniform_int_distribution<int>;
using lst = vector<vector<int>>;

const int type = 1;
const int MAXN = 2e5 + 1;
const int LOGN = 17;
const int MOD = 1e9 + 7;

class SegmentTree {
private:
	int n;
	int size;
	vector<int> t;

	void build(int v, int tl, int tr) {
		if (tl == tr) {
			return;
		}
		int tm = (tl + tr) >> 1;
		build(v << 1, tl, tm);
		build(v << 1 | 1, tm + 1, tr);
	}

public:

	SegmentTree(const int _n)
		:n(_n)
		, size(n << 2)
		, t(size) {
		build(1, 0, n);
	}
};

struct HashTable {
	static const int TABLE_SIZE = 1e7;
	static const int MAX_ELEMENTS = 1e5;
	int head[TABLE_SIZE];
	int next[MAX_ELEMENTS];
	int val[MAX_ELEMENTS];
	int itemCount = 0;

	void update() {
		for (int i = 0;i < MAX_ELEMENTS;i++) {
			next[i] = -1;
		}
		for (int i = 0;i < TABLE_SIZE;i++) {
			head[i] = -1;
		}
		return;
	}

	HashTable() {
		update();
	}

	void add(ll hash) {
		ll h = hash % TABLE_SIZE;
		for (int i = head[h]; i != -1;i = next[i]) {
			if (val[i] == hash) {
				return;
			}
		}
		next[++itemCount] = head[h];
		head[h] = itemCount;
		val[itemCount] = hash;
		return;
	}
};

bool isPrime(ll x) {
	for (int i = 2;i <= x / i;i++) {
		if (x % i == 0) {
			return false;
		}
 	}
	return (x > 1);
}

void genPrime(mt19937& gen, ll& x) {
	while (1) {
		x = uid(1e9, 2e9)(gen);
		if (isPrime(x)) return;
	}
}

int _lg(ll x) {
	int ans = 0;
	for (int i = 0;i < 63;i++) {
		if (x & (1 << i)) ans = i;
	}
	return ans;
}

int _builtin_popcount(ll x) {
	int ans = 0;
	for (int i = 0;i < 63;i++) {
		if (x & (1 << i)) ans++;
	}
	return ans;
}

void solve2() {

}

int getRoot(int v, vector<int>& p) {
	int v0 = v;
	while (p[v] >= 0) {
		v = p[v];
	}
	int root = v;
	v = v0;
	while (p[v] >= 0) {
		int par = p[v];
		p[v] = root;
		v = par;
	}
	return root;
}

bool join(int a, int b, vector<int>& p) {
	a = getRoot(a, p), b = getRoot(b, p);
	if (a == b) return false;
	if (p[a] > p[b]) swap(a, b);
	p[a] += p[b];
	p[b] = a;
	return true;
}

bool tryjoin(int a, int b, vector<int>& p) {  // можно ли соединить?
	a = getRoot(a, p), b = getRoot(b, p);
	if (a == b) return false;
	return true;
}

int tin[MAXN], up[MAXN];

void findBridges(int v, lst& adj, set<ii>& bridges, int t = 0, int p = -1) {
	tin[v] = up[v] = ++t;
	for (int u : adj[v]) {
		if (u == p) continue;
		if (tin[u] == 0) findBridges(u, adj, bridges, t, v);
		if (tin[u] > tin[v]) up[v] = min(up[u], up[v]);
		if (tin[u] < tin[v]) up[v] = min(up[v], tin[u]);
	}
	if (p != -1 && tin[v] == up[v]) {
		bridges.insert({ min(p, v), max(p,v) });
	}
}

void colorComp(int v, lst& adj, vector<int>& clr, int color) {
	clr[v] = color;
	for (int u : adj[v]) {
		if (clr[u] == -1) colorComp(u, adj, clr, color);
	}
	return;
}

void solve(int group) {
	int n;
	cin >> n;
	vector<ll> a(n), b(n);
	for (ll& v : a) {
		cin >> v;
	}
	for (ll& v : b) {
		cin >> v;
	}
	if (n <= 3000) {
		multiset<ll> an, bn;
		cout << 0 << " ";
		if (n == 1) {
			return;
		}
		if (a[0] != b[0]) {
			an.insert(a[0]);
			bn.insert(b[0]);
		}
		for (int i = 1; i < n;i++) {
			an.insert(a[i]);
			bn.insert(b[i]);  // возможно в an присутствует элемент bn, который можно удалить
			if (bn.find(a[i]) != bn.end()) {
				an.extract(a[i]);
				bn.extract(a[i]);
			}
			if (an.find(b[i]) != an.end()) {
				an.extract(b[i]);
				bn.extract(b[i]);  // лишние элементы необходимо всего удалять
			}  // теперь на этом мультисете нужно найти норм префикс
			vector<ll> aN, bN;
			for (auto it = an.begin(); it != an.end();it++) {
				aN.pb(*it);
			}
			for (auto it = bn.begin(); it != bn.end();it++) {
				bN.pb(*it);
			}
			// теперь можно искать префикс
			int nn = aN.size();
			ll ans = 0;
			int ind = -1;
			for (int i = 0;i + 1 < nn;i++) {
				if (aN[i] > bN[i + 1]) {
					ans += aN[i] - bN[i + 1];
					ind = i;
				}
				else break;
			}  // куда-то мы смогли уйти, то значит, мы уже поплюсовали всё, что между 0
			if (ind == -1) {  // мы можем потратить максимум с умом
				ll mx = 0;
				for (int i = 0;i < nn;i++) {
					ans += a[i] - b[i];
					mx = max(mx, a[i] - b[i]);
				}
				cout << ans - mx << " ";
				continue;
			}
			for (int i = ind + 2; i < nn;i++) {
				ans += aN[i] - bN[i];
			}
			cout << ans << " ";
			continue;
		}
		return;
	}
	ll nmb = 0;
	ll mx = 0;
	for (int i = 0;i < n;i++) {
		mx = max(mx, a[i] - b[i]);
		nmb += a[i] - b[i];
		cout << nmb - mx << " ";
	}
	return;
}

int main() {
	cin.tie(0)->ios::sync_with_stdio(false);
	//freopen("input.txt", "r", stdin);
	//freopen("output.txt", "w", stdout);
	int nT = 1;
	if (type) cin >> nT;
	for (int t = 0;t < nT;t++) {
		solve(t);
		if (t != nT - 1) std::cout << "\n";
	}
	return 0;
}