#include <iostream>
#include <vector>
#include <set>

using namespace std;

typedef long long ll;

struct Dsu {
    int n;
    vector<int> par, rank;
    vector<int> mn, mx;

    Dsu(int n) : n(n), rank(n), mn(n), mx(n) {
        par.assign(n, 0);
        for (int i = 0; i < n; ++i) {
            par[i] = i;
        }
    }

    int get_par(int v) {
        if (par[v] == v) {
            return v;
        }
        return par[v] = get_par(par[v]);
    }

    void unite(int a, int b) {
        a = get_par(a);
        b = get_par(b);
        if (a == b) {
            return;
        }
        if (rank[a] < rank[b]) {
            swap(a, b);
        }
        par[b] = a;
        rank[a] += (rank[a] == rank[b]);
        mn[a] = min(mn[a], mn[b]);
        mx[a] = max(mx[a], mx[b]);
    }

    int get_dif(int v) {
        v = get_par(v);
        return mx[v] - mn[v];
    }
};

struct Node {
    int pos, type, id;

    Node(int pos, int type, int id) : pos(pos), type(type), id(id) {}

    bool operator<(const Node &node) const {
        return pos < node.pos || (pos == node.pos && (type < node.type || (type == node.type && id < node.id)));
    }
};

void solve() {
    int n;
    cin >> n;
    vector<int> a(n);
    for (int &i : a) {
        cin >> i;
    }
    vector<int> b(n);
    for (int &i : b) {
        cin >> i;
    }
    Dsu d(n);
    for (int i = 0; i < n; ++i) {
        d.mx[i] = a[i];
        d.mn[i] = b[i];
    }

    set<Node> s;
    ll sum_a = 0, sum_b = 0;
    int mx_dist = 0;
    for (int i = 0; i < n; ++i) {
        sum_a += a[i];
        sum_b += b[i];
        s.insert({a[i], 1, i});
        s.insert({b[i], 0, i});
        {
            auto it = s.find({a[i], 1, i});
            it--;
            d.unite(i, it->id);
            it++;
            it++;
            if (it != s.end()) {
                if (it->type == 1) {
                    d.unite(i, it->id);
                }
            }
        }
        {
            auto it = s.find({b[i], 0, i});
            it++;
            //cerr << it->pos << ',' << it->type << ',' << it->id << ' ';
            d.unite(i, it->id);
            it--;
            if (it != s.begin()) {
                it--;
                if (it->type == 0) {
                    d.unite(i, it->id);
                }
            }
        }
        mx_dist = max(mx_dist, d.get_dif(i));
        //cerr << mx_dist << endl;
        cout << sum_a - sum_b - mx_dist << ' ';
    }
    //cerr << '\n';
    cout << '\n';
}

signed main() {
    ios_base::sync_with_stdio(false), cin.tie(nullptr);
#ifdef LOCAL
    freopen("input.txt", "r", stdin);
    freopen("output.txt", "w", stdout);
#endif
    int t;
    cin >> t;
    while (t--) {
        solve();
    }
}