#include <iostream>
#include <vector>
#include <set>

using namespace std;

typedef long long ll;

struct Dsu {
    ll n;
    vector<ll> par, rank;
    vector<ll> mn, mx;

    Dsu(ll n) : n(n), rank(n), mn(n), mx(n) {
        par.assign(n, 0);
        for (ll i = 0; i < n; ++i) {
            par[i] = i;
        }
    }

    ll get_par(ll v) {
        if (par[v] == v) {
            return v;
        }
        return par[v] = get_par(par[v]);
    }

    void unite(ll a, ll b) {
        a = get_par(a);
        b = get_par(b);
        if (a == b) {
            return;
        }
        if (rank[a] < rank[b]) {
            swap(a, b);
        }
        par[b] = a;
        rank[a] += (rank[a] == rank[b]);
        mn[a] = min(mn[a], mn[b]);
        mx[a] = max(mx[a], mx[b]);
    }

    ll get_dif(ll v) {
        v = get_par(v);
        return mx[v] - mn[v];
    }
};

struct Node {
    ll pos, type, id;

    Node(ll pos, ll type, ll id) : pos(pos), type(type), id(id) {}

    bool operator<(const Node &node) const {
        return pos < node.pos || (pos == node.pos && (type < node.type || (type == node.type && id < node.id)));
    }
};

void solve() {
    ll n;
    cin >> n;
    vector<ll> a(n);
    for (ll &i : a) {
        cin >> i;
    }
    vector<ll> b(n);
    for (ll &i : b) {
        cin >> i;
    }
    Dsu d(n);
    for (ll i = 0; i < n; ++i) {
        d.mx[i] = a[i];
        d.mn[i] = b[i];
    }

    set<Node> s;
    ll sum_a = 0, sum_b = 0;
    ll mx_dist = 0;
    for (ll i = 0; i < n; ++i) {
        sum_a += a[i];
        sum_b += b[i];
        s.insert({a[i], 1, i});
        s.insert({b[i], 0, i});
        auto it = s.find({a[i], 1, i});
        {
            int k = 5;
            while (k > 0 && it != s.begin()) {
                k--;
                it--;
                if (it->type == 0 && it->id == i) {
                    break;
                }
                d.unite(i, it->id);
                //cerr << "SOS1 " << it->id << endl;
            }
        }
        it = s.find({a[i], 1, i});
        {
            int k = 5;
            set<int> e;
            it++;
            while (k > 0 && it != s.end()) {
                if (it->type == 1 && e.find(it->id) == e.end()) {
                    d.unite(i, it->id);
                    //cerr << "SOS2 " << it->id << endl;
                } else {
                    e.insert(it->id);
                }
                k--;
                it++;
            }
        }
        it = s.find({b[i], 0, i});
        {
            int k = 5;
            set<int> e;
            while (k > 0 && it != s.begin()) {
                k--;
                it--;
                if (it->type == 0 && e.find(it->id) == e.end()) {
                    d.unite(i, it->id);
                    //cerr << "SOS3 " << it->id << endl;
                } else {
                    e.insert(it->id);
                }
            }
        }
        it = s.find({b[i], 0, i});
        {
            int k = 5;
            it++;
            while (k > 0 && it != s.end() && !(it->id == i && it->type == 1)) {
                d.unite(i, it->id);
                //cerr << "SOS4 " << it->id << endl;
                k--;
                it++;
            }
        }
        mx_dist = max(mx_dist, d.get_dif(i));
        //cerr << mx_dist << endl;
        cout << sum_a - sum_b - mx_dist << ' ';
        /*for (int j = 0; j < n; ++j) {
            cerr << d.get_par(j) << ' ';
        }
        cerr << endl;
        for (auto [pos, type, id] : s) {
            cerr << pos << ',' << type << ',' << id << "; ";
        }
        cerr << endl;*/
    }
    //cerr << '\n';
    cout << '\n';
}

signed main() {
    ios_base::sync_with_stdio(false), cin.tie(nullptr);
#ifdef LOCAL
    freopen("input.txt", "r", stdin);
    freopen("output.txt", "w", stdout);
#endif
    ll t;
    cin >> t;
    while (t--) {
        solve();
    }
}